;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defclass room602-door-object
  :super cascaded-link
  :slots (sensors
         joint0 ))
(defmethod room602-door-object
  (:init
   (&rest args &key (name "room602-door") (pos (float-vector 0 0 0)) (rot (unit-matrix 3)) &allow-other-keys)
   (let (c bc
         blink0 blink1
                 )
     (send-super* :init :name name args)

     ;; definition of link

     ;; definition of :room602-door-bodyset2
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -12.0 -12.0 1900.0) (float-vector -22.0 -12.0 1900.0) (float-vector -22.0 -22.0 1900.0) (float-vector -12.0 -22.0 1900.0)))
         (instance face :init :vertices (list (float-vector -22.0 -12.0 0.0) (float-vector -12.0 -12.0 0.0) (float-vector -12.0 -22.0 0.0) (float-vector -22.0 -22.0 0.0)))
         (instance face :init :vertices (list (float-vector -12.0 -12.0 1900.0) (float-vector -12.0 -12.0 0.0) (float-vector -22.0 -12.0 0.0) (float-vector -22.0 -12.0 1900.0)))
         (instance face :init :vertices (list (float-vector -12.0 -22.0 1900.0) (float-vector -12.0 -22.0 0.0) (float-vector -12.0 -12.0 0.0) (float-vector -12.0 -12.0 1900.0)))
         (instance face :init :vertices (list (float-vector -22.0 -22.0 1900.0) (float-vector -22.0 -22.0 0.0) (float-vector -12.0 -22.0 0.0) (float-vector -12.0 -22.0 1900.0)))
         (instance face :init :vertices (list (float-vector -22.0 -12.0 1900.0) (float-vector -22.0 -12.0 0.0) (float-vector -22.0 -22.0 0.0) (float-vector -22.0 -22.0 1900.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :darkseagreen2)
     (setq blink0 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room602-door-bodyset2 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of :room602-door-bodyset3
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 17.5 17.0 1900.0) (float-vector -17.5 17.0 1900.0) (float-vector -17.5 -863.0 1900.0) (float-vector 17.5 -863.0 1900.0)))
         (instance face :init :vertices (list (float-vector -17.5 17.0 0.0) (float-vector 17.5 17.0 0.0) (float-vector 17.5 -863.0 0.0) (float-vector -17.5 -863.0 0.0)))
         (instance face :init :vertices (list (float-vector 17.5 17.0 1900.0) (float-vector 17.5 17.0 0.0) (float-vector -17.5 17.0 0.0) (float-vector -17.5 17.0 1900.0)))
         (instance face :init :vertices (list (float-vector 17.5 -863.0 1900.0) (float-vector 17.5 -863.0 0.0) (float-vector 17.5 17.0 0.0) (float-vector 17.5 17.0 1900.0)))
         (instance face :init :vertices (list (float-vector -17.5 -863.0 1900.0) (float-vector -17.5 -863.0 0.0) (float-vector 17.5 -863.0 0.0) (float-vector 17.5 -863.0 1900.0)))
         (instance face :init :vertices (list (float-vector -17.5 17.0 1900.0) (float-vector -17.5 17.0 0.0) (float-vector -17.5 -863.0 0.0) (float-vector -17.5 -863.0 1900.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :darkseagreen2)
     (setq blink1 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room602-door-bodyset3 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of assoc
     (send blink1 :newcoords (make-coords :pos (float-vector -17.0 -17.0 0.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
     (send blink0 :assoc blink1)
     (send self :assoc blink0)

     ;; definition of end-coords

     ;; definition of joint

     ;; definition of :door
     (setq joint0 (instance rotational-joint :init
			     :parent-link blink0 :child-link blink1 :name :door      :axis :z 
			     :min    0.0 :max   90.0 :max-joint-velocity 5 :max-joint-torque 100))


     ;; init-ending 
     (setq links (list blink0 blink1))
     (setq joint-list (list joint0))
     (send self :init-ending)
     (send self :move-to (make-coords :pos pos :rot rot))
     (send-all links :worldcoords)

     self))

  (:door (&rest args) (forward-message-to joint0 args))
  (:handle (&rest args) (forward-message-to-all (list ) args))
  (:attention (&rest args) (forward-message-to-all (list ) args))
  (:button (&rest args) (forward-message-to-all (list ) args))
  )

(defun room602-door (&rest args) (instance* room602-door-object :init args))
;; (format *error-output* "(instance room602-door-object :init) for generating model~%")
