;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defclass room83b1-shelf-1-object
  :super cascaded-link
  :slots (sensors
         handle0
         ))
(defmethod room83b1-shelf-1-object
  (:init
   (&rest args &key (name "room83b1-shelf-1") (pos (float-vector 0 0 0)) (rot (unit-matrix 3)) &allow-other-keys)
   (let (c bc
         blink0
                 )
     (send-super* :init :name name args)

     ;; definition of link

     ;; definition of :room83b1-shelf-1-bodyset2
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 125.0 400.0 50.0) (float-vector -125.0 400.0 50.0) (float-vector -125.0 -400.0 50.0) (float-vector 125.0 -400.0 50.0)))
         (instance face :init :vertices (list (float-vector -125.0 400.0 0.0) (float-vector 125.0 400.0 0.0) (float-vector 125.0 -400.0 0.0) (float-vector -125.0 -400.0 0.0)))
         (instance face :init :vertices (list (float-vector 125.0 400.0 50.0) (float-vector 125.0 400.0 0.0) (float-vector -125.0 400.0 0.0) (float-vector -125.0 400.0 50.0)))
         (instance face :init :vertices (list (float-vector 125.0 -400.0 50.0) (float-vector 125.0 -400.0 0.0) (float-vector 125.0 400.0 0.0) (float-vector 125.0 400.0 50.0)))
         (instance face :init :vertices (list (float-vector -125.0 -400.0 50.0) (float-vector -125.0 -400.0 0.0) (float-vector 125.0 -400.0 0.0) (float-vector 125.0 -400.0 50.0)))
         (instance face :init :vertices (list (float-vector -125.0 400.0 50.0) (float-vector -125.0 400.0 0.0) (float-vector -125.0 -400.0 0.0) (float-vector -125.0 -400.0 50.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 125.0 400.0 1800.0) (float-vector -125.0 400.0 1800.0) (float-vector -125.0 -400.0 1800.0) (float-vector 125.0 -400.0 1800.0)))
         (instance face :init :vertices (list (float-vector -125.0 400.0 1750.0) (float-vector 125.0 400.0 1750.0) (float-vector 125.0 -400.0 1750.0) (float-vector -125.0 -400.0 1750.0)))
         (instance face :init :vertices (list (float-vector 125.0 400.0 1800.0) (float-vector 125.0 400.0 1750.0) (float-vector -125.0 400.0 1750.0) (float-vector -125.0 400.0 1800.0)))
         (instance face :init :vertices (list (float-vector 125.0 -400.0 1800.0) (float-vector 125.0 -400.0 1750.0) (float-vector 125.0 400.0 1750.0) (float-vector 125.0 400.0 1800.0)))
         (instance face :init :vertices (list (float-vector -125.0 -400.0 1800.0) (float-vector -125.0 -400.0 1750.0) (float-vector 125.0 -400.0 1750.0) (float-vector 125.0 -400.0 1800.0)))
         (instance face :init :vertices (list (float-vector -125.0 400.0 1800.0) (float-vector -125.0 400.0 1750.0) (float-vector -125.0 -400.0 1750.0) (float-vector -125.0 -400.0 1800.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -75.0 375.0 1750.0) (float-vector -125.0 375.0 1750.0) (float-vector -125.0 -375.0 1750.0) (float-vector -75.0 -375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 375.0 50.0) (float-vector -75.0 375.0 50.0) (float-vector -75.0 -375.0 50.0) (float-vector -125.0 -375.0 50.0)))
         (instance face :init :vertices (list (float-vector -75.0 375.0 1750.0) (float-vector -75.0 375.0 50.0) (float-vector -125.0 375.0 50.0) (float-vector -125.0 375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -75.0 -375.0 1750.0) (float-vector -75.0 -375.0 50.0) (float-vector -75.0 375.0 50.0) (float-vector -75.0 375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 -375.0 1750.0) (float-vector -125.0 -375.0 50.0) (float-vector -75.0 -375.0 50.0) (float-vector -75.0 -375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 375.0 1750.0) (float-vector -125.0 375.0 50.0) (float-vector -125.0 -375.0 50.0) (float-vector -125.0 -375.0 1750.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 125.0 425.0 1750.0) (float-vector -125.0 425.0 1750.0) (float-vector -125.0 375.0 1750.0) (float-vector 125.0 375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 425.0 50.0) (float-vector 125.0 425.0 50.0) (float-vector 125.0 375.0 50.0) (float-vector -125.0 375.0 50.0)))
         (instance face :init :vertices (list (float-vector 125.0 425.0 1750.0) (float-vector 125.0 425.0 50.0) (float-vector -125.0 425.0 50.0) (float-vector -125.0 425.0 1750.0)))
         (instance face :init :vertices (list (float-vector 125.0 375.0 1750.0) (float-vector 125.0 375.0 50.0) (float-vector 125.0 425.0 50.0) (float-vector 125.0 425.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 375.0 1750.0) (float-vector -125.0 375.0 50.0) (float-vector 125.0 375.0 50.0) (float-vector 125.0 375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 425.0 1750.0) (float-vector -125.0 425.0 50.0) (float-vector -125.0 375.0 50.0) (float-vector -125.0 375.0 1750.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 125.0 -375.0 1750.0) (float-vector -125.0 -375.0 1750.0) (float-vector -125.0 -425.0 1750.0) (float-vector 125.0 -425.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 -375.0 50.0) (float-vector 125.0 -375.0 50.0) (float-vector 125.0 -425.0 50.0) (float-vector -125.0 -425.0 50.0)))
         (instance face :init :vertices (list (float-vector 125.0 -375.0 1750.0) (float-vector 125.0 -375.0 50.0) (float-vector -125.0 -375.0 50.0) (float-vector -125.0 -375.0 1750.0)))
         (instance face :init :vertices (list (float-vector 125.0 -425.0 1750.0) (float-vector 125.0 -425.0 50.0) (float-vector 125.0 -375.0 50.0) (float-vector 125.0 -375.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 -425.0 1750.0) (float-vector -125.0 -425.0 50.0) (float-vector 125.0 -425.0 50.0) (float-vector 125.0 -425.0 1750.0)))
         (instance face :init :vertices (list (float-vector -125.0 -375.0 1750.0) (float-vector -125.0 -375.0 50.0) (float-vector -125.0 -425.0 50.0) (float-vector -125.0 -425.0 1750.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 112.5 387.5 1435.0) (float-vector -112.5 387.5 1435.0) (float-vector -112.5 -387.5 1435.0) (float-vector 112.5 -387.5 1435.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 1385.0) (float-vector 112.5 387.5 1385.0) (float-vector 112.5 -387.5 1385.0) (float-vector -112.5 -387.5 1385.0)))
         (instance face :init :vertices (list (float-vector 112.5 387.5 1435.0) (float-vector 112.5 387.5 1385.0) (float-vector -112.5 387.5 1385.0) (float-vector -112.5 387.5 1435.0)))
         (instance face :init :vertices (list (float-vector 112.5 -387.5 1435.0) (float-vector 112.5 -387.5 1385.0) (float-vector 112.5 387.5 1385.0) (float-vector 112.5 387.5 1435.0)))
         (instance face :init :vertices (list (float-vector -112.5 -387.5 1435.0) (float-vector -112.5 -387.5 1385.0) (float-vector 112.5 -387.5 1385.0) (float-vector 112.5 -387.5 1435.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 1435.0) (float-vector -112.5 387.5 1385.0) (float-vector -112.5 -387.5 1385.0) (float-vector -112.5 -387.5 1435.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 112.5 387.5 1095.0) (float-vector -112.5 387.5 1095.0) (float-vector -112.5 -387.5 1095.0) (float-vector 112.5 -387.5 1095.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 1045.0) (float-vector 112.5 387.5 1045.0) (float-vector 112.5 -387.5 1045.0) (float-vector -112.5 -387.5 1045.0)))
         (instance face :init :vertices (list (float-vector 112.5 387.5 1095.0) (float-vector 112.5 387.5 1045.0) (float-vector -112.5 387.5 1045.0) (float-vector -112.5 387.5 1095.0)))
         (instance face :init :vertices (list (float-vector 112.5 -387.5 1095.0) (float-vector 112.5 -387.5 1045.0) (float-vector 112.5 387.5 1045.0) (float-vector 112.5 387.5 1095.0)))
         (instance face :init :vertices (list (float-vector -112.5 -387.5 1095.0) (float-vector -112.5 -387.5 1045.0) (float-vector 112.5 -387.5 1045.0) (float-vector 112.5 -387.5 1095.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 1095.0) (float-vector -112.5 387.5 1045.0) (float-vector -112.5 -387.5 1045.0) (float-vector -112.5 -387.5 1095.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 112.5 387.5 755.0) (float-vector -112.5 387.5 755.0) (float-vector -112.5 -387.5 755.0) (float-vector 112.5 -387.5 755.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 705.0) (float-vector 112.5 387.5 705.0) (float-vector 112.5 -387.5 705.0) (float-vector -112.5 -387.5 705.0)))
         (instance face :init :vertices (list (float-vector 112.5 387.5 755.0) (float-vector 112.5 387.5 705.0) (float-vector -112.5 387.5 705.0) (float-vector -112.5 387.5 755.0)))
         (instance face :init :vertices (list (float-vector 112.5 -387.5 755.0) (float-vector 112.5 -387.5 705.0) (float-vector 112.5 387.5 705.0) (float-vector 112.5 387.5 755.0)))
         (instance face :init :vertices (list (float-vector -112.5 -387.5 755.0) (float-vector -112.5 -387.5 705.0) (float-vector 112.5 -387.5 705.0) (float-vector 112.5 -387.5 755.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 755.0) (float-vector -112.5 387.5 705.0) (float-vector -112.5 -387.5 705.0) (float-vector -112.5 -387.5 755.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 112.5 387.5 415.0) (float-vector -112.5 387.5 415.0) (float-vector -112.5 -387.5 415.0) (float-vector 112.5 -387.5 415.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 365.0) (float-vector 112.5 387.5 365.0) (float-vector 112.5 -387.5 365.0) (float-vector -112.5 -387.5 365.0)))
         (instance face :init :vertices (list (float-vector 112.5 387.5 415.0) (float-vector 112.5 387.5 365.0) (float-vector -112.5 387.5 365.0) (float-vector -112.5 387.5 415.0)))
         (instance face :init :vertices (list (float-vector 112.5 -387.5 415.0) (float-vector 112.5 -387.5 365.0) (float-vector 112.5 387.5 365.0) (float-vector 112.5 387.5 415.0)))
         (instance face :init :vertices (list (float-vector -112.5 -387.5 415.0) (float-vector -112.5 -387.5 365.0) (float-vector 112.5 -387.5 365.0) (float-vector 112.5 -387.5 415.0)))
         (instance face :init :vertices (list (float-vector -112.5 387.5 415.0) (float-vector -112.5 387.5 365.0) (float-vector -112.5 -387.5 365.0) (float-vector -112.5 -387.5 415.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :gray20)
     (send (elt bc 1) :set-color :gray20)
     (send (elt bc 2) :set-color :gray20)
     (send (elt bc 3) :set-color :gray20)
     (send (elt bc 4) :set-color :gray20)
     (send (elt bc 5) :set-color :gray20)
     (send (elt bc 6) :set-color :gray20)
     (send (elt bc 7) :set-color :gray20)
     (send (elt bc 8) :set-color :gray20)
     (setq blink0 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room83b1-shelf-1-bodyset2 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of assoc
     (send self :assoc blink0)

     ;; definition of end-coords

     ;; definition of joint


     ;; definition of :handle
     (setq handle0 (make-cascoords :pos (float-vector 0.0 0.0 0.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0)) :name "handle0"))
     (send blink0 :assoc handle0)

     ;; init-ending 
     (setq links (list blink0))
     (setq joint-list (list))
     (send self :init-ending)
     (send self :move-to (make-coords :pos pos :rot rot))
     (send-all links :worldcoords)

     self))

  (:handle (&rest args) (forward-message-to-all (list  handle0) args))
  (:handle-handle0 (&rest args) (forward-message-to handle0 args))
  (:attention (&rest args) (forward-message-to-all (list ) args))
  (:button (&rest args) (forward-message-to-all (list ) args))
  )

(defun room83b1-shelf-1 (&rest args) (instance* room83b1-shelf-1-object :init args))
;; (format *error-output* "(instance room83b1-shelf-1-object :init) for generating model~%")
