(defconstant XV_XID #i(1247873793)) ; #x4a610b01
(defconstant XV_DISPLAY #i(1248725505)) ; #x4a6e0a01
(defconstant XV_X #i(1246103617)) ; #x4a460841
(defconstant XV_Y #i(1246169217)) ; #x4a470881
(defconstant XV_WIDTH #i(1246234689)) ; #x4a480841
(defconstant XV_HEIGHT #i(1246300289)) ; #x4a490881
(defconstant XV_VISUAL #i(1248004609)) ; #x4a630a01
(defconstant ITIMER_REAL 0)
(defconstant ITIMER_VIRTUAL 1)
(defconstant NOTIFY_POLLING_ITIMER (make-string 16))
(replace NOTIFY_POLLING_ITIMER  (list 0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 1) )
(defconstant PIX_SRC (* #xC 2))
(defconstant PIX_DST (* #xA 2))
(defun PIX_NOT (op) (logand (* #xf 2) (lognot op)))
(defconstant PIX_CLR (logand PIX_SRC (pix_not pix_src)))
(defconstant PIX_SET (logior PIX_SRC (pix_not pix_src)))
(defun PIX_COLOR (c) (* c 32))
(defun PIX_OPCOLOR (op) (/ op 32))
(defun PIXOP_NEEDS_DST (op)
(logand (logxor op (/ op 2)) (pix_not pix_dst)))
(defun PIXOP_NEEDS_SRC (op)
  (logand (logxor op (/ op 4)) (pix_not pix_src)))
(defconstant PIX_DONTCLIP	#x1)
(defconstant PIX_CLIP		#x0)
(defconstant NOTIFY_DONE 0)
(defconstant NOTIFY_IGNORED 1)
(defconstant NOTIFY_UNEXPECTED 2)
(defconstant NOTIFY_OK 0)
(defconstant NOTIFY_UNKNOWN_CLIENT 1)
(defconstant NOTIFY_NO_CONDITION 2)
(defconstant NOTIFY_BAD_ITIMER 3)
(defconstant NOTIFY_BAD_SIGNAL 4)
(defconstant NOTIFY_NOT_STARTED 5)
(defconstant NOTIFY_DESTROY_VETOED 6)
(defconstant NOTIFY_INTERNAL_ERROR 7)
(defconstant NOTIFY_SRCH 8)
(defconstant NOTIFY_BADF 9)
(defconstant NOTIFY_NOMEM 10)
(defconstant NOTIFY_INVAL 11)
(defconstant NOTIFY_SYNC 0)
(defconstant NOTIFY_ASYNC 1)
(defconstant NOTIFY_SAFE 0)
(defconstant NOTIFY_IMMEDIATE 1)
(defconstant DESTROY_PROCESS_DEATH 0)
(defconstant DESTROY_CHECKING 1)
(defconstant DESTROY_CLEANUP 2)
(defconstant ATTR_BASE_INT 64)
(defconstant ATTR_BASE_X 66)
(defconstant ATTR_BASE_INDEX_X 67)
(defconstant ATTR_BASE_Y 68)
(defconstant ATTR_BASE_INDEX_Y 69)
(defconstant ATTR_BASE_XY 70)
(defconstant ATTR_BASE_INDEX_XY 71)
(defconstant ATTR_BASE_BOOLEAN 72)
(defconstant ATTR_BASE_ENUM 73)
(defconstant ATTR_BASE_CHAR 74)
(defconstant ATTR_BASE_STRING 75)
(defconstant ATTR_BASE_PIXRECT_PTR 76)
(defconstant ATTR_BASE_PIXFONT_PTR 77)
(defconstant ATTR_BASE_RECT_PTR 79)
(defconstant ATTR_BASE_OPAQUE 80)
(defconstant ATTR_BASE_NO_VALUE 81)
(defconstant ATTR_BASE_AV 82)
(defconstant ATTR_BASE_FUNCTION_PTR 83)
(defconstant ATTR_BASE_ICON_PTR 84)
(defconstant ATTR_BASE_SINGLE_COLOR_PTR 85)
(defconstant ATTR_BASE_LONG 88)
(defconstant ATTR_BASE_SHORT 89)
(defconstant ATTR_INT 2049)
(defconstant ATTR_INT_PAIR 2050)
(defconstant ATTR_INT_TRIPLE 2051)
(defconstant ATTR_X 2113)
(defconstant ATTR_INDEX_X 2146)
(defconstant ATTR_Y 2177)
(defconstant ATTR_INDEX_Y 2210)
(defconstant ATTR_XY 2242)
(defconstant ATTR_INDEX_XY 2275)
(defconstant ATTR_BOOLEAN 2305)
(defconstant ATTR_ENUM 2337)
(defconstant ATTR_CHAR 2369)
(defconstant ATTR_STRING 2401)
(defconstant ATTR_PIXRECT_PTR 2433)
(defconstant ATTR_PIXFONT_PTR 2465)
(defconstant ATTR_PIXWIN_PTR 2561)
(defconstant ATTR_RECT_PTR 2529)
(defconstant ATTR_OPAQUE 2561)
(defconstant ATTR_NO_VALUE 2592)
(defconstant ATTR_AV 2625)
(defconstant ATTR_FUNCTION_PTR 2657)
(defconstant ATTR_ICON_PTR 2689)
(defconstant ATTR_SINGLE_COLOR_PTR 2721)
(defconstant ATTR_LONG 2817)
(defconstant ATTR_SHORT 2849)
(defconstant ATTR_LIST_IS_INLINE 0)
(defconstant ATTR_LIST_IS_PTR 1)
(defconstant ATTR_NONE 0)
(defconstant ATTR_RECURSIVE 1)
(defconstant ATTR_NULL 2)
(defconstant ATTR_COUNTED 3)
(defconstant ATTR_PKG_ZERO 0)
(defconstant ATTR_PKG_GENERIC 64)
(defconstant ATTR_PKG_PANEL 8)
(defconstant ATTR_PKG_SCROLLBAR 20)
(defconstant ATTR_PKG_CURSOR 65)
(defconstant ATTR_PKG_MENU 7)
(defconstant ATTR_PKG_TEXTSW 10)
(defconstant ATTR_PKG_SELN_BASE 71)
(defconstant ATTR_PKG_ENTITY 3)
(defconstant ATTR_PKG_IMAGE 68)
(defconstant ATTR_PKG_WIN 73)
(defconstant ATTR_PKG_FRAME 5)
(defconstant ATTR_PKG_CANVAS 2)
(defconstant ATTR_PKG_TTY 11)
(defconstant ATTR_PKG_ICON 6)
(defconstant ATTR_PKG_LAST 76)
(defconstant ATTR_LIST #i(1073768992)) ; #x40006a20
(defconstant ATTR_CU_POSITION 0)
(defconstant ATTR_CU_LENGTH #i(536870912)) ; #x20000000
(defconstant VUID_ASCII 0)
(defconstant VUID_TOP 1)
(defconstant VUID_PANEL 125)
(defconstant VUID_SCROLL 126)
(defconstant VUID_WORKSTATION 127)
(defconstant VUID_LAST 255)
(defconstant ICON_WIDTH #i(1246234689)) ; #x4a480841
(defconstant ICON_HEIGHT #i(1246300289)) ; #x4a490881
(defconstant ICON_IMAGE_RECT 101321185)
(defconstant ICON_IMAGE 100993409)
(defconstant ICON_LABEL #i(1078397281)) ; #x40470961
(defconstant ICON_FONT #i(1245710849)) ; #x4a400a01
(defconstant MENU_ACTION_PROC 117836385)
(defconstant MENU_ACTION 117836385)
(defconstant MENU_ACTION_IMAGE 117508098)
(defconstant MENU_ACTION_ITEM 117639170)
(defconstant MENU_APPEND_ITEM 118032897)
(defconstant MENU_CLIENT_DATA 118426113)
(defconstant MENU_DEFAULT 118818817)
(defconstant MENU_DEFAULT_ITEM 119015937)
(defconstant MENU_DEFAULT_IMAGE 130615809)
(defconstant MENU_DISABLE_ITEM 119408929)
(defconstant MENU_DEFAULT_SELECTION 132778273)
(defconstant MENU_DESCEND_FIRST 119212289)
(defconstant MENU_FEEDBACK 119605505)
(defconstant MENU_FIRST_EVENT 119802400)
(defconstant MENU_GEN_PULLRIGHT 120785505)
(defconstant MENU_GEN_PULLRIGHT_IMAGE 120981506)
(defconstant MENU_GEN_PULLRIGHT_ITEM 121178114)
(defconstant MENU_GEN_PROC 120195681)
(defconstant MENU_GEN_PROC_IMAGE 120391682)
(defconstant MENU_GEN_PROC_ITEM 120588290)
(defconstant MENU_HEIGHT #i(1246300289)) ; #x4a490881
(defconstant MENU_IMAGE 121375233)
(defconstant MENU_IMAGE_ITEM 121767938)
(defconstant MENU_IMAGES 121604609)
(defconstant MENU_INITIAL_SELECTION 122161441)
(defconstant MENU_INITIAL_SELECTION_SELECTED 122554625)
(defconstant MENU_INITIAL_SELECTION_EXPANDED 122358017)
(defconstant MENU_INACTIVE 121964801)
(defconstant MENU_INSERT 122750978)
(defconstant MENU_INSERT_ITEM 122947586)
(defconstant MENU_ITEM 123357761)
(defconstant MENU_INVERT 123144449)
(defconstant MENU_JUMP_AFTER_SELECTION 123734273)
(defconstant MENU_JUMP_AFTER_NO_SELECTION 123537665)
(defconstant MENU_LAST_EVENT 123931168)
(defconstant MENU_LEFT_MARGIN #i(1245841409)) ; #x4a420801
(defconstant MENU_MARGIN #i(1245775873)) ; #x4a410801
(defconstant MENU_MENU 124127776)
(defconstant MENU_NITEMS 124520992)
(defconstant MENU_NOTIFY_PROC 117836385)
(defconstant MENU_NTH_ITEM 124913665)
(defconstant MENU_NCOLS 124323841)
(defconstant MENU_NROWS 125110273)
(defconstant MENU_OWNER #i(1078856193)) ; #x404e0a01
(defconstant MENU_PULLRIGHT 125897217)
(defconstant MENU_PULLRIGHT_DELTA 126093313)
(defconstant MENU_PULLRIGHT_IMAGE 126289922)
(defconstant MENU_PULLRIGHT_ITEM 126486530)
(defconstant MENU_RELEASE 126683680)
(defconstant MENU_RELEASE_IMAGE 126880288)
(defconstant MENU_REMOVE 127076353)
(defconstant MENU_REMOVE_ITEM 127273473)
(defconstant MENU_REPLACE 127469570)
(defconstant MENU_REPLACE_ITEM 127666178)
(defconstant MENU_RIGHT_MARGIN #i(1245972481)) ; #x4a440801
(defconstant MENU_SHADOW 132974977)
(defconstant MENU_STRING 128256353)
(defconstant MENU_STRING_ITEM 128649218)
(defconstant MENU_STRINGS 128485729)
(defconstant MENU_TITLE_ITEM 129239393)
(defconstant MENU_TITLE_IMAGE 129042945)
(defconstant MENU_TYPE 129436192)
(defconstant MENU_VALID_RESULT 129632257)
(defconstant MENU_VALUE 129829377)
(defconstant MENU_WIDTH #i(1246234689)) ; #x4a480841
(defconstant MENU_BUTTON 130222081)
(defconstant MENU_FD 131008513)
(defconstant MENU_IE 131401729)
(defconstant MENU_POS 131794946)
(defconstant WIN_ERROR_MSG #i(1227884897)) ; #x49300961
(defconstant WIN_X #i(1246103617)) ; #x4a460841
(defconstant WIN_Y #i(1246169217)) ; #x4a470881
(defconstant WIN_RIGHT_OF #i(1232341505)) ; #x49740a01
(defconstant WIN_BELOW #i(1225001473)) ; #x49040a01
(defconstant WIN_WIDTH #i(1246234689)) ; #x4a480841
(defconstant WIN_HEIGHT #i(1246300289)) ; #x4a490881
(defconstant WIN_PERCENT_WIDTH #i(1231816705)) ; #x496c0801
(defconstant WIN_PERCENT_HEIGHT #i(1231554561)) ; #x49680801
(defconstant WIN_COLUMNS #i(1225525249)) ; #x490c0801
(defconstant WIN_ROWS #i(1232603137)) ; #x49780801
(defconstant WIN_RECT #i(1246366177)) ; #x4a4a09e1
(defconstant WIN_SCREEN_RECT #i(1233390112)) ; #x49840a20
(defconstant WIN_FIT_HEIGHT #i(1228671105)) ; #x493c0881
(defconstant WIN_FIT_WIDTH #i(1228933185)) ; #x49400841
(defconstant WIN_SHOW #i(1079052545)) ; #x40510901
(defconstant WIN_MENU #i(1230768641)) ; #x495c0a01
(defconstant WIN_CURSOR #i(1226836673)) ; #x49200ac1
(defconstant WIN_EVENT_PROC #i(1228147297)) ; #x49340a61
(defconstant WIN_GRAB_ALL_INPUT #i(1229195521)) ; #x49440901
(defconstant WIN_KBD_INPUT_MASK #i(1230244353)) ; #x49540a01
(defconstant WIN_CONSUME_KBD_EVENT #i(1226311969)) ; #x49180921
(defconstant WIN_IGNORE_KBD_EVENT #i(1229719841)) ; #x494c0921
(defconstant WIN_CONSUME_KBD_EVENTS #i(1226606881)) ; #x491c8921
(defconstant WIN_IGNORE_KBD_EVENTS #i(1230014753)) ; #x49508921
(defconstant WIN_PICK_INPUT_MASK #i(1230244353)) ; #x49540a01
(defconstant WIN_CONSUME_PICK_EVENT #i(1226311969)) ; #x49180921
(defconstant WIN_IGNORE_PICK_EVENT #i(1229719841)) ; #x494c0921
(defconstant WIN_CONSUME_PICK_EVENTS #i(1226606881)) ; #x491c8921
(defconstant WIN_IGNORE_PICK_EVENTS #i(1230014753)) ; #x49508921
(defconstant WIN_FONT #i(1245710849)) ; #x4a400a01
(defconstant WIN_TOP_MARGIN #i(1245906945)) ; #x4a430801
(defconstant WIN_BOTTOM_MARGIN #i(1246038017)) ; #x4a450801
(defconstant WIN_LEFT_MARGIN #i(1245841409)) ; #x4a420801
(defconstant WIN_RIGHT_MARGIN #i(1245972481)) ; #x4a440801
(defconstant WIN_ROW_HEIGHT #i(1233127425)) ; #x49800801
(defconstant WIN_COLUMN_WIDTH #i(1226049537)) ; #x49140801
(defconstant WIN_ROW_GAP #i(1232865281)) ; #x497c0801
(defconstant WIN_COLUMN_GAP #i(1225787393)) ; #x49100801
(defconstant WIN_TYPE #i(1239419169)) ; #x49e00921
(defconstant WIN_OWNER #i(1078856193)) ; #x404e0a01
(defconstant WIN_NAME #i(1078462817)) ; #x40480961
(defconstant WIN_CLIENT_DATA #i(1225263617)) ; #x49080a01
(defconstant WIN_XNAME #i(1247807841)) ; #x4a600961
(defconstant WIN_DEVICE_NUMBER #i(1247873793)) ; #x4a610b01
(defconstant WIN_DYNAMIC_VISUAL #i(1239943425)) ; #x49e80901
(defconstant WIN_PIXWIN #i(1078790657)) ; #x404d0a01
(defconstant WIN_MOUSE_XY #i(1231030466)) ; #x496008c2
(defconstant WIN_NOTIFY_EVENT_PROC #i(1237322337)) ; #x49c00a61
(defconstant WIN_KBD_FOCUS #i(1236011265)) ; #x49ac0901
(defconstant WIN_EVENT_STATE #i(1228409089)) ; #x49380901
(defconstant FRAME_CLOSED 84871425)
(defconstant FRAME_INHERIT_COLORS 87820545)
(defconstant FRAME_SHOW_LABEL 92408065)
(defconstant FRAME_LABEL #i(1078397281)) ; #x40470961
(defconstant FRAME_FOREGROUND_COLOR 87165601)
(defconstant FRAME_BACKGROUND_COLOR 84216481)
(defconstant FRAME_ICON 87493121)
(defconstant FRAME_CLOSED_RECT 85199329)
(defconstant FRAME_CURRENT_RECT 86182369)
(defconstant FRAME_OPEN_RECT #i(1246366177)) ; #x4a4a09e1
(defconstant FRAME_CMDLINE_HELP_PROC #i(1242106465)) ; #x4a090a61
(defconstant FRAME_NTH_SUBWINDOW 89131009)
(defconstant FRAME_NTH_SUBFRAME 88803329)
(defconstant FRAME_DONE_PROC 86837857)
(defconstant FRAME_DEFAULT_DONE_PROC 86510177)
(defconstant FRAME_NO_CONFIRM 88475905)
(defconstant PANEL_ITEM_X #i(1246103617)) ; #x4a460841
(defconstant PANEL_ITEM_Y #i(1246169217)) ; #x4a470881
(defconstant PANEL_LABEL_X 139331649)
(defconstant PANEL_VALUE_X 146671681)
(defconstant PANEL_SLIDER_WIDTH 145754177)
(defconstant PANEL_WIDTH #i(1246234689)) ; #x4a480841
(defconstant PANEL_CHOICE_X 135923810)
(defconstant PANEL_MARK_X 142608482)
(defconstant PANEL_CHOICE_XS 136087617)
(defconstant PANEL_MARK_XS 142772289)
(defconstant PANEL_X #i(1246103617)) ; #x4a460841
(defconstant PANEL_Y #i(1246169217)) ; #x4a470881
(defconstant PANEL_X_GAP #i(1246103617)) ; #x4a460841
(defconstant PANEL_Y_GAP #i(1246169217)) ; #x4a470881
(defconstant PANEL_LABEL_Y 139462785)
(defconstant PANEL_VALUE_Y 146802817)
(defconstant PANEL_HEIGHT #i(1246300289)) ; #x4a490881
(defconstant PANEL_CHOICE_Y 136186018)
(defconstant PANEL_MARK_Y 142870690)
(defconstant PANEL_CHOICE_YS 136349825)
(defconstant PANEL_MARK_YS 143034497)
(defconstant PANEL_VALUE 146016257)
(defconstant PANEL_VALUE_DISPLAY_LENGTH 146147329)
(defconstant PANEL_VALUE_STORED_LENGTH 146409473)
(defconstant PANEL_MIN_VALUE 143656961)
(defconstant PANEL_MAX_VALUE 143263745)
(defconstant PANEL_CHOICE_OFFSET 147064833)
(defconstant PANEL_VALUE_UNDERLINED 146540801)
(defconstant PANEL_LABEL_BOLD 138676481)
(defconstant PANEL_CHOICES_BOLD 134875393)
(defconstant PANEL_SHOW #i(1079052545)) ; #x40510901
(defconstant PANEL_SHOW_VALUE 145623297)
(defconstant PANEL_SHOW_RANGE 145492225)
(defconstant PANEL_SHOW_MENU 145230081)
(defconstant PANEL_SHOW_MENU_MARK 145361153)
(defconstant PANEL_CHOOSE_ONE 136448257)
(defconstant PANEL_BLINK_CARET 134482177)
(defconstant PANEL_ACCEPT_KEYSTROKE 134285569)
(defconstant PANEL_TOGGLE_VALUE 145883202)
(defconstant PANEL_NOTIFY_LEVEL 144181537)
(defconstant PANEL_DISPLAY_LEVEL 136972577)
(defconstant PANEL_LAYOUT 139594017)
(defconstant PANEL_FEEDBACK 137496865)
(defconstant PANEL_PAINT 144705825)
(defconstant PANEL_MASK_CHAR 143132993)
(defconstant PANEL_LABEL_STRING 139069793)
(defconstant PANEL_NOTIFY_STRING 144574817)
(defconstant PANEL_CHOICE_STRINGS 135694689)
(defconstant PANEL_CHOICE_STRING 135528450)
(defconstant PANEL_CHOICE_IMAGES 135432577)
(defconstant PANEL_LABEL_IMAGE 138938753)
(defconstant PANEL_MARK_IMAGES 142510465)
(defconstant PANEL_NOMARK_IMAGES 144083329)
(defconstant PANEL_CHOICE_IMAGE 135266338)
(defconstant PANEL_MARK_IMAGE 142344226)
(defconstant PANEL_NOMARK_IMAGE 143917090)
(defconstant PANEL_LABEL_FONT 138807713)
(defconstant PANEL_VALUE_FONT 146278817)
(defconstant PANEL_FONT #i(1245710849)) ; #x4a400a01
(defconstant PANEL_CHOICE_FONTS 135170465)
(defconstant PANEL_NOTIFY_PROC 144312929)
(defconstant PANEL_EVENT_PROC 137103969)
(defconstant PANEL_BACKGROUND_PROC 134351457)
(defconstant PANEL_ITEM_RECT 138283489)
(defconstant PANEL_CLIENT_DATA 136579585)
(defconstant WIN_VERTICAL_SCROLLBAR #i(1233914369)) ; #x498c0a01
(defconstant WIN_HORIZONTAL_SCROLLBAR #i(1229457921)) ; #x49480a01
(defconstant PANEL_CARET_ITEM 134744577)
(defconstant PANEL_FIRST_ITEM 137628161)
(defconstant PANEL_NEXT_ITEM 143788545)
(defconstant PANEL_PARENT_PANEL #i(1078856193)) ; #x404e0a01
(defconstant CANVAS_WIDTH 37488705)
(defconstant CANVAS_HEIGHT 34539649)
(defconstant CANVAS_VIEW_PAINT_WINDOW 268503553)
(defconstant CANVAS_VIEWABLE_RECT 37161441)
(defconstant CANVAS_X_PAINT_WINDOW 37816577)
(defconstant CANVAS_VIEW_CANVAS_WINDOW 268569089)
(defconstant CANVAS_PAINT_CANVAS_WINDOW 285280769)
(defconstant CANVAS_NTH_PAINT_WINDOW 35523073)
(defconstant CANVAS_PAINTWINDOW_ATTRS 38160961)
(defconstant CANVAS_NO_CLIPPING 38471937)
(defconstant CANVAS_FIXED_IMAGE 34212097)
(defconstant CANVAS_REPAINT_PROC 35850849)
(defconstant CANVAS_RESIZE_PROC 36178529)
(defconstant OPENWIN_AUTO_CLEAR 151652609)
(defconstant CANVAS_AUTO_EXPAND 33622273)
(defconstant CANVAS_AUTO_SHRINK 33884417)
(defconstant CANVAS_FAST_MONO #i(1248004609)) ; #x4a630a01
(defconstant TTY_ARGV 184617473)
(defconstant TTY_CONSOLE 184879361)
(defconstant TTY_PAGE_MODE 185862401)
(defconstant TTY_QUIT_ON_CHILD_DEATH 186190081)
(defconstant TTY_BOLDSTYLE 186517505)
(defconstant TTY_BOLDSTYLE_NAME 186845537)
(defconstant TTY_PID 187500545)
(defconstant TEXTSW_INFINITY #i(2004318071)) ; #x77777777
(defconstant TEXTSW_DELIMITER_BACKWARD 5)
(defconstant TEXTSW_DELIMITER_FORWARD 4)
(defconstant TEXTSW_DELIMITER_ENCLOSE 6)
(defconstant TEXTSW_ADJUST_IS_PENDING_DELETE 167840001)
(defconstant TEXTSW_AGAIN_LIMIT 173279233)
(defconstant TEXTSW_AGAIN_RECORDING 167905537)
(defconstant TEXTSW_AUTO_INDENT 168036609)
(defconstant TEXTSW_BLINK_CARET 168298753)
(defconstant TEXTSW_BROWSING 168429825)
(defconstant TEXTSW_CHECKPOINT_FREQUENCY 168560641)
(defconstant TEXTSW_CLIENT_DATA 168692225)
(defconstant TEXTSW_CONFIRM_OVERWRITE 168823041)
(defconstant TEXTSW_CONTENTS 168954209)
(defconstant TEXTSW_CONTROL_CHARS_USE_FONT 169085185)
(defconstant TEXTSW_DISABLE_CD 169216257)
(defconstant TEXTSW_DISABLE_LOAD 169347329)
(defconstant TEXTSW_EDIT_COUNT 169609217)
(defconstant TEXTSW_ERROR_MSG 174590305)
(defconstant TEXTSW_FILE 169871713)
(defconstant TEXTSW_FILE_CONTENTS 170133857)
(defconstant TEXTSW_FONT #i(1245710849)) ; #x4a400a01
(defconstant TEXTSW_HISTORY_LIMIT 170657793)
(defconstant TEXTSW_IGNORE_LIMIT 170788865)
(defconstant TEXTSW_INSERT_MAKES_VISIBLE 171182369)
(defconstant TEXTSW_INSERTION_POINT 170919937)
(defconstant TEXTSW_LENGTH 171313153)
(defconstant TEXTSW_LOAD_DIR_IS_CD 174983457)
(defconstant TEXTSW_LOWER_CONTEXT 171575297)
(defconstant TEXTSW_MEMORY_MAXIMUM 171706369)
(defconstant TEXTSW_MODIFIED 171837697)
(defconstant TEXTSW_MULTI_CLICK_SPACE 171968513)
(defconstant TEXTSW_MULTI_CLICK_TIMEOUT 172099585)
(defconstant TEXTSW_NAME_TO_USE 175376737)
(defconstant TEXTSW_NO_RESET_TO_SCRATCH 175769857)
(defconstant TEXTSW_NO_SELECTION_SERVICE 175900929)
(defconstant TEXTSW_NOTIFY_LEVEL 175507457)
(defconstant TEXTSW_NOTIFY_PROC 172231265)
(defconstant TEXTSW_PIXWIN #i(1078790657)) ; #x404d0a01
(defconstant TEXTSW_READ_ONLY 172361985)
(defconstant TEXTSW_RESET_MODE 176163105)
(defconstant TEXTSW_STATUS 172493313)
(defconstant TEXTSW_STORE_CHANGES_FILE 172624129)
(defconstant TEXTSW_TAB_WIDTH 176687105)
(defconstant TEXTSW_TOOL #i(1234962945)) ; #x499c0a01
(defconstant TEXTSW_UPPER_CONTEXT 173017089)
(defconstant TEXTSW_END_ALL_VIEWS 177211936)
(defconstant TEXTSW_FOR_ALL_VIEWS 177343008)
(defconstant TEXTSW_FIRST 170395649)
(defconstant TEXTSW_FIRST_LINE 170526721)
(defconstant TEXTSW_LEFT_MARGIN #i(1245841409)) ; #x4a420801
(defconstant TEXTSW_LINE_BREAK_ACTION 171444513)
(defconstant TEXTSW_RIGHT_MARGIN #i(1245972481)) ; #x4a440801
(defconstant TEXTSW_SCROLLBAR #i(1233914369)) ; #x498c0a01
(defconstant TEXTSW_UPDATE_SCROLLBAR 172886560)
(defconstant TEXTSW_SPARE_1 176457729)
(defconstant TEXTSW_SPARE_2 176588801)
(defconstant TEXTSW_NEVER 0)
(defconstant TEXTSW_ALWAYS 1)
(defconstant TEXTSW_ONLY 2)
(defconstant TEXTSW_ACTION_CAPS_LOCK 167840001)
(defconstant TEXTSW_ACTION_CHANGED_DIRECTORY 168102241)
(defconstant TEXTSW_ACTION_EDITED_FILE 168429921)
(defconstant TEXTSW_ACTION_FILE_IS_READONLY 169085281)
(defconstant TEXTSW_ACTION_LOADED_FILE 169412961)
(defconstant TEXTSW_ACTION_SAVING_FILE 172362272)
(defconstant TEXTSW_ACTION_STORING_FILE 173345121)
(defconstant TEXTSW_ACTION_USING_MEMORY 171051552)
(defconstant TEXTSW_ACTION_WRITE_FAILED 173672992)
(defconstant TEXTSW_ACTION_TOOL_CLOSE 169740832)
(defconstant TEXTSW_ACTION_TOOL_DESTROY 170068481)
(defconstant TEXTSW_ACTION_TOOL_MGR 170396161)
(defconstant TEXTSW_ACTION_TOOL_QUIT 170723841)
(defconstant TEXTSW_ACTION_REPLACED 172034053)
(defconstant TEXTSW_ACTION_PAINTED 171706849)
(defconstant TEXTSW_ACTION_SCROLLED 172689890)
(defconstant TEXTSW_ACTION_DESTROY_VIEW 171379232)
(defconstant TEXTSW_ACTION_SPLIT_VIEW 173017601)
(defconstant TEXTSW_FATTR_INPUT 167840257)
(defconstant TEXTSW_FATTR_INPUT_EVENT 168102401)
(defconstant TEXTSW_FATTR_INSERTION_POINTS 168430081)
(defconstant TEXTSW_FATTR_INSERTION_LINE 168757761)
(defconstant TEXTSW_FATTR_SELECTION_ENDPOINTS 169085441)
(defconstant TEXTSW_FILTER_DELETE_RANGE 0)
(defconstant TEXTSW_FILTER_INSERT 1)
(defconstant TEXTSW_FILTER_SEND_RANGE 2)
(defconstant TEXTSW_FILTER_SET_INSERTION 3)
(defconstant TEXTSW_FILTER_SET_SELECTION 4)
(defconstant XDR_ENCODE 0)
(defconstant XDR_DECODE 1)
(defconstant XDR_FREE 2)
(defconstant AUTH_OK 0)
(defconstant AUTH_BADCRED 1)
(defconstant AUTH_REJECTEDCRED 2)
(defconstant AUTH_BADVERF 3)
(defconstant AUTH_REJECTEDVERF 4)
(defconstant AUTH_TOOWEAK 5)
(defconstant AUTH_INVALIDRESP 6)
(defconstant AUTH_FAILED 7)
(defconstant RPC_SUCCESS 0)
(defconstant RPC_CANTENCODEARGS 1)
(defconstant RPC_CANTDECODERES 2)
(defconstant RPC_CANTSEND 3)
(defconstant RPC_CANTRECV 4)
(defconstant RPC_TIMEDOUT 5)
(defconstant RPC_VERSMISMATCH 6)
(defconstant RPC_AUTHERROR 7)
(defconstant RPC_PROGUNAVAIL 8)
(defconstant RPC_PROGVERSMISMATCH 9)
(defconstant RPC_PROCUNAVAIL 10)
(defconstant RPC_CANTDECODEARGS 11)
(defconstant RPC_SYSTEMERROR 12)
(defconstant RPC_UNKNOWNHOST 13)
(defconstant RPC_PMAPFAILURE 14)
(defconstant RPC_PROGNOTREGISTERED 15)
(defconstant RPC_FAILED 16)
(defconstant CALL 0)
(defconstant REPLY 1)
(defconstant MSG_ACCEPTED 0)
(defconstant MSG_DENIED 1)
(defconstant SUCCESS 0)
(defconstant PROG_UNAVAIL 1)
(defconstant PROG_MISMATCH 2)
(defconstant PROC_UNAVAIL 3)
(defconstant GARBAGE_ARGS 4)
(defconstant SYSTEM_ERR 5)
(defconstant RPC_MISMATCH 0)
(defconstant AUTH_ERROR 1)
(defconstant SELN_REQ_BYTESIZE #i(1191249921)) ; #x47010801
(defconstant SELN_REQ_CONTENTS_ASCII #i(1191348545)) ; #x47028941
(defconstant SELN_REQ_CONTENTS_PIECES #i(1191414081)) ; #x47038941
(defconstant SELN_REQ_FIRST #i(1191446529)) ; #x47040801
(defconstant SELN_REQ_FIRST_UNIT #i(1191512065)) ; #x47050801
(defconstant SELN_REQ_LAST #i(1191577601)) ; #x47060801
(defconstant SELN_REQ_LAST_UNIT #i(1191643137)) ; #x47070801
(defconstant SELN_REQ_LEVEL #i(1191708673)) ; #x47080801
(defconstant SELN_REQ_FILE_NAME #i(1191807297)) ; #x47098941
(defconstant SELN_REQ_DELETE #i(1195510304)) ; #x47420a20
(defconstant SELN_REQ_RESTORE #i(1195575840)) ; #x47430a20
(defconstant SELN_REQ_YIELD #i(1197541665)) ; #x47610921
(defconstant SELN_REQ_FAKE_LEVEL #i(1197606913)) ; #x47620801
(defconstant SELN_REQ_SET_LEVEL #i(1197672449)) ; #x47630801
(defconstant SELN_TRACE_ACQUIRE #i(1203833089)) ; #x47c10901
(defconstant SELN_TRACE_DONE #i(1203898625)) ; #x47c20901
(defconstant SELN_TRACE_HOLD_FILE #i(1203964161)) ; #x47c30901
(defconstant SELN_TRACE_INFORM #i(1204029697)) ; #x47c40901
(defconstant SELN_TRACE_INQUIRE #i(1204095233)) ; #x47c50901
(defconstant SELN_TRACE_YIELD #i(1204160769)) ; #x47c60901
(defconstant SELN_TRACE_STOP #i(1204226305)) ; #x47c70901
(defconstant SELN_TRACE_DUMP #i(1204291873)) ; #x47c80921
(defconstant SELN_REQ_END_REQUEST #i(1207765536)) ; #x47fd0a20
(defconstant SELN_REQ_UNKNOWN #i(1207830529)) ; #x47fe0801
(defconstant SELN_REQ_FAILED #i(1207896065)) ; #x47ff0801
(defconstant SELN_LEVEL_FIRST #i(1073741825)) ; #x40000001
(defconstant SELN_LEVEL_LINE #i(1073742081)) ; #x40000101
(defconstant SELN_LEVEL_ALL #i(1073774593)) ; #x40008001
(defconstant SELN_LEVEL_NEXT #i(1073803265)) ; #x4000f001
(defconstant SELN_LEVEL_PREVIOUS #i(1073803266)) ; #x4000f002
(defconstant SELN_FAILED 0)
(defconstant SELN_SUCCESS 1)
(defconstant SELN_NON_EXIST 2)
(defconstant SELN_DIDNT_HAVE 3)
(defconstant SELN_WRONG_RANK 4)
(defconstant SELN_CONTINUED 5)
(defconstant SELN_CANCEL 6)
(defconstant SELN_UNRECOGNIZED  7)
(defconstant SELN_UNKNOWN 0)
(defconstant SELN_CARET 1)
(defconstant SELN_PRIMARY 2)
(defconstant SELN_SECONDARY 3)
(defconstant SELN_SHELF 4)
(defconstant SELN_UNSPECIFIED 5)
(defconstant SELN_FN_ERROR 0)
(defconstant SELN_FN_STOP 1)
(defconstant SELN_FN_AGAIN 2)
(defconstant SELN_FN_PROPS 3)
(defconstant SELN_FN_UNDO 4)
(defconstant SELN_FN_FRONT 5)
(defconstant SELN_FN_PUT 6)
(defconstant SELN_FN_OPEN 7)
(defconstant SELN_FN_GET 8)
(defconstant SELN_FN_FIND 9)
(defconstant SELN_FN_DELETE 10)
(defconstant SELN_NONE 0)
(defconstant SELN_EXISTS 1)
(defconstant SELN_FILE 2)
(defconstant SELN_IGNORE 0)
(defconstant SELN_REQUEST 1)
(defconstant SELN_FIND 2)
(defconstant SELN_SHELVE 3)
(defconstant SELN_DELETE 4)
(defconstant PANEL_CLEAR 0)
(defconstant PANEL_NO_CLEAR 1)
(defconstant PANEL_NONE 2)
(defconstant PANEL_ALL 3)
(defconstant PANEL_NON_PRINTABLE 4)
(defconstant PANEL_SPECIFIED 5)
(defconstant PANEL_CURRENT 6)
(defconstant PANEL_DONE 7)
(defconstant PANEL_MARKED 8)
(defconstant PANEL_VERTICAL 9)
(defconstant PANEL_HORIZONTAL 10)
(defconstant PANEL_INVERTED 11)
(defconstant PANEL_INSERT 12)
(defconstant PANEL_NEXT 13)
(defconstant PANEL_PREVIOUS 14)
(defconstant PANEL_NONE_DOWN 15)
(defconstant PANEL_LEFT_DOWN 16)
(defconstant PANEL_MIDDLE_DOWN 17)
(defconstant PANEL_RIGHT_DOWN 18)
(defconstant PANEL_CHORD_DOWN 19)
(defconstant TTYOPT_PAGEMODE 1)
(defconstant TTYOPT_SELSVC 3)
(defconstant TTYOPT_TEXT 4)
(defconstant TTYSW_BOLD_NONE 0)
(defconstant TTYSW_BOLD_OFFSET_X 1)
(defconstant TTYSW_BOLD_OFFSET_Y 2)
(defconstant TTYSW_BOLD_OFFSET_XY 4)
(defconstant TTYSW_BOLD_INVERT 8)
(defconstant TTYSW_BOLD_MAX 8)
(defconstant TEXTSW_STATUS_OKAY 0)
(defconstant TEXTSW_STATUS_OTHER_ERROR 1)
(defconstant TEXTSW_STATUS_CANNOT_ALLOCATE 2)
(defconstant TEXTSW_STATUS_CANNOT_OPEN_INPUT 3)
(defconstant TEXTSW_STATUS_BAD_ATTR 4)
(defconstant TEXTSW_STATUS_BAD_ATTR_VALUE 5)
(defconstant TEXTSW_IF_AUTO_SCROLL 3)
(defconstant TEXTSW_CLIP 4)
(defconstant TEXTSW_WRAP_AT_CHAR 5)
(defconstant TEXTSW_WRAP_AT_LINE 7)
(defconstant WIN_NULL_VALUE 0)
(defconstant WIN_NO_EVENTS 1)
(defconstant WIN_UP_EVENTS 2)
(defconstant WIN_ASCII_EVENTS 3)
(defconstant WIN_UP_ASCII_EVENTS 4)
(defconstant WIN_MOUSE_BUTTONS 5)
(defconstant WIN_IN_TRANSIT_EVENTS 6)
(defconstant WIN_LEFT_KEYS 7)
(defconstant WIN_TOP_KEYS 8)
(defconstant WIN_RIGHT_KEYS 9)
(defconstant WIN_CREATE  0)
(defconstant WIN_INSTALL 13)
(defconstant WIN_INSERT 1)
(defconstant WIN_REMOVE 2)
(defconstant WIN_DESTROY 3)
(defconstant WIN_ADJUST_RECT  6)
(defconstant WIN_GET_X  7)
(defconstant WIN_GET_Y  8)
(defconstant WIN_GET_WIDTH  9)
(defconstant WIN_GET_HEIGHT 10)
(defconstant WIN_GET_RECT  11)
(defconstant WIN_LAYOUT 12)
(defconstant NULL 0)
(defconstant TRUE 1)
(defconstant FALSE 0)
(defconstant LOC_MOVE 32512)
(defconstant LOC_WINENTER 32513)
(defconstant LOC_WINEXIT 32514)
(defconstant LOC_MOVEWHILEBUTDOWN 32515)
(defconstant LOC_DRAG 32515)
(defconstant WIN_REPAINT 32516)
(defconstant WIN_RESIZE 32517)
(defconstant WIN_STOP 32573)
(defconstant KBD_USE 32520)
(defconstant KBD_DONE 32521)
(defconstant IM_NEGEVENT 1)
(defconstant IM_POSASCII 2)
(defconstant IM_ANSI 4)
(defconstant IM_UNENCODED 8)
(defconstant IM_ASCII 16)
(defconstant IM_META 32)
(defconstant IM_NEGASCII 64)
(defconstant IM_NEGMETA 128)
(defconstant IM_TOP 256)
(defconstant IM_NEGTOP 512)
(defconstant IM_INTRANSIT 1024)
(defconstant INPUTMASK_NULL 0)
(defconstant IE_NEGEVENT 1)
(defconstant CAPSLOCK 0)
(defconstant CAPSMASK 1)
(defconstant SHIFTLOCK 1)
(defconstant LEFTSHIFT 2)
(defconstant RIGHTSHIFT 3)
(defconstant SHIFTMASK 14)
(defconstant LEFTCTRL 4)
(defconstant RIGHTCTRL 5)
(defconstant CTRLMASK 48)
(defconstant META_SHIFT_MASK 64)
(defconstant EVENT_NULL 0)
(defconstant WS_ORDER_LMR 0)
(defconstant WS_ORDER_LRM 1)
(defconstant WS_ORDER_MLR 2)
(defconstant WS_ORDER_MRL 3)
(defconstant WS_ORDER_RLM 4)
(defconstant WS_ORDER_RML 5)
(defconstant WS_SCALE_MAX_COUNT 16)
(defconstant WS_SCALE_MAX_CEILING 65535)
(defconstant VUID_SEG_SIZE 256)
(defconstant ASCII_DEVID 0)
(defconstant TOP_DEVID 1)
(defconstant PANEL_DEVID 125)
(defconstant WORKSTATION_DEVID 127)
(defconstant LAST_DEVID 255)
(defconstant ASCII_FIRST 0)
(defconstant ASCII_LAST 127)
(defconstant META_FIRST 128)
(defconstant META_LAST 255)
(defconstant TOP_FIRST 256)
(defconstant TOP_LAST 511)
(defconstant VKEY_FIRST 32512)
(defconstant VKEY_UP 0)
(defconstant VKEY_DOWN 1)
(defconstant VKEY_KBD_CODES 128)
(defconstant VKEY_FIRSTPSEUDO 32512)
(defconstant VKEY_LASTPSEUDO 32546)
(defconstant VKEY_FIRSTSHIFT 32547)
(defconstant SHIFT_CAPSLOCK 32547)
(defconstant SHIFT_LOCK 32548)
(defconstant SHIFT_LEFT 32549)
(defconstant SHIFT_RIGHT 32550)
(defconstant SHIFT_LEFTCTRL 32551)
(defconstant SHIFT_CTRL 32551)
(defconstant SHIFT_RIGHTCTRL 32552)
(defconstant SHIFT_META 32553)
(defconstant SHIFT_TOP 32554)
(defconstant SHIFT_CMD 32555)
(defconstant VKEY_LASTSHIFT 32562)
(defconstant VKEY_FIRSTFUNC 32563)
(defconstant BUT_FIRST 32563)
(defconstant BUT_LAST 32572)
(defconstant KEY_LEFTFIRST 32573)
(defconstant KEY_LEFTLAST 32588)
(defconstant KEY_RIGHTFIRST 32589)
(defconstant KEY_RIGHTLAST 32604)
(defconstant KEY_TOPFIRST 32605)
(defconstant KEY_TOPLAST 32620)
(defconstant KEY_BOTTOMLEFT 32621)
(defconstant KEY_BOTTOMRIGHT 32622)
(defconstant VKEY_LASTFUNC 32664)
(defconstant VKEY_LAST 32639)
(defconstant MOUSE_DEVID 127)
(defconstant VLOC_FIRST 32640)
(defconstant LOC_FIRST_DELTA 32640)
(defconstant LOC_X_DELTA 32640)
(defconstant LOC_Y_DELTA 32641)
(defconstant LOC_LAST_DELTA 32641)
(defconstant LOC_FIRST_ABSOLUTE 32642)
(defconstant LOC_X_ABSOLUTE 32642)
(defconstant LOC_Y_ABSOLUTE 32643)
(defconstant LOC_LAST_ABSOLUTE 32643)
(defconstant VLOC_BATCH 32644)
(defconstant VLOC_LAST 32645)
(defconstant MS_LEFT 32563)
(defconstant MS_MIDDLE 32564)
(defconstant MS_RIGHT 32565)
(defconstant FE_PAIR_NONE 0)
(defconstant FE_PAIR_SET 1)
(defconstant FE_PAIR_DELTA 2)
(defconstant FE_PAIR_ABSOLUTE 3)
(defconstant FIRM_EVENT_NULL 0)
(defconstant VUID_NATIVE 0)
(defconstant VUID_FIRM_EVENT 1)
