#include <stdio.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/canvas.h>
#include <xview/tty.h>
#include <xview/ttysw.h>
#include <xview/textsw.h>
/*
#include <xview/menu.h>
*/
#include <xview/scrollbar.h>
#include <xview/seln.h>

#include <xview/pixwin.h>
#include <xview/notify.h>

iabs(v)
     int v;
{
  if (v < 0) return(-v);
  else return(v);
}
/*
pr_const(name,v)
     char *name;
     int v;
{
  if (iabs(v) > 0x1fffffff)
    printf("(defconstant %s #i(#x%x #x%x))\n",name,
	   ((v >> 16) & 0xffff), (v & 0xffff));
  else 
    printf("(defconstant %s #x%x)\n",name,v);
}
*/
pr_const(name,v)
     char *name;
     int v;
{
  if (iabs(v) > 0x1fffffff)
    printf("(defconstant %s #i(%d)) ; #x%x\n",name, v, v);
  else 
    printf("(defconstant %s %d)\n",name,v);
}

main()
{
  pr_const("XV_XID",XV_XID			);
  pr_const("XV_DISPLAY",XV_DISPLAY);
  pr_const("XV_X",XV_X			);
  pr_const("XV_Y",XV_Y			);
  pr_const("XV_WIDTH",XV_WIDTH			);
  pr_const("XV_HEIGHT",XV_HEIGHT			);
  pr_const("XV_VISUAL",XV_VISUAL);

  printf("(defconstant ITIMER_REAL 0)\n");
  printf("(defconstant ITIMER_VIRTUAL 1)\n");
  printf("(defconstant NOTIFY_POLLING_ITIMER (make-string 16))\n");
  printf("(store-bytes NOTIFY_POLLING_ITIMER  0   0 0 0 0 0 0 0 1 0 0 0 0 0 0 0 1)\n");
  printf("(defconstant PIX_SRC (* #xC 2))\n");
  printf("(defconstant PIX_DST (* #xA 2))\n");
  printf("(defun PIX_NOT (op) (logand (* #xf 2) (lognot op)))\n");
  printf("(defconstant PIX_CLR (logand PIX_SRC (pix_not pix_src)))\n");
  printf("(defconstant PIX_SET (logior PIX_SRC (pix_not pix_src)))\n");
  printf("(defun PIX_COLOR (c) (* c 32))\n");
  printf("(defun PIX_OPCOLOR (op) (/ op 32))\n");
  printf("(defun PIXOP_NEEDS_DST (op)\n");
  printf("(logand (logxor op (/ op 2)) (pix_not pix_dst)))\n");
  printf("(defun PIXOP_NEEDS_SRC (op)\n");
  printf("  (logand (logxor op (/ op 4)) (pix_not pix_src)))\n");
  printf("(defconstant PIX_DONTCLIP	#x1)\n");
  printf("(defconstant PIX_CLIP		#x0)\n");

  pr_const("NOTIFY_DONE",NOTIFY_DONE);
  pr_const("NOTIFY_IGNORED",NOTIFY_IGNORED);
  pr_const("NOTIFY_UNEXPECTED",NOTIFY_UNEXPECTED);
  pr_const("NOTIFY_OK",NOTIFY_OK);
  pr_const("NOTIFY_UNKNOWN_CLIENT",NOTIFY_UNKNOWN_CLIENT);
  pr_const("NOTIFY_NO_CONDITION",NOTIFY_NO_CONDITION);
  pr_const("NOTIFY_BAD_ITIMER",NOTIFY_BAD_ITIMER);
  pr_const("NOTIFY_BAD_SIGNAL",NOTIFY_BAD_SIGNAL);
  pr_const("NOTIFY_NOT_STARTED",NOTIFY_NOT_STARTED);
  pr_const("NOTIFY_DESTROY_VETOED",NOTIFY_DESTROY_VETOED);
  pr_const("NOTIFY_INTERNAL_ERROR",NOTIFY_INTERNAL_ERROR);
  pr_const("NOTIFY_SRCH",NOTIFY_SRCH);
  pr_const("NOTIFY_BADF",NOTIFY_BADF);
  pr_const("NOTIFY_NOMEM",NOTIFY_NOMEM);
  pr_const("NOTIFY_INVAL",NOTIFY_INVAL);
#ifdef sun3
  pr_const("NOTIFY_FUNC_LIMIT",NOTIFY_FUNC_LIMIT);
#endif
  pr_const("NOTIFY_SYNC",NOTIFY_SYNC);
  pr_const("NOTIFY_ASYNC",NOTIFY_ASYNC);
  pr_const("NOTIFY_SAFE",NOTIFY_SAFE);
  pr_const("NOTIFY_IMMEDIATE",NOTIFY_IMMEDIATE);
  pr_const("DESTROY_PROCESS_DEATH",DESTROY_PROCESS_DEATH);
  pr_const("DESTROY_CHECKING",DESTROY_CHECKING);
  pr_const("DESTROY_CLEANUP",DESTROY_CLEANUP);
  pr_const("ATTR_BASE_INT",ATTR_BASE_INT		);

  pr_const("ATTR_BASE_X",ATTR_BASE_X			);
  pr_const("ATTR_BASE_INDEX_X",ATTR_BASE_INDEX_X		);
  pr_const("ATTR_BASE_Y",ATTR_BASE_Y			);
  pr_const("ATTR_BASE_INDEX_Y",ATTR_BASE_INDEX_Y		);
  pr_const("ATTR_BASE_XY",ATTR_BASE_XY		);
  pr_const("ATTR_BASE_INDEX_XY",ATTR_BASE_INDEX_XY		);
  pr_const("ATTR_BASE_BOOLEAN",ATTR_BASE_BOOLEAN		);
  pr_const("ATTR_BASE_ENUM",ATTR_BASE_ENUM		);
  pr_const("ATTR_BASE_CHAR",ATTR_BASE_CHAR		);
  pr_const("ATTR_BASE_STRING",ATTR_BASE_STRING		);
  pr_const("ATTR_BASE_PIXRECT_PTR",ATTR_BASE_PIXRECT_PTR	);
  pr_const("ATTR_BASE_PIXFONT_PTR",ATTR_BASE_PIXFONT_PTR	);
  pr_const("ATTR_BASE_RECT_PTR",ATTR_BASE_RECT_PTR		);
  pr_const("ATTR_BASE_OPAQUE",ATTR_BASE_OPAQUE		);
  pr_const("ATTR_BASE_NO_VALUE",ATTR_BASE_NO_VALUE		);
  pr_const("ATTR_BASE_AV",ATTR_BASE_AV		);
  pr_const("ATTR_BASE_FUNCTION_PTR",ATTR_BASE_FUNCTION_PTR	);
  pr_const("ATTR_BASE_ICON_PTR",ATTR_BASE_ICON_PTR		);
  pr_const("ATTR_BASE_SINGLE_COLOR_PTR",ATTR_BASE_SINGLE_COLOR_PTR	);

  pr_const("ATTR_BASE_LONG",ATTR_BASE_LONG		);
  pr_const("ATTR_BASE_SHORT",ATTR_BASE_SHORT		);
  pr_const("ATTR_INT",ATTR_INT			);
  pr_const("ATTR_INT_PAIR",ATTR_INT_PAIR		);
  pr_const("ATTR_INT_TRIPLE",ATTR_INT_TRIPLE		);
  pr_const("ATTR_X",ATTR_X			);
  pr_const("ATTR_INDEX_X",ATTR_INDEX_X		);
  pr_const("ATTR_Y",ATTR_Y			);
  pr_const("ATTR_INDEX_Y",ATTR_INDEX_Y		);
  pr_const("ATTR_XY",ATTR_XY			);
  pr_const("ATTR_INDEX_XY",ATTR_INDEX_XY		);
  pr_const("ATTR_BOOLEAN",ATTR_BOOLEAN		);
  pr_const("ATTR_ENUM",ATTR_ENUM			);
  pr_const("ATTR_CHAR",ATTR_CHAR			);
  pr_const("ATTR_STRING",ATTR_STRING			);
  pr_const("ATTR_PIXRECT_PTR",ATTR_PIXRECT_PTR		);
  pr_const("ATTR_PIXFONT_PTR",ATTR_PIXFONT_PTR		);
  pr_const("ATTR_PIXWIN_PTR",ATTR_PIXWIN_PTR		);
  pr_const("ATTR_RECT_PTR",ATTR_RECT_PTR		);
  pr_const("ATTR_OPAQUE",ATTR_OPAQUE			);
  pr_const("ATTR_NO_VALUE",ATTR_NO_VALUE		);
  pr_const("ATTR_AV",ATTR_AV			);
  pr_const("ATTR_FUNCTION_PTR",ATTR_FUNCTION_PTR		);
  pr_const("ATTR_ICON_PTR",ATTR_ICON_PTR		);
  pr_const("ATTR_SINGLE_COLOR_PTR",ATTR_SINGLE_COLOR_PTR	);
  pr_const("ATTR_LONG",ATTR_LONG			);
  pr_const("ATTR_SHORT",ATTR_SHORT			);
  pr_const("ATTR_LIST_IS_INLINE",ATTR_LIST_IS_INLINE	);
  pr_const("ATTR_LIST_IS_PTR",ATTR_LIST_IS_PTR	);
  pr_const("ATTR_NONE",ATTR_NONE		);
  pr_const("ATTR_RECURSIVE",ATTR_RECURSIVE	);
  pr_const("ATTR_NULL",ATTR_NULL		);
  pr_const("ATTR_COUNTED",ATTR_COUNTED	);
  pr_const("ATTR_PKG_ZERO",ATTR_PKG_ZERO	);
  pr_const("ATTR_PKG_GENERIC",ATTR_PKG_GENERIC	);
  pr_const("ATTR_PKG_PANEL",ATTR_PKG_PANEL	);
  pr_const("ATTR_PKG_SCROLLBAR",ATTR_PKG_SCROLLBAR	);
  pr_const("ATTR_PKG_CURSOR",ATTR_PKG_CURSOR	);
  pr_const("ATTR_PKG_MENU",ATTR_PKG_MENU	);
  pr_const("ATTR_PKG_TEXTSW",ATTR_PKG_TEXTSW	);
  pr_const("ATTR_PKG_SELN_BASE",ATTR_PKG_SELN_BASE	);
  pr_const("ATTR_PKG_ENTITY",ATTR_PKG_ENTITY	);
  pr_const("ATTR_PKG_IMAGE",ATTR_PKG_IMAGE	);
  pr_const("ATTR_PKG_WIN",ATTR_PKG_WIN	);
  pr_const("ATTR_PKG_FRAME",ATTR_PKG_FRAME	);
  pr_const("ATTR_PKG_CANVAS",ATTR_PKG_CANVAS	);
  pr_const("ATTR_PKG_TTY",ATTR_PKG_TTY	);
  pr_const("ATTR_PKG_ICON",ATTR_PKG_ICON	);
  pr_const("ATTR_PKG_LAST",ATTR_PKG_LAST	);
  pr_const("ATTR_LIST",ATTR_LIST );
  pr_const("ATTR_CU_POSITION",ATTR_CU_POSITION	);
  pr_const("ATTR_CU_LENGTH",ATTR_CU_LENGTH	);
  pr_const("VUID_ASCII",VUID_ASCII );
  pr_const("VUID_TOP",VUID_TOP );
  pr_const("VUID_PANEL",VUID_PANEL );
  pr_const("VUID_SCROLL",VUID_SCROLL );
  pr_const("VUID_WORKSTATION",VUID_WORKSTATION );
  pr_const("VUID_LAST",VUID_LAST );
  pr_const("ICON_WIDTH",XV_WIDTH		);
  pr_const("ICON_HEIGHT",XV_HEIGHT		);
  pr_const("ICON_IMAGE_RECT",ICON_IMAGE_RECT	);
  pr_const("ICON_IMAGE",ICON_IMAGE		);
  pr_const("ICON_LABEL",XV_LABEL		);
  pr_const("ICON_FONT",XV_FONT		);
  pr_const("MENU_ACTION_PROC",MENU_ACTION_PROC		);
  pr_const("MENU_ACTION",MENU_ACTION			);
  pr_const("MENU_ACTION_IMAGE",MENU_ACTION_IMAGE		);
  pr_const("MENU_ACTION_ITEM",MENU_ACTION_ITEM		);
  pr_const("MENU_APPEND_ITEM",MENU_APPEND_ITEM		);
  pr_const("MENU_CLIENT_DATA",MENU_CLIENT_DATA		);
  pr_const("MENU_DEFAULT",MENU_DEFAULT		);
  pr_const("MENU_DEFAULT_ITEM",MENU_DEFAULT_ITEM		);
  pr_const("MENU_DEFAULT_IMAGE",MENU_DEFAULT_IMAGE		);
  pr_const("MENU_DISABLE_ITEM",MENU_DISABLE_ITEM		);
  pr_const("MENU_DEFAULT_SELECTION",MENU_DEFAULT_SELECTION	);
  pr_const("MENU_DESCEND_FIRST",MENU_DESCEND_FIRST		);
  pr_const("MENU_FEEDBACK",MENU_FEEDBACK		);
  pr_const("MENU_FIRST_EVENT",MENU_FIRST_EVENT		);
  pr_const("MENU_GEN_PULLRIGHT",MENU_GEN_PULLRIGHT		);
  pr_const("MENU_GEN_PULLRIGHT_IMAGE",MENU_GEN_PULLRIGHT_IMAGE	);
  pr_const("MENU_GEN_PULLRIGHT_ITEM",MENU_GEN_PULLRIGHT_ITEM	);
  pr_const("MENU_GEN_PROC",MENU_GEN_PROC		);
  pr_const("MENU_GEN_PROC_IMAGE",MENU_GEN_PROC_IMAGE		);
  pr_const("MENU_GEN_PROC_ITEM",MENU_GEN_PROC_ITEM		);
  pr_const("MENU_HEIGHT",XV_HEIGHT			);
  pr_const("MENU_IMAGE",MENU_IMAGE			);
  pr_const("MENU_IMAGE_ITEM",MENU_IMAGE_ITEM		);
  pr_const("MENU_IMAGES",MENU_IMAGES			);
  pr_const("MENU_INITIAL_SELECTION",MENU_INITIAL_SELECTION	);
#ifdef sun3
  pr_const("MENU_ACCELERATED_SELECTION",MENU_ACCELERATED_SELECTION	);
  pr_const("MENU_DISPLAY_ONE_LEVEL",MENU_DISPLAY_ONE_LEVEL	);
#endif
  pr_const("MENU_INITIAL_SELECTION_SELECTED",MENU_INITIAL_SELECTION_SELECTED );
  pr_const("MENU_INITIAL_SELECTION_EXPANDED",MENU_INITIAL_SELECTION_EXPANDED );
  pr_const("MENU_INACTIVE",MENU_INACTIVE		);
  pr_const("MENU_INSERT",MENU_INSERT			);
  pr_const("MENU_INSERT_ITEM",MENU_INSERT_ITEM		);
  pr_const("MENU_ITEM",MENU_ITEM			);
  pr_const("MENU_INVERT",MENU_INVERT			);
  pr_const("MENU_JUMP_AFTER_SELECTION",MENU_JUMP_AFTER_SELECTION	);
  pr_const("MENU_JUMP_AFTER_NO_SELECTION",MENU_JUMP_AFTER_NO_SELECTION );
  pr_const("MENU_LAST_EVENT",MENU_LAST_EVENT		);
  pr_const("MENU_LEFT_MARGIN",XV_LEFT_MARGIN		);
  pr_const("MENU_MARGIN",XV_MARGIN			);
  pr_const("MENU_MENU",MENU_MENU			);
  pr_const("MENU_NITEMS",MENU_NITEMS			);
  pr_const("MENU_NOTIFY_PROC",MENU_NOTIFY_PROC		);
  pr_const("MENU_NTH_ITEM",MENU_NTH_ITEM		);
  pr_const("MENU_NCOLS",MENU_NCOLS			);
  pr_const("MENU_NROWS",MENU_NROWS			);
  pr_const("MENU_OWNER",XV_OWNER			);
  pr_const("MENU_PULLRIGHT",MENU_PULLRIGHT		);
  pr_const("MENU_PULLRIGHT_DELTA",MENU_PULLRIGHT_DELTA	);
  pr_const("MENU_PULLRIGHT_IMAGE",MENU_PULLRIGHT_IMAGE	);
  pr_const("MENU_PULLRIGHT_ITEM",MENU_PULLRIGHT_ITEM		);
  pr_const("MENU_RELEASE",MENU_RELEASE		);
  pr_const("MENU_RELEASE_IMAGE",MENU_RELEASE_IMAGE		);
  pr_const("MENU_REMOVE",MENU_REMOVE			);
  pr_const("MENU_REMOVE_ITEM",MENU_REMOVE_ITEM		);
  pr_const("MENU_REPLACE",MENU_REPLACE		);
  pr_const("MENU_REPLACE_ITEM",MENU_REPLACE_ITEM		);
  pr_const("MENU_RIGHT_MARGIN",XV_RIGHT_MARGIN		);
  pr_const("MENU_SHADOW",MENU_SHADOW			);
  pr_const("MENU_STRING",MENU_STRING			);
  pr_const("MENU_STRING_ITEM",MENU_STRING_ITEM		);
  pr_const("MENU_STRINGS",MENU_STRINGS		);
  pr_const("MENU_TITLE_ITEM",MENU_TITLE_ITEM		);
  pr_const("MENU_TITLE_IMAGE",MENU_TITLE_IMAGE		);
  pr_const("MENU_TYPE",MENU_TYPE			);
  pr_const("MENU_VALID_RESULT",MENU_VALID_RESULT		);
  pr_const("MENU_VALUE",MENU_VALUE			);
  pr_const("MENU_WIDTH",XV_WIDTH			);
  pr_const("MENU_BUTTON",MENU_BUTTON			);
  pr_const("MENU_FD",MENU_FD			);
  pr_const("MENU_IE",MENU_IE			);
  pr_const("MENU_POS",MENU_POS			);
  pr_const("WIN_ERROR_MSG",WIN_ERROR_MSG		);
  pr_const("WIN_X",WIN_X			);
  pr_const("WIN_Y",WIN_Y			);
  pr_const("WIN_RIGHT_OF",WIN_RIGHT_OF		);
  pr_const("WIN_BELOW",WIN_BELOW			);
  pr_const("WIN_WIDTH",WIN_WIDTH			);
  pr_const("WIN_HEIGHT",WIN_HEIGHT			);
  pr_const("WIN_PERCENT_WIDTH",WIN_PERCENT_WIDTH		);
  pr_const("WIN_PERCENT_HEIGHT",WIN_PERCENT_HEIGHT		);
  pr_const("WIN_COLUMNS",WIN_COLUMNS			);
  pr_const("WIN_ROWS",WIN_ROWS			);
  pr_const("WIN_RECT",WIN_RECT			);
  pr_const("WIN_SCREEN_RECT",WIN_SCREEN_RECT		);
  pr_const("WIN_FIT_HEIGHT",WIN_FIT_HEIGHT		);
  pr_const("WIN_FIT_WIDTH",WIN_FIT_WIDTH		);
  pr_const("WIN_SHOW",WIN_SHOW			);
  pr_const("WIN_MENU",WIN_MENU			);
  pr_const("WIN_CURSOR",WIN_CURSOR			);
  pr_const("WIN_EVENT_PROC",WIN_EVENT_PROC		);
  pr_const("WIN_GRAB_ALL_INPUT",WIN_GRAB_ALL_INPUT		);

  pr_const("WIN_KBD_INPUT_MASK",WIN_INPUT_MASK);
  pr_const("WIN_CONSUME_KBD_EVENT",WIN_CONSUME_EVENT	);
  pr_const("WIN_IGNORE_KBD_EVENT",WIN_IGNORE_EVENT	);
  pr_const("WIN_CONSUME_KBD_EVENTS",WIN_CONSUME_EVENTS	);
  pr_const("WIN_IGNORE_KBD_EVENTS",WIN_IGNORE_EVENTS	);
  pr_const("WIN_PICK_INPUT_MASK",WIN_INPUT_MASK);
  pr_const("WIN_CONSUME_PICK_EVENT",WIN_CONSUME_EVENT	);
  pr_const("WIN_IGNORE_PICK_EVENT",WIN_IGNORE_EVENT	);
  pr_const("WIN_CONSUME_PICK_EVENTS",WIN_CONSUME_EVENTS	);
  pr_const("WIN_IGNORE_PICK_EVENTS",WIN_IGNORE_EVENTS	);

  pr_const("WIN_FONT",XV_FONT			);
  pr_const("WIN_TOP_MARGIN",XV_TOP_MARGIN		);
  pr_const("WIN_BOTTOM_MARGIN",XV_BOTTOM_MARGIN		);
  pr_const("WIN_LEFT_MARGIN",XV_LEFT_MARGIN		);
  pr_const("WIN_RIGHT_MARGIN",XV_RIGHT_MARGIN		);
  pr_const("WIN_ROW_HEIGHT",WIN_ROW_HEIGHT		);
  pr_const("WIN_COLUMN_WIDTH",WIN_COLUMN_WIDTH		);
  pr_const("WIN_ROW_GAP",WIN_ROW_GAP			);
  pr_const("WIN_COLUMN_GAP",WIN_COLUMN_GAP		);
  pr_const("WIN_TYPE",WIN_TYPE			);
  pr_const("WIN_OWNER",XV_OWNER			);
  pr_const("WIN_NAME",XV_NAME			);
  pr_const("WIN_CLIENT_DATA",WIN_CLIENT_DATA		);
  pr_const("WIN_XNAME",XV_XNAME		);

  pr_const("WIN_DEVICE_NUMBER",WIN_DEVICE_NUMBER		);
  pr_const("WIN_DYNAMIC_VISUAL",WIN_DYNAMIC_VISUAL		);
  pr_const("WIN_PIXWIN",WIN_PIXWIN			);
  pr_const("WIN_MOUSE_XY",WIN_MOUSE_XY		);
/*
  pr_const("WIN_PRESET_PROC",WIN_PRESET_PROC		);
  pr_const("WIN_POSTSET_PROC",WIN_POSTSET_PROC		);
  pr_const("WIN_LAYOUT_PROC",WIN_LAYOUT_PROC		);
  pr_const("WIN_NOTIFY_DESTROY_PROC",WIN_NOTIFY_DESTROY_PROC	);
  pr_const("WIN_DEFAULT_EVENT_PROC",WIN_DEFAULT_EVENT_PROC	);
  pr_const("WIN_CREATED",WIN_CREATED			);
  pr_const("WIN_COMPATIBILITY",WIN_COMPATIBILITY		);
  pr_const("WIN_COMPATIBILITY_INFO",WIN_COMPATIBILITY_INFO	);
  pr_const("WIN_REGISTER",WIN_REGISTER		);
  pr_const("WIN_NOTIFY_INFO",WIN_NOTIFY_INFO		);

  pr_const("FRAME_CLOSED_X",FRAME_CLOSED_X		);
  pr_const("FRAME_CLOSED_Y",FRAME_CLOSED_Y		);
  pr_const("FRAME_BORDER_STYLE",FRAME_BORDER_STYLE		);
  pr_const("FRAME_NULL_ATTR",FRAME_NULL_ATTR		);
*/

  pr_const("WIN_NOTIFY_EVENT_PROC",WIN_NOTIFY_EVENT_PROC 	);
  pr_const("WIN_KBD_FOCUS",WIN_KBD_FOCUS		);
  pr_const("WIN_EVENT_STATE",WIN_EVENT_STATE		);

  pr_const("FRAME_CLOSED",FRAME_CLOSED		);
  pr_const("FRAME_INHERIT_COLORS",FRAME_INHERIT_COLORS	);
  pr_const("FRAME_SHOW_LABEL",FRAME_SHOW_LABEL		);
  pr_const("FRAME_LABEL",XV_LABEL			);
  pr_const("FRAME_FOREGROUND_COLOR",FRAME_FOREGROUND_COLOR	);
  pr_const("FRAME_BACKGROUND_COLOR",FRAME_BACKGROUND_COLOR	);
  pr_const("FRAME_ICON",FRAME_ICON			);

  pr_const("FRAME_CLOSED_RECT",FRAME_CLOSED_RECT		);
  pr_const("FRAME_CURRENT_RECT",FRAME_CURRENT_RECT		);
  pr_const("FRAME_OPEN_RECT",FRAME_OPEN_RECT		);
  pr_const("FRAME_CMDLINE_HELP_PROC",FRAME_CMDLINE_HELP_PROC	);

  pr_const("FRAME_NTH_SUBWINDOW",FRAME_NTH_SUBWINDOW		);
  pr_const("FRAME_NTH_SUBFRAME",FRAME_NTH_SUBFRAME		);
  pr_const("FRAME_DONE_PROC",FRAME_DONE_PROC		);
  pr_const("FRAME_DEFAULT_DONE_PROC",FRAME_DEFAULT_DONE_PROC	);
  pr_const("FRAME_NO_CONFIRM",FRAME_NO_CONFIRM		);

  pr_const("PANEL_ITEM_X",XV_X		);
  pr_const("PANEL_ITEM_Y",XV_Y		);

  pr_const("PANEL_LABEL_X",PANEL_LABEL_X		);
  pr_const("PANEL_VALUE_X",PANEL_VALUE_X		);
  pr_const("PANEL_SLIDER_WIDTH",PANEL_SLIDER_WIDTH		);
  pr_const("PANEL_WIDTH",XV_WIDTH			);
  pr_const("PANEL_CHOICE_X",PANEL_CHOICE_X		);
  pr_const("PANEL_MARK_X",PANEL_MARK_X		);
  pr_const("PANEL_CHOICE_XS",PANEL_CHOICE_XS	);
  pr_const("PANEL_MARK_XS",PANEL_MARK_XS	);
  pr_const("PANEL_X",XV_X		);
  pr_const("PANEL_Y",XV_Y		);
  pr_const("PANEL_X_GAP",XV_X		);
  pr_const("PANEL_Y_GAP",XV_Y		);

  pr_const("PANEL_LABEL_Y",PANEL_LABEL_Y		);
  pr_const("PANEL_VALUE_Y",PANEL_VALUE_Y		);
  pr_const("PANEL_HEIGHT",XV_HEIGHT		);
  pr_const("PANEL_CHOICE_Y",PANEL_CHOICE_Y		);
  pr_const("PANEL_MARK_Y",PANEL_MARK_Y		);
  pr_const("PANEL_CHOICE_YS",PANEL_CHOICE_YS	);
  pr_const("PANEL_MARK_YS",PANEL_MARK_YS	);
  pr_const("PANEL_VALUE",PANEL_VALUE			);

  pr_const("PANEL_VALUE_DISPLAY_LENGTH",PANEL_VALUE_DISPLAY_LENGTH	);
  pr_const("PANEL_VALUE_STORED_LENGTH",PANEL_VALUE_STORED_LENGTH	);
  pr_const("PANEL_MIN_VALUE",PANEL_MIN_VALUE		);
  pr_const("PANEL_MAX_VALUE",PANEL_MAX_VALUE		);
  pr_const("PANEL_CHOICE_OFFSET",PANEL_CHOICE_OFFSET		);

  pr_const("PANEL_VALUE_UNDERLINED",PANEL_VALUE_UNDERLINED	);

  pr_const("PANEL_LABEL_BOLD",PANEL_LABEL_BOLD		);
  pr_const("PANEL_CHOICES_BOLD",PANEL_CHOICES_BOLD		);
  pr_const("PANEL_SHOW",XV_SHOW		);
  pr_const("PANEL_SHOW_VALUE",PANEL_SHOW_VALUE		);
  pr_const("PANEL_SHOW_RANGE",PANEL_SHOW_RANGE		);
  pr_const("PANEL_SHOW_MENU",PANEL_SHOW_MENU		);
  pr_const("PANEL_SHOW_MENU_MARK",PANEL_SHOW_MENU_MARK	);
  pr_const("PANEL_CHOOSE_ONE",PANEL_CHOOSE_ONE		);

/*
  pr_const("PANEL_LABEL_DISPLAY_LENGTH",PANEL_LABEL_DISPLAY_LENGTH	);
  pr_const("PANEL_TIMER_SECS",PANEL_TIMER_SECS		);
  pr_const("PANEL_TIMER_USECS",PANEL_TIMER_USECS		);
  pr_const("PANEL_ITEM_BOXED",PANEL_ITEM_BOXED		);
  pr_const("PANEL_ADJUSTABLE",PANEL_ADJUSTABLE		);
  pr_const("PANEL_LABEL_SHADED",PANEL_LABEL_SHADED		);
  pr_const("PANEL_MOUSE_STATE",PANEL_MOUSE_STATE		);
  pr_const("PANEL_VALUE_STRING",PANEL_VALUE_STRING		);
  pr_const("PANEL_NAME",PANEL_NAME			);
  pr_const("PANEL_TYPE_IMAGE",PANEL_TYPE_IMAGE		);
  pr_const("PANEL_TIMER_PROC",PANEL_TIMER_PROC		);
*/
  pr_const("PANEL_BLINK_CARET",PANEL_BLINK_CARET		);
  pr_const("PANEL_ACCEPT_KEYSTROKE",PANEL_ACCEPT_KEYSTROKE	);
  pr_const("PANEL_TOGGLE_VALUE",PANEL_TOGGLE_VALUE		);
  pr_const("PANEL_NOTIFY_LEVEL",PANEL_NOTIFY_LEVEL		);
  pr_const("PANEL_DISPLAY_LEVEL",PANEL_DISPLAY_LEVEL		);
  pr_const("PANEL_LAYOUT",PANEL_LAYOUT		);
  pr_const("PANEL_FEEDBACK",PANEL_FEEDBACK		);
  pr_const("PANEL_PAINT",PANEL_PAINT			);
  pr_const("PANEL_MASK_CHAR",PANEL_MASK_CHAR		);
  pr_const("PANEL_LABEL_STRING",PANEL_LABEL_STRING		);
  pr_const("PANEL_NOTIFY_STRING",PANEL_NOTIFY_STRING		);

  pr_const("PANEL_CHOICE_STRINGS",PANEL_CHOICE_STRINGS	);

  pr_const("PANEL_CHOICE_STRING",PANEL_CHOICE_STRING		);

  pr_const("PANEL_CHOICE_IMAGES",PANEL_CHOICE_IMAGES		);
  pr_const("PANEL_LABEL_IMAGE",PANEL_LABEL_IMAGE		);
  pr_const("PANEL_MARK_IMAGES",PANEL_MARK_IMAGES		);
  pr_const("PANEL_NOMARK_IMAGES",PANEL_NOMARK_IMAGES		);
  pr_const("PANEL_CHOICE_IMAGE",PANEL_CHOICE_IMAGE		);
  pr_const("PANEL_MARK_IMAGE",PANEL_MARK_IMAGE		);
  pr_const("PANEL_NOMARK_IMAGE",PANEL_NOMARK_IMAGE		);
  pr_const("PANEL_LABEL_FONT",PANEL_LABEL_FONT		);
  pr_const("PANEL_VALUE_FONT",PANEL_VALUE_FONT		);

  pr_const("PANEL_FONT",PANEL_FONT			);
  pr_const("PANEL_CHOICE_FONTS",PANEL_CHOICE_FONTS		);

  pr_const("PANEL_NOTIFY_PROC",PANEL_NOTIFY_PROC		);

  pr_const("PANEL_EVENT_PROC",PANEL_EVENT_PROC		);
  pr_const("PANEL_BACKGROUND_PROC",PANEL_BACKGROUND_PROC	);
  pr_const("PANEL_ITEM_RECT",PANEL_ITEM_RECT		);
  pr_const("PANEL_CLIENT_DATA",PANEL_CLIENT_DATA		);
  pr_const("WIN_VERTICAL_SCROLLBAR",WIN_VERTICAL_SCROLLBAR	);
  pr_const("WIN_HORIZONTAL_SCROLLBAR",WIN_HORIZONTAL_SCROLLBAR	);

  pr_const("PANEL_CARET_ITEM",PANEL_CARET_ITEM		);
  pr_const("PANEL_FIRST_ITEM",PANEL_FIRST_ITEM		);
  pr_const("PANEL_NEXT_ITEM",PANEL_NEXT_ITEM		);
  pr_const("PANEL_PARENT_PANEL",XV_OWNER		);

  pr_const("CANVAS_WIDTH",CANVAS_WIDTH		);
  pr_const("CANVAS_HEIGHT",CANVAS_HEIGHT		);
  pr_const("CANVAS_VIEW_PAINT_WINDOW",CANVAS_VIEW_PAINT_WINDOW		);
  pr_const("CANVAS_VIEWABLE_RECT",CANVAS_VIEWABLE_RECT		);
  pr_const("CANVAS_X_PAINT_WINDOW",CANVAS_X_PAINT_WINDOW		);
  pr_const("CANVAS_VIEW_CANVAS_WINDOW",CANVAS_VIEW_CANVAS_WINDOW	);
  pr_const("CANVAS_PAINT_CANVAS_WINDOW",CANVAS_PAINT_CANVAS_WINDOW	);
  pr_const("CANVAS_NTH_PAINT_WINDOW",CANVAS_NTH_PAINT_WINDOW		);
  pr_const("CANVAS_PAINTWINDOW_ATTRS",CANVAS_PAINTWINDOW_ATTRS		);
  pr_const("CANVAS_NO_CLIPPING",CANVAS_NO_CLIPPING		);
  pr_const("CANVAS_FIXED_IMAGE",CANVAS_FIXED_IMAGE		);
  pr_const("CANVAS_REPAINT_PROC",CANVAS_REPAINT_PROC		);
  pr_const("CANVAS_RESIZE_PROC",CANVAS_RESIZE_PROC		);
  pr_const("OPENWIN_AUTO_CLEAR",OPENWIN_AUTO_CLEAR		);
  pr_const("CANVAS_AUTO_EXPAND",CANVAS_AUTO_EXPAND		);
  pr_const("CANVAS_AUTO_SHRINK",CANVAS_AUTO_SHRINK		);
  pr_const("CANVAS_FAST_MONO",CANVAS_FAST_MONO		);

  pr_const("TTY_ARGV",TTY_ARGV			);
  pr_const("TTY_CONSOLE",TTY_CONSOLE			);
  pr_const("TTY_PAGE_MODE",TTY_PAGE_MODE		);
  pr_const("TTY_QUIT_ON_CHILD_DEATH",TTY_QUIT_ON_CHILD_DEATH	);
  pr_const("TTY_BOLDSTYLE",TTY_BOLDSTYLE		);
  pr_const("TTY_BOLDSTYLE_NAME",TTY_BOLDSTYLE_NAME		);
  pr_const("TTY_PID",TTY_PID		);

  pr_const("TEXTSW_INFINITY",TEXTSW_INFINITY	);
  pr_const("TEXTSW_DELIMITER_BACKWARD",TEXTSW_DELIMITER_BACKWARD	);
  pr_const("TEXTSW_DELIMITER_FORWARD",TEXTSW_DELIMITER_FORWARD	);
  pr_const("TEXTSW_DELIMITER_ENCLOSE",TEXTSW_DELIMITER_ENCLOSE	);

  pr_const("TEXTSW_ADJUST_IS_PENDING_DELETE",TEXTSW_ADJUST_IS_PENDING_DELETE	);
  pr_const("TEXTSW_AGAIN_LIMIT",TEXTSW_AGAIN_LIMIT		);
  pr_const("TEXTSW_AGAIN_RECORDING",TEXTSW_AGAIN_RECORDING		);
  pr_const("TEXTSW_AUTO_INDENT",TEXTSW_AUTO_INDENT		);

  pr_const("TEXTSW_BLINK_CARET",TEXTSW_BLINK_CARET		);
  pr_const("TEXTSW_BROWSING",TEXTSW_BROWSING			);
  pr_const("TEXTSW_CHECKPOINT_FREQUENCY",TEXTSW_CHECKPOINT_FREQUENCY	);
  pr_const("TEXTSW_CLIENT_DATA",TEXTSW_CLIENT_DATA		);
  pr_const("TEXTSW_CONFIRM_OVERWRITE",TEXTSW_CONFIRM_OVERWRITE	);
  pr_const("TEXTSW_CONTENTS",TEXTSW_CONTENTS			);
  pr_const("TEXTSW_CONTROL_CHARS_USE_FONT",TEXTSW_CONTROL_CHARS_USE_FONT	);
  pr_const("TEXTSW_DISABLE_CD",TEXTSW_DISABLE_CD		);
  pr_const("TEXTSW_DISABLE_LOAD",TEXTSW_DISABLE_LOAD		);
  pr_const("TEXTSW_EDIT_COUNT",TEXTSW_EDIT_COUNT		);
  pr_const("TEXTSW_ERROR_MSG",TEXTSW_ERROR_MSG		);
  pr_const("TEXTSW_FILE",TEXTSW_FILE			);
  pr_const("TEXTSW_FILE_CONTENTS",TEXTSW_FILE_CONTENTS		);
  pr_const("TEXTSW_FONT",XV_FONT			);

  pr_const("TEXTSW_HISTORY_LIMIT",TEXTSW_HISTORY_LIMIT		);
  pr_const("TEXTSW_IGNORE_LIMIT",TEXTSW_IGNORE_LIMIT		);
  pr_const("TEXTSW_INSERT_MAKES_VISIBLE",TEXTSW_INSERT_MAKES_VISIBLE	);
  pr_const("TEXTSW_INSERTION_POINT",TEXTSW_INSERTION_POINT		);
  pr_const("TEXTSW_LENGTH",TEXTSW_LENGTH			);
  pr_const("TEXTSW_LOAD_DIR_IS_CD",TEXTSW_LOAD_DIR_IS_CD		);
  pr_const("TEXTSW_LOWER_CONTEXT",TEXTSW_LOWER_CONTEXT		);
  pr_const("TEXTSW_MEMORY_MAXIMUM",TEXTSW_MEMORY_MAXIMUM		);
  pr_const("TEXTSW_MODIFIED",TEXTSW_MODIFIED			);
  pr_const("TEXTSW_MULTI_CLICK_SPACE",TEXTSW_MULTI_CLICK_SPACE	);
  pr_const("TEXTSW_MULTI_CLICK_TIMEOUT",TEXTSW_MULTI_CLICK_TIMEOUT	);
  pr_const("TEXTSW_NAME_TO_USE",TEXTSW_NAME_TO_USE		);
  pr_const("TEXTSW_NO_RESET_TO_SCRATCH",TEXTSW_NO_RESET_TO_SCRATCH	);
  pr_const("TEXTSW_NO_SELECTION_SERVICE",TEXTSW_NO_SELECTION_SERVICE	);
  pr_const("TEXTSW_NOTIFY_LEVEL",TEXTSW_NOTIFY_LEVEL		);
  pr_const("TEXTSW_NOTIFY_PROC",TEXTSW_NOTIFY_PROC		);
  pr_const("TEXTSW_PIXWIN",TEXTSW_PIXWIN			);
  pr_const("TEXTSW_READ_ONLY",TEXTSW_READ_ONLY		);
  pr_const("TEXTSW_RESET_MODE",TEXTSW_RESET_MODE		);
  pr_const("TEXTSW_STATUS",TEXTSW_STATUS			);
  pr_const("TEXTSW_STORE_CHANGES_FILE",TEXTSW_STORE_CHANGES_FILE 	);
  pr_const("TEXTSW_TAB_WIDTH",TEXTSW_TAB_WIDTH		);
  pr_const("TEXTSW_TOOL",TEXTSW_TOOL			);
  pr_const("TEXTSW_UPPER_CONTEXT",TEXTSW_UPPER_CONTEXT		);
/*
  pr_const("TEXTSW_HEIGHT",TEXTSW_HEIGHT			);
  pr_const("TEXTSW_NAME",TEXTSW_NAME			);
  pr_const("TEXTSW_STORE_SELF_IS_SAVE",TEXTSW_STORE_SELF_IS_SAVE	);
  pr_const("TEXTSW_WIDTH",TEXTSW_WIDTH			);
*/
  pr_const("TEXTSW_END_ALL_VIEWS",TEXTSW_END_ALL_VIEWS		);
  pr_const("TEXTSW_FOR_ALL_VIEWS",TEXTSW_FOR_ALL_VIEWS		);
  pr_const("TEXTSW_FIRST",TEXTSW_FIRST			);
  pr_const("TEXTSW_FIRST_LINE",TEXTSW_FIRST_LINE		);
  pr_const("TEXTSW_LEFT_MARGIN",XV_LEFT_MARGIN		);
  pr_const("TEXTSW_LINE_BREAK_ACTION",TEXTSW_LINE_BREAK_ACTION	);
  pr_const("TEXTSW_RIGHT_MARGIN",XV_RIGHT_MARGIN		);
  pr_const("TEXTSW_SCROLLBAR",TEXTSW_SCROLLBAR		);
  pr_const("TEXTSW_UPDATE_SCROLLBAR",TEXTSW_UPDATE_SCROLLBAR		);
  pr_const("TEXTSW_SPARE_1",TEXTSW_SPARE_1			);
  pr_const("TEXTSW_SPARE_2",TEXTSW_SPARE_2			);
  pr_const("TEXTSW_NEVER",TEXTSW_NEVER		);
  pr_const("TEXTSW_ALWAYS",TEXTSW_ALWAYS		);
  pr_const("TEXTSW_ONLY",TEXTSW_ONLY		);
  pr_const("TEXTSW_ACTION_CAPS_LOCK",TEXTSW_ACTION_CAPS_LOCK		);
  pr_const("TEXTSW_ACTION_CHANGED_DIRECTORY",TEXTSW_ACTION_CHANGED_DIRECTORY	);
  pr_const("TEXTSW_ACTION_EDITED_FILE",TEXTSW_ACTION_EDITED_FILE	);
  pr_const("TEXTSW_ACTION_FILE_IS_READONLY",TEXTSW_ACTION_FILE_IS_READONLY	);
  pr_const("TEXTSW_ACTION_LOADED_FILE",TEXTSW_ACTION_LOADED_FILE	);
  pr_const("TEXTSW_ACTION_SAVING_FILE",TEXTSW_ACTION_SAVING_FILE	);
  pr_const("TEXTSW_ACTION_STORING_FILE",TEXTSW_ACTION_STORING_FILE	);
  pr_const("TEXTSW_ACTION_USING_MEMORY",TEXTSW_ACTION_USING_MEMORY	);
  pr_const("TEXTSW_ACTION_WRITE_FAILED",TEXTSW_ACTION_WRITE_FAILED	);
  pr_const("TEXTSW_ACTION_TOOL_CLOSE",TEXTSW_ACTION_TOOL_CLOSE	);
  pr_const("TEXTSW_ACTION_TOOL_DESTROY",TEXTSW_ACTION_TOOL_DESTROY	);
  pr_const("TEXTSW_ACTION_TOOL_MGR",TEXTSW_ACTION_TOOL_MGR		);
  pr_const("TEXTSW_ACTION_TOOL_QUIT",TEXTSW_ACTION_TOOL_QUIT		);
  pr_const("TEXTSW_ACTION_REPLACED",TEXTSW_ACTION_REPLACED		);
  pr_const("TEXTSW_ACTION_PAINTED",TEXTSW_ACTION_PAINTED		);
  pr_const("TEXTSW_ACTION_SCROLLED",TEXTSW_ACTION_SCROLLED		);
  pr_const("TEXTSW_ACTION_DESTROY_VIEW",TEXTSW_ACTION_DESTROY_VIEW	);
  pr_const("TEXTSW_ACTION_SPLIT_VIEW",TEXTSW_ACTION_SPLIT_VIEW	);
  pr_const("TEXTSW_FATTR_INPUT",TEXTSW_FATTR_INPUT		);
  pr_const("TEXTSW_FATTR_INPUT_EVENT",TEXTSW_FATTR_INPUT_EVENT	);
  pr_const("TEXTSW_FATTR_INSERTION_POINTS",TEXTSW_FATTR_INSERTION_POINTS	);
  pr_const("TEXTSW_FATTR_INSERTION_LINE",TEXTSW_FATTR_INSERTION_LINE	);
  pr_const("TEXTSW_FATTR_SELECTION_ENDPOINTS",TEXTSW_FATTR_SELECTION_ENDPOINTS);
  pr_const("TEXTSW_FILTER_DELETE_RANGE",TEXTSW_FILTER_DELETE_RANGE	);
  pr_const("TEXTSW_FILTER_INSERT",TEXTSW_FILTER_INSERT		);
  pr_const("TEXTSW_FILTER_SEND_RANGE",TEXTSW_FILTER_SEND_RANGE	);
  pr_const("TEXTSW_FILTER_SET_INSERTION",TEXTSW_FILTER_SET_INSERTION	);
  pr_const("TEXTSW_FILTER_SET_SELECTION",TEXTSW_FILTER_SET_SELECTION	);








  pr_const("XDR_ENCODE",XDR_ENCODE);
  pr_const("XDR_DECODE",XDR_DECODE);
  pr_const("XDR_FREE",XDR_FREE);
  pr_const("AUTH_OK",AUTH_OK);
  pr_const("AUTH_BADCRED",AUTH_BADCRED);
  pr_const("AUTH_REJECTEDCRED",AUTH_REJECTEDCRED);
  pr_const("AUTH_BADVERF",AUTH_BADVERF);
  pr_const("AUTH_REJECTEDVERF",AUTH_REJECTEDVERF);
  pr_const("AUTH_TOOWEAK",AUTH_TOOWEAK);
  pr_const("AUTH_INVALIDRESP",AUTH_INVALIDRESP);
  pr_const("AUTH_FAILED",AUTH_FAILED);
  pr_const("RPC_SUCCESS",RPC_SUCCESS);
  pr_const("RPC_CANTENCODEARGS",RPC_CANTENCODEARGS);
  pr_const("RPC_CANTDECODERES",RPC_CANTDECODERES);
  pr_const("RPC_CANTSEND",RPC_CANTSEND);
  pr_const("RPC_CANTRECV",RPC_CANTRECV);
  pr_const("RPC_TIMEDOUT",RPC_TIMEDOUT);
  pr_const("RPC_VERSMISMATCH",RPC_VERSMISMATCH);
  pr_const("RPC_AUTHERROR",RPC_AUTHERROR);
  pr_const("RPC_PROGUNAVAIL",RPC_PROGUNAVAIL);
  pr_const("RPC_PROGVERSMISMATCH",RPC_PROGVERSMISMATCH);
  pr_const("RPC_PROCUNAVAIL",RPC_PROCUNAVAIL);
  pr_const("RPC_CANTDECODEARGS",RPC_CANTDECODEARGS);
  pr_const("RPC_SYSTEMERROR",RPC_SYSTEMERROR);
  pr_const("RPC_UNKNOWNHOST",RPC_UNKNOWNHOST);
  pr_const("RPC_PMAPFAILURE",RPC_PMAPFAILURE);
  pr_const("RPC_PROGNOTREGISTERED",RPC_PROGNOTREGISTERED);
  pr_const("RPC_FAILED",RPC_FAILED);
  pr_const("CALL",CALL);
  pr_const("REPLY",REPLY);
  pr_const("MSG_ACCEPTED",MSG_ACCEPTED);
  pr_const("MSG_DENIED",MSG_DENIED);
  pr_const("SUCCESS",SUCCESS);
  pr_const("PROG_UNAVAIL",PROG_UNAVAIL);
  pr_const("PROG_MISMATCH",PROG_MISMATCH);
  pr_const("PROC_UNAVAIL",PROC_UNAVAIL);
  pr_const("GARBAGE_ARGS",GARBAGE_ARGS);
  pr_const("SYSTEM_ERR",SYSTEM_ERR);
  pr_const("RPC_MISMATCH",RPC_MISMATCH);
  pr_const("AUTH_ERROR",AUTH_ERROR);
  pr_const("SELN_REQ_BYTESIZE",SELN_REQ_BYTESIZE		);
  pr_const("SELN_REQ_CONTENTS_ASCII",SELN_REQ_CONTENTS_ASCII	);
  pr_const("SELN_REQ_CONTENTS_PIECES",SELN_REQ_CONTENTS_PIECES	);
  pr_const("SELN_REQ_FIRST",SELN_REQ_FIRST		);
  pr_const("SELN_REQ_FIRST_UNIT",SELN_REQ_FIRST_UNIT		);
  pr_const("SELN_REQ_LAST",SELN_REQ_LAST		);
  pr_const("SELN_REQ_LAST_UNIT",SELN_REQ_LAST_UNIT		);
  pr_const("SELN_REQ_LEVEL",SELN_REQ_LEVEL		);
  pr_const("SELN_REQ_FILE_NAME",SELN_REQ_FILE_NAME		);
  pr_const("SELN_REQ_DELETE",SELN_REQ_DELETE		);
  pr_const("SELN_REQ_RESTORE",SELN_REQ_RESTORE		);
  pr_const("SELN_REQ_YIELD",SELN_REQ_YIELD		);
  pr_const("SELN_REQ_FAKE_LEVEL",SELN_REQ_FAKE_LEVEL		);
  pr_const("SELN_REQ_SET_LEVEL",SELN_REQ_SET_LEVEL		);
  pr_const("SELN_TRACE_ACQUIRE",SELN_TRACE_ACQUIRE		);
  pr_const("SELN_TRACE_DONE",SELN_TRACE_DONE		);
  pr_const("SELN_TRACE_HOLD_FILE",SELN_TRACE_HOLD_FILE	);
  pr_const("SELN_TRACE_INFORM",SELN_TRACE_INFORM		);
  pr_const("SELN_TRACE_INQUIRE",SELN_TRACE_INQUIRE		);
  pr_const("SELN_TRACE_YIELD",SELN_TRACE_YIELD		);
  pr_const("SELN_TRACE_STOP",SELN_TRACE_STOP		);
  pr_const("SELN_TRACE_DUMP",SELN_TRACE_DUMP		);
  pr_const("SELN_REQ_END_REQUEST",SELN_REQ_END_REQUEST	);
  pr_const("SELN_REQ_UNKNOWN",SELN_REQ_UNKNOWN		);
  pr_const("SELN_REQ_FAILED",SELN_REQ_FAILED		);
  pr_const("SELN_LEVEL_FIRST",SELN_LEVEL_FIRST	);
  pr_const("SELN_LEVEL_LINE",SELN_LEVEL_LINE	);
  pr_const("SELN_LEVEL_ALL",SELN_LEVEL_ALL	);
  pr_const("SELN_LEVEL_NEXT",SELN_LEVEL_NEXT	);
  pr_const("SELN_LEVEL_PREVIOUS",SELN_LEVEL_PREVIOUS	);

  pr_const("SELN_FAILED",SELN_FAILED);
  pr_const("SELN_SUCCESS",SELN_SUCCESS);
  pr_const("SELN_NON_EXIST",SELN_NON_EXIST);
  pr_const("SELN_DIDNT_HAVE",SELN_DIDNT_HAVE);
  pr_const("SELN_WRONG_RANK",SELN_WRONG_RANK);
  pr_const("SELN_CONTINUED",SELN_CONTINUED);
  pr_const("SELN_CANCEL",SELN_CANCEL);
  pr_const("SELN_UNRECOGNIZED ",SELN_UNRECOGNIZED );
  pr_const("SELN_UNKNOWN",SELN_UNKNOWN);
  pr_const("SELN_CARET",SELN_CARET);
  pr_const("SELN_PRIMARY",SELN_PRIMARY);
  pr_const("SELN_SECONDARY",SELN_SECONDARY);
  pr_const("SELN_SHELF",SELN_SHELF);
  pr_const("SELN_UNSPECIFIED",SELN_UNSPECIFIED);
  pr_const("SELN_FN_ERROR",SELN_FN_ERROR);
  pr_const("SELN_FN_STOP",SELN_FN_STOP);
  pr_const("SELN_FN_AGAIN",SELN_FN_AGAIN);
  pr_const("SELN_FN_PROPS",SELN_FN_PROPS);
  pr_const("SELN_FN_UNDO",SELN_FN_UNDO);
  pr_const("SELN_FN_FRONT",SELN_FN_FRONT);
  pr_const("SELN_FN_PUT",SELN_FN_PUT);
  pr_const("SELN_FN_OPEN",SELN_FN_OPEN);
  pr_const("SELN_FN_GET",SELN_FN_GET);
  pr_const("SELN_FN_FIND",SELN_FN_FIND);
  pr_const("SELN_FN_DELETE",SELN_FN_DELETE);
  pr_const("SELN_NONE",SELN_NONE);
  pr_const("SELN_EXISTS",SELN_EXISTS);
  pr_const("SELN_FILE",SELN_FILE);
  pr_const("SELN_IGNORE",SELN_IGNORE);
  pr_const("SELN_REQUEST",SELN_REQUEST);
  pr_const("SELN_FIND",SELN_FIND);
  pr_const("SELN_SHELVE",SELN_SHELVE);
  pr_const("SELN_DELETE",SELN_DELETE);

  pr_const("PANEL_CLEAR",PANEL_CLEAR);
  pr_const("PANEL_NO_CLEAR",PANEL_NO_CLEAR);
  pr_const("PANEL_NONE",PANEL_NONE);
  pr_const("PANEL_ALL",PANEL_ALL);
  pr_const("PANEL_NON_PRINTABLE",PANEL_NON_PRINTABLE);
  pr_const("PANEL_SPECIFIED",PANEL_SPECIFIED);
  pr_const("PANEL_CURRENT",PANEL_CURRENT);
  pr_const("PANEL_DONE",PANEL_DONE);
  pr_const("PANEL_MARKED",PANEL_MARKED);
  pr_const("PANEL_VERTICAL",PANEL_VERTICAL);
  pr_const("PANEL_HORIZONTAL",PANEL_HORIZONTAL);
  pr_const("PANEL_INVERTED",PANEL_INVERTED);
  pr_const("PANEL_INSERT",PANEL_INSERT);
  pr_const("PANEL_NEXT",PANEL_NEXT);
  pr_const("PANEL_PREVIOUS",PANEL_PREVIOUS);
  pr_const("PANEL_NONE_DOWN",PANEL_NONE_DOWN);
  pr_const("PANEL_LEFT_DOWN",PANEL_LEFT_DOWN);
  pr_const("PANEL_MIDDLE_DOWN",PANEL_MIDDLE_DOWN);
  pr_const("PANEL_RIGHT_DOWN",PANEL_RIGHT_DOWN);
  pr_const("PANEL_CHORD_DOWN",PANEL_CHORD_DOWN);

  pr_const("TTYOPT_PAGEMODE",TTYOPT_PAGEMODE);
  pr_const("TTYOPT_SELSVC",TTYOPT_SELSVC);
  pr_const("TTYOPT_TEXT",TTYOPT_TEXT);
  pr_const("TTYSW_BOLD_NONE",TTYSW_BOLD_NONE);
  pr_const("TTYSW_BOLD_OFFSET_X",TTYSW_BOLD_OFFSET_X);
  pr_const("TTYSW_BOLD_OFFSET_Y",TTYSW_BOLD_OFFSET_Y);
  pr_const("TTYSW_BOLD_OFFSET_XY",TTYSW_BOLD_OFFSET_XY);
  pr_const("TTYSW_BOLD_INVERT",TTYSW_BOLD_INVERT);
  pr_const("TTYSW_BOLD_MAX",TTYSW_BOLD_MAX);

  pr_const("TEXTSW_STATUS_OKAY",TEXTSW_STATUS_OKAY);
  pr_const("TEXTSW_STATUS_OTHER_ERROR",TEXTSW_STATUS_OTHER_ERROR);
  pr_const("TEXTSW_STATUS_CANNOT_ALLOCATE",
	   TEXTSW_STATUS_CANNOT_ALLOCATE);
  pr_const("TEXTSW_STATUS_CANNOT_OPEN_INPUT",TEXTSW_STATUS_CANNOT_OPEN_INPUT);
  pr_const("TEXTSW_STATUS_BAD_ATTR",TEXTSW_STATUS_BAD_ATTR);
  pr_const("TEXTSW_STATUS_BAD_ATTR_VALUE",TEXTSW_STATUS_BAD_ATTR_VALUE);
/*
  pr_const("WIN_MENU_NO_CMD",WIN_MENU_NO_CMD);
  pr_const("WIN_MENU_SAVE",WIN_MENU_SAVE);
  pr_const("WIN_MENU_SAVE_QUIT",WIN_MENU_SAVE_QUIT);
  pr_const("WIN_MENU_SAVE_RESET",WIN_MENU_SAVE_RESET);
  pr_const("WIN_MENU_SAVE_CLOSE",WIN_MENU_SAVE_CLOSE);
  pr_const("WIN_MENU_STORE",WIN_MENU_STORE);
  pr_const("WIN_MENU_STORE_QUIT",WIN_MENU_STORE_QUIT);
  pr_const("WIN_MENU_STORE_CLOSE",WIN_MENU_STORE_CLOSE);
  pr_const("WIN_MENU_CLIP_LINES",WIN_MENU_CLIP_LINES);
  pr_const("WIN_MENU_WRAP_LINES_AT_CHAR",WIN_MENU_WRAP_LINES_AT_CHAR);
  pr_const("WIN_MENU_RESET",WIN_MENU_RESET);
  pr_const("WIN_MENU_LOAD",WIN_MENU_LOAD);
  pr_const("WIN_MENU_CREATE_VIEW",WIN_MENU_CREATE_VIEW);
  pr_const("WIN_MENU_DESTROY_VIEW",WIN_MENU_DESTROY_VIEW);
  pr_const("WIN_MENU_CD",WIN_MENU_CD);
  pr_const("WIN_MENU_NORMALIZE_INSERTION",WIN_MENU_NORMALIZE_INSERTION);
  pr_const("WIN_MENU_NORMALIZE_LINE",WIN_MENU_NORMALIZE_LINE);
  pr_const("WIN_MENU_COUNT_TO_LINE",WIN_MENU_COUNT_TO_LINE);
  pr_const("WIN_MENU_FILE_STUFF",WIN_MENU_FILE_STUFF);
  pr_const("WIN_MENU_FIND",WIN_MENU_FIND);
  pr_const("WIN_MENU_FIND_BACKWARD",WIN_MENU_FIND_BACKWARD);
  pr_const("WIN_MENU_FIND_SHELF",WIN_MENU_FIND_SHELF);
  pr_const("WIN_MENU_FIND_SHELF_BACKWARD",WIN_MENU_FIND_SHELF_BACKWARD);
  pr_const("WIN_MENU_PUT_THEN_GET",WIN_MENU_PUT_THEN_GET);
  pr_const("WIN_MENU_LAST_CMD",WIN_MENU_LAST_CMD);
*/

  pr_const("TEXTSW_IF_AUTO_SCROLL",TEXTSW_IF_AUTO_SCROLL);
  pr_const("TEXTSW_CLIP",TEXTSW_CLIP);
  pr_const("TEXTSW_WRAP_AT_CHAR",TEXTSW_WRAP_AT_CHAR);
  pr_const("TEXTSW_WRAP_AT_LINE",TEXTSW_WRAP_AT_LINE);


  pr_const("WIN_NULL_VALUE",WIN_NULL_VALUE);
  pr_const("WIN_NO_EVENTS",WIN_NO_EVENTS);
  pr_const("WIN_UP_EVENTS",WIN_UP_EVENTS);
  pr_const("WIN_ASCII_EVENTS",WIN_ASCII_EVENTS);
  pr_const("WIN_UP_ASCII_EVENTS",WIN_UP_ASCII_EVENTS);
  pr_const("WIN_MOUSE_BUTTONS",WIN_MOUSE_BUTTONS);
  pr_const("WIN_IN_TRANSIT_EVENTS",WIN_IN_TRANSIT_EVENTS);
  pr_const("WIN_LEFT_KEYS",WIN_LEFT_KEYS);
  pr_const("WIN_TOP_KEYS",WIN_TOP_KEYS);
  pr_const("WIN_RIGHT_KEYS",WIN_RIGHT_KEYS);
  pr_const("WIN_CREATE ",WIN_CREATE );
  pr_const("WIN_INSTALL",WIN_INSTALL);
  pr_const("WIN_INSERT",WIN_INSERT);
  pr_const("WIN_REMOVE",WIN_REMOVE);
  pr_const("WIN_DESTROY",WIN_DESTROY);

/*
  pr_const("WIN_ADJUST_X ",WIN_ADJUST_X );
  pr_const("WIN_ADJUST_RIGHT_OF ",WIN_ADJUST_RIGHT_OF );
  pr_const("WIN_ADJUST_Y ",WIN_ADJUST_Y );
  pr_const("WIN_ADJUST_BELOW ",WIN_ADJUST_BELOW );
  pr_const("WIN_ADJUST_WIDTH ",WIN_ADJUST_WIDTH );
  pr_const("WIN_ADJUST_HEIGHT ",WIN_ADJUST_HEIGHT );
  pr_const("WIN_ADJUSTED ",WIN_ADJUSTED );
*/
  pr_const("WIN_ADJUST_RECT ",WIN_ADJUST_RECT );
  pr_const("WIN_GET_X ",WIN_GET_X );
  pr_const("WIN_GET_Y ",WIN_GET_Y );
  pr_const("WIN_GET_WIDTH ",WIN_GET_WIDTH );
  pr_const("WIN_GET_HEIGHT",WIN_GET_HEIGHT);
  pr_const("WIN_GET_RECT ",WIN_GET_RECT );
  pr_const("WIN_LAYOUT",WIN_LAYOUT);
  pr_const("NULL",NULL);
  pr_const("TRUE",TRUE);
  pr_const("FALSE",FALSE);

  pr_const("LOC_MOVE",LOC_MOVE);
  pr_const("LOC_WINENTER",LOC_WINENTER);
  pr_const("LOC_WINEXIT",LOC_WINEXIT);
  pr_const("LOC_MOVEWHILEBUTDOWN",LOC_MOVEWHILEBUTDOWN);
  pr_const("LOC_DRAG",LOC_DRAG);
  pr_const("WIN_REPAINT",WIN_REPAINT);
  pr_const("WIN_RESIZE",WIN_RESIZE);
  pr_const("WIN_STOP",WIN_STOP);
  pr_const("KBD_USE",KBD_USE);
  pr_const("KBD_DONE",KBD_DONE);
  pr_const("IM_NEGEVENT",IM_NEGEVENT);
  pr_const("IM_POSASCII",IM_POSASCII);
  pr_const("IM_ANSI",IM_ANSI);
  pr_const("IM_UNENCODED",IM_UNENCODED);
  pr_const("IM_ASCII",IM_ASCII);
  pr_const("IM_META",IM_META);
  pr_const("IM_NEGASCII",IM_NEGASCII);
  pr_const("IM_NEGMETA",IM_NEGMETA);
  pr_const("IM_TOP",IM_TOP);
  pr_const("IM_NEGTOP",IM_NEGTOP);
  pr_const("IM_INTRANSIT",IM_INTRANSIT);
  pr_const("INPUTMASK_NULL",INPUTMASK_NULL);
  pr_const("IE_NEGEVENT",IE_NEGEVENT);
  pr_const("CAPSLOCK",CAPSLOCK);

  pr_const("CAPSMASK",CAPSMASK);
  pr_const("SHIFTLOCK",SHIFTLOCK);
  pr_const("LEFTSHIFT",LEFTSHIFT);
  pr_const("RIGHTSHIFT",RIGHTSHIFT);
  pr_const("SHIFTMASK",SHIFTMASK);
  pr_const("LEFTCTRL",LEFTCTRL);
  pr_const("RIGHTCTRL",RIGHTCTRL);
  pr_const("CTRLMASK",CTRLMASK);
  pr_const("META_SHIFT_MASK",META_SHIFT_MASK);
  pr_const("EVENT_NULL",EVENT_NULL);
  pr_const("WS_ORDER_LMR",WS_ORDER_LMR);
  pr_const("WS_ORDER_LRM",WS_ORDER_LRM);
  pr_const("WS_ORDER_MLR",WS_ORDER_MLR);
  pr_const("WS_ORDER_MRL",WS_ORDER_MRL);
  pr_const("WS_ORDER_RLM",WS_ORDER_RLM);
  pr_const("WS_ORDER_RML",WS_ORDER_RML);
  pr_const("WS_SCALE_MAX_COUNT",WS_SCALE_MAX_COUNT);
  pr_const("WS_SCALE_MAX_CEILING",WS_SCALE_MAX_CEILING);

  pr_const("VUID_SEG_SIZE",VUID_SEG_SIZE);
  pr_const("ASCII_DEVID",ASCII_DEVID);
  pr_const("TOP_DEVID",TOP_DEVID);
  pr_const("PANEL_DEVID",PANEL_DEVID);
  pr_const("WORKSTATION_DEVID",WORKSTATION_DEVID);
  pr_const("LAST_DEVID",LAST_DEVID);
  pr_const("ASCII_FIRST",ASCII_FIRST);
  pr_const("ASCII_LAST",ASCII_LAST);
  pr_const("META_FIRST",META_FIRST);
  pr_const("META_LAST",META_LAST);
  pr_const("TOP_FIRST",TOP_FIRST);
  pr_const("TOP_LAST",TOP_LAST);
  pr_const("VKEY_FIRST",VKEY_FIRST);
  pr_const("VKEY_UP",VKEY_UP);
  pr_const("VKEY_DOWN",VKEY_DOWN);
  pr_const("VKEY_KBD_CODES",VKEY_KBD_CODES);
  pr_const("VKEY_FIRSTPSEUDO",VKEY_FIRSTPSEUDO);
  pr_const("VKEY_LASTPSEUDO",VKEY_LASTPSEUDO);
  pr_const("VKEY_FIRSTSHIFT",VKEY_FIRSTSHIFT);
  pr_const("SHIFT_CAPSLOCK",SHIFT_CAPSLOCK);
  pr_const("SHIFT_LOCK",SHIFT_LOCK);
  pr_const("SHIFT_LEFT",SHIFT_LEFT);
  pr_const("SHIFT_RIGHT",SHIFT_RIGHT);
  pr_const("SHIFT_LEFTCTRL",SHIFT_LEFTCTRL);
  pr_const("SHIFT_CTRL",SHIFT_CTRL);
  pr_const("SHIFT_RIGHTCTRL",SHIFT_RIGHTCTRL);
  pr_const("SHIFT_META",SHIFT_META);
  pr_const("SHIFT_TOP",SHIFT_TOP);
  pr_const("SHIFT_CMD",SHIFT_CMD);
  pr_const("VKEY_LASTSHIFT",VKEY_LASTSHIFT);
  pr_const("VKEY_FIRSTFUNC",VKEY_FIRSTFUNC);
  pr_const("BUT_FIRST",BUT_FIRST);
  pr_const("BUT_LAST",BUT_LAST);
  pr_const("KEY_LEFTFIRST",KEY_LEFTFIRST);
  pr_const("KEY_LEFTLAST",KEY_LEFTLAST);
  pr_const("KEY_RIGHTFIRST",KEY_RIGHTFIRST);
  pr_const("KEY_RIGHTLAST",KEY_RIGHTLAST);
  pr_const("KEY_TOPFIRST",KEY_TOPFIRST);
  pr_const("KEY_TOPLAST",KEY_TOPLAST);
  pr_const("KEY_BOTTOMLEFT",KEY_BOTTOMLEFT);
  pr_const("KEY_BOTTOMRIGHT",KEY_BOTTOMRIGHT);
  pr_const("VKEY_LASTFUNC",VKEY_LASTFUNC);
  pr_const("VKEY_LAST",VKEY_LAST);
  pr_const("MOUSE_DEVID",MOUSE_DEVID);
  pr_const("VLOC_FIRST",VLOC_FIRST);
  pr_const("LOC_FIRST_DELTA",LOC_FIRST_DELTA);
  pr_const("LOC_X_DELTA",LOC_X_DELTA);
  pr_const("LOC_Y_DELTA",LOC_Y_DELTA);
  pr_const("LOC_LAST_DELTA",LOC_LAST_DELTA);
  pr_const("LOC_FIRST_ABSOLUTE",LOC_FIRST_ABSOLUTE);
  pr_const("LOC_X_ABSOLUTE",LOC_X_ABSOLUTE);
  pr_const("LOC_Y_ABSOLUTE",LOC_Y_ABSOLUTE);
  pr_const("LOC_LAST_ABSOLUTE",LOC_LAST_ABSOLUTE);
  pr_const("VLOC_BATCH",VLOC_BATCH);
  pr_const("VLOC_LAST",VLOC_LAST);
  pr_const("MS_LEFT",MS_LEFT);
  pr_const("MS_MIDDLE",MS_MIDDLE);
  pr_const("MS_RIGHT",MS_RIGHT);
  pr_const("FE_PAIR_NONE",FE_PAIR_NONE);
  pr_const("FE_PAIR_SET",FE_PAIR_SET);
  pr_const("FE_PAIR_DELTA",FE_PAIR_DELTA);
  pr_const("FE_PAIR_ABSOLUTE",FE_PAIR_ABSOLUTE);
  pr_const("FIRM_EVENT_NULL",FIRM_EVENT_NULL);
  pr_const("VUID_NATIVE",VUID_NATIVE);
  pr_const("VUID_FIRM_EVENT",VUID_FIRM_EVENT);
}
