;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(provide :irt-all-robot "models/load-irt-all-robots.l")

(warning-message 2 "loading human-robot.l	(human) for make model~%")
(load "models/human-robot.l")

(warning-message 2 "loading patra-robot.l	(patra) for make model~%")
(load "models/patra-robot.l")

(warning-message 2 "loading h7-robot.l	(h7) for make model~%")
(load "models/h7-robot.l")

(warning-message 2 "loading h6-robot.l	(h6) for make model~%")
(load "models/h6-robot.l")

(warning-message 2 "loading h4-robot.l	(h4) for make model~%")
(load "models/h4-robot.l")

(warning-message 2 "loading h3s-robot.l	(h3s) for make model~%")
(load "models/h3s-robot.l")

(warning-message 2 "loading h3-robot.l	(h3) for make model~%")
(load "models/h3-robot.l")

(warning-message 2 "loading jrob1-robot.l	(jrob1) for make model~%")
(load "models/jrob1-robot.l")

(warning-message 2 "loading macket-robot.l	(macket) for make model~%")
(load "models/macket-robot.l")

(warning-message 2 "loading macra-robot.l	(macra) for make model~%")
(load "models/macra-robot.l")

(warning-message 2 "loading penguin-robot.l	(penguin) for make model~%")
(load "models/penguin-robot.l")

(warning-message 2 "loading tamaii-robot.l	(tamaii) for make model~%")
(load "models/tamaii-robot.l")

(warning-message 2 "loading tama-robot.l	(tama) for make model~%")
(load "models/tama-robot.l")

(warning-message 2 "loading chibikaz-robot.l	(chibikaz) for make model~%")
(load "models/chibikaz-robot.l")

(warning-message 2 "loading tot-robot.l	(tot) for make model~%")
(load "models/tot-robot.l")

(warning-message 2 "loading kaz3-robot.l	(kaz3) for make model~%")
(load "models/kaz3-robot.l")

(warning-message 2 "loading kaz2-robot.l	(kaz2) for make model~%")
(load "models/kaz2-robot.l")

(warning-message 2 "loading kaz-robot.l	(kaz) for make model~%")
(load "models/kaz-robot.l")

(warning-message 2 "loading zero-robot.l	(zero) for make model~%")
(load "models/zero-robot.l")

(warning-message 2 "loading akira3-robot.l	(akira3) for make model~%")
(load "models/akira3-robot.l")

(warning-message 2 "loading akira2-robot.l	(akira2) for make model~%")
(load "models/akira2-robot.l")

(warning-message 2 "loading akira-robot.l	(akira) for make model~%")
(load "models/akira-robot.l")

(warning-message 2 "loading millennium-robot.l	(millennium) for make model~%")
(load "models/millennium-robot.l")

(warning-message 2 "loading taro-robot.l	(taro) for make model~%")
(load "models/taro-robot.l")

(warning-message 2 "loading kuma-robot.l	(kuma) for make model~%")
(load "models/kuma-robot.l")

(warning-message 2 "loading igoid3-robot.l	(igoid3) for make model~%")
(load "models/igoid3-robot.l")

(warning-message 2 "loading igoid2-robot.l	(igoid2) for make model~%")
(load "models/igoid2-robot.l")

(warning-message 2 "loading igoid-robot.l	(igoid) for make model~%")
(load "models/igoid-robot.l")

(warning-message 2 "loading bishamon-robot.l	(bishamon) for make model~%")
(load "models/bishamon-robot.l")

(warning-message 2 "loading hanzous-robot.l	(hanzous) for make model~%")
(load "models/hanzous-robot.l")

(warning-message 2 "loading ikuo-haru-robot.l	(ikuo-haru) for make model~%")
(load "models/ikuo-haru-robot.l")

(warning-message 2 "loading saizo-robot.l	(saizo) for make model~%")
(load "models/saizo-robot.l")

(warning-message 2 "loading miharu-robot.l	(miharu) for make model~%")
(load "models/miharu-robot.l")

(warning-message 2 "loading hanzou-robot.l	(hanzou) for make model~%")
(load "models/hanzou-robot.l")

(warning-message 2 "loading ape3-robot.l	(ape3) for make model~%")
(load "models/ape3-robot.l")

(warning-message 2 "loading ape-robot.l	(ape) for make model~%")
(load "models/ape-robot.l")
;;
;;
(setq *irt-all-robots* '( (ape) (ape3) (hanzou) (miharu) (saizo) (ikuo-haru) (hanzous) (bishamon) (igoid) (igoid2) (igoid3) (kuma) (taro) (millennium) (akira) (akira2) (akira3) (zero) (kaz) (kaz2) (kaz3) (tot) (chibikaz) (tama) (tamaii) (penguin) (macra) (macket) (jrob1) (h3) (h3s) (h4) (h6) (h7) (patra) (human)))