;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defun execute-irtviewer (lst &key (width 100) (height 100)) (let ((x 0) (y 0) (x-offset 10) (y-offset 25) (x-limit (elt (send x:*root* :size) 0))) (mapcar #'(lambda (f) (warning-message 2 "~A~%" f) (let* ((r (eval f)) (p (make-irtviewer :width width :height height :title (send r :name) :no-menu t))) (send p :move x y) (send p :objects (list r)) (send p :look-all) (send p :draw-objects) (incf x (+ width x-offset)) (if (>= x (- x-limit width)) (setq x 0 y (incf y (+ height y-offset)))) p)) lst)))
;;
;;
(load "load-irt-all-robots.l")
;;
;;
;;
(warning-message 3 "(make-all-robots) to make all robots model~%")
(defun make-all-robots ()
    (execute-irtviewer *irt-all-robots* :width 70 :height 70))
