;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(provide :irt-all-scene "models/load-irt-all-scenes.l")

(warning-message 2 "loading eng2-7f-elevatorhall-scene.l	(eng2-7f-elevatorhall) for make model~%")
(load "models/eng2-7f-elevatorhall-scene.l")

(warning-message 2 "loading eng2-7f-southwing-scene.l	(eng2-7f-southwing) for make model~%")
(load "models/eng2-7f-southwing-scene.l")

(warning-message 2 "loading drcbox-scene.l	(drcbox) for make model~%")
(load "models/drcbox-scene.l")

(warning-message 2 "loading room83b1-scene.l	(room83b1) for make model~%")
(load "models/room83b1-scene.l")

(warning-message 2 "loading room73b2-scene.l	(room73b2) for make model~%")
(load "models/room73b2-scene.l")

(warning-message 2 "loading room73a3-scene.l	(room73a3) for make model~%")
(load "models/room73a3-scene.l")

(warning-message 2 "loading room610-scene.l	(room610) for make model~%")
(load "models/room610-scene.l")

(warning-message 2 "loading room602-scene.l	(room602) for make model~%")
(load "models/room602-scene.l")
;;
;;
(setq *irt-all-scenes* '( (room602) (room610) (room73a3) (room73b2) (room83b1) (drcbox) (eng2-7f-southwing) (eng2-7f-elevatorhall)))