;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(provide :irt-all-object "models/load-irt-all-objects.l")

(warning-message 2 "loading alinco-tbf-5-ladder-object.l	(alinco-tbf-5-ladder) for make model~%")
(load "models/alinco-tbf-5-ladder-object.l")

(warning-message 2 "loading alinco-ccst-100-ladder-object.l	(alinco-ccst-100-ladder) for make model~%")
(load "models/alinco-ccst-100-ladder-object.l")

(warning-message 2 "loading hasegawa-ladder-object.l	(hasegawa-ladder) for make model~%")
(load "models/hasegawa-ladder-object.l")

(warning-message 2 "loading asahikasei-small-basket-object.l	(asahikasei-small-basket) for make model~%")
(load "models/asahikasei-small-basket-object.l")

(warning-message 2 "loading maxvalu-supermarket-basket-object.l	(maxvalu-supermarket-basket) for make model~%")
(load "models/maxvalu-supermarket-basket-object.l")

(warning-message 2 "loading 73b2-cupboard-without-door-object.l	(73b2-cupboard-without-door) for make model~%")
(load "models/73b2-cupboard-without-door-object.l")

(warning-message 2 "loading unknown-side-table-object.l	(unknown-side-table) for make model~%")
(load "models/unknown-side-table-object.l")

(warning-message 2 "loading dewalt-drill-object.l	(dewalt-drill) for make model~%")
(load "models/dewalt-drill-object.l")

(warning-message 2 "loading misumi-handrail-object.l	(misumi-handrail) for make model~%")
(load "models/misumi-handrail-object.l")

(warning-message 2 "loading darkgoldenrod-stairs-object.l	(darkgoldenrod-stairs) for make model~%")
(load "models/darkgoldenrod-stairs-object.l")

(warning-message 2 "loading gifuplastic-720-cart-object.l	(gifuplastic-720-cart) for make model~%")
(load "models/gifuplastic-720-cart-object.l")

(warning-message 2 "loading gifuplastic-900-cart-object.l	(gifuplastic-900-cart) for make model~%")
(load "models/gifuplastic-900-cart-object.l")

(warning-message 2 "loading foldable-desk-object.l	(foldable-desk) for make model~%")
(load "models/foldable-desk-object.l")

(warning-message 2 "loading chessboard-10-3x3-object.l	(chessboard-10-3x3) for make model~%")
(load "models/chessboard-10-3x3-object.l")

(warning-message 2 "loading chessboard-30-6x5-object.l	(chessboard-30-6x5) for make model~%")
(load "models/chessboard-30-6x5-object.l")

(warning-message 2 "loading chessboard-30-7x5-object.l	(chessboard-30-7x5) for make model~%")
(load "models/chessboard-30-7x5-object.l")

(warning-message 2 "loading chessboard-108-9x7-object.l	(chessboard-108-9x7) for make model~%")
(load "models/chessboard-108-9x7-object.l")

(warning-message 2 "loading elevator_inside_panel_eng2-object.l	(elevator_inside_panel_eng2) for make model~%")
(load "models/elevator_inside_panel_eng2-object.l")

(warning-message 2 "loading elevator_call_panel_eng2-object.l	(elevator_call_panel_eng2) for make model~%")
(load "models/elevator_call_panel_eng2-object.l")

(warning-message 2 "loading elevator_inside_panel-object.l	(elevator_inside_panel) for make model~%")
(load "models/elevator_inside_panel-object.l")

(warning-message 2 "loading elevator_call_panel-object.l	(elevator_call_panel) for make model~%")
(load "models/elevator_call_panel-object.l")

(warning-message 2 "loading broom-object.l	(broom) for make model~%")
(load "models/broom-object.l")

(warning-message 2 "loading bamboo-broom-object.l	(bamboo-broom) for make model~%")
(load "models/bamboo-broom-object.l")

(warning-message 2 "loading azuma-short-broom-object.l	(azuma-short-broom) for make model~%")
(load "models/azuma-short-broom-object.l")

(warning-message 2 "loading azuma-broom-object.l	(azuma-broom) for make model~%")
(load "models/azuma-broom-object.l")

(warning-message 2 "loading picaladder-object.l	(picaladder) for make model~%")
(load "models/picaladder-object.l")

(warning-message 2 "loading coffee-cup-saucer-object.l	(coffee-cup-saucer) for make model~%")
(load "models/coffee-cup-saucer-object.l")

(warning-message 2 "loading coffee-cup-cup-object.l	(coffee-cup-cup) for make model~%")
(load "models/coffee-cup-cup-object.l")

(warning-message 2 "loading hrp2-shelf-object.l	(hrp2-shelf) for make model~%")
(load "models/hrp2-shelf-object.l")

(warning-message 2 "loading hrp2-rack-object.l	(hrp2-rack) for make model~%")
(load "models/hrp2-rack-object.l")

(warning-message 2 "loading plasmatv-object.l	(plasmatv) for make model~%")
(load "models/plasmatv-object.l")

(warning-message 2 "loading pockey-object.l	(pockey) for make model~%")
(load "models/pockey-object.l")

(warning-message 2 "loading petbottle-object.l	(petbottle) for make model~%")
(load "models/petbottle-object.l")

(warning-message 2 "loading book-object.l	(book) for make model~%")
(load "models/book-object.l")

(warning-message 2 "loading ball-object.l	(ball) for make model~%")
(load "models/ball-object.l")

(warning-message 2 "loading mug-cup-object.l	(mug-cup) for make model~%")
(load "models/mug-cup-object.l")

(warning-message 2 "loading cup-object.l	(cup) for make model~%")
(load "models/cup-object.l")

(warning-message 2 "loading dining-table-object.l	(dining-table) for make model~%")
(load "models/dining-table-object.l")

(warning-message 2 "loading sponge-object.l	(sponge) for make model~%")
(load "models/sponge-object.l")

(warning-message 2 "loading knife-object.l	(knife) for make model~%")
(load "models/knife-object.l")

(warning-message 2 "loading kettle-object.l	(kettle) for make model~%")
(load "models/kettle-object.l")

(warning-message 2 "loading chair-object.l	(chair) for make model~%")
(load "models/chair-object.l")

(warning-message 2 "loading cardboard-box-object.l	(cardboard-box) for make model~%")
(load "models/cardboard-box-object.l")

(warning-message 2 "loading single-arrow-object.l	(single-arrow) for make model~%")
(load "models/single-arrow-object.l")

(warning-message 2 "loading multibox-shelf-object.l	(multibox-shelf) for make model~%")
(load "models/multibox-shelf-object.l")

(warning-message 2 "loading brown-board-object.l	(brown-board) for make model~%")
(load "models/brown-board-object.l")

(warning-message 2 "loading bears-panel-object.l	(bears-panel) for make model~%")
(load "models/bears-panel-object.l")

(warning-message 2 "loading tray-object.l	(tray) for make model~%")
(load "models/tray-object.l")

(warning-message 2 "loading karimoku-kannonbiraki-cabinet-object.l	(karimoku-kannonbiraki-cabinet) for make model~%")
(load "models/karimoku-kannonbiraki-cabinet-object.l")

(warning-message 2 "loading shelf-object.l	(shelf) for make model~%")
(load "models/shelf-object.l")

(warning-message 2 "loading asplund-cabinet-4danbase-object.l	(asplund-cabinet-4danbase) for make model~%")
(load "models/asplund-cabinet-4danbase-object.l")

(warning-message 2 "loading asplund-cabinet-4dan-object.l	(asplund-cabinet-4dan) for make model~%")
(load "models/asplund-cabinet-4dan-object.l")

(warning-message 2 "loading audio_shelf-object.l	(audio_shelf) for make model~%")
(load "models/audio_shelf-object.l")

(warning-message 2 "loading cupboard-object.l	(cupboard) for make model~%")
(load "models/cupboard-object.l")

(warning-message 2 "loading fridge-object.l	(fridge) for make model~%")
(load "models/fridge-object.l")

(warning-message 2 "loading askul-1600-desk-object.l	(askul-1600-desk) for make model~%")
(load "models/askul-1600-desk-object.l")

(warning-message 2 "loading coe-light-object.l	(coe-light) for make model~%")
(load "models/coe-light-object.l")

(warning-message 2 "loading coe-stairs-object.l	(coe-stairs) for make model~%")
(load "models/coe-stairs-object.l")

(warning-message 2 "loading trashbox-object.l	(trashbox) for make model~%")
(load "models/trashbox-object.l")

(warning-message 2 "loading red-chair0-object.l	(red-chair0) for make model~%")
(load "models/red-chair0-object.l")

(warning-message 2 "loading uchida-shelf-1300-object.l	(uchida-shelf-1300) for make model~%")
(load "models/uchida-shelf-1300-object.l")

(warning-message 2 "loading uchida-shelf-1100-object.l	(uchida-shelf-1100) for make model~%")
(load "models/uchida-shelf-1100-object.l")

(warning-message 2 "loading coe-corner-shelf-object.l	(coe-corner-shelf) for make model~%")
(load "models/coe-corner-shelf-object.l")

(warning-message 2 "loading coe-450-shelf-object.l	(coe-450-shelf) for make model~%")
(load "models/coe-450-shelf-object.l")

(warning-message 2 "loading coe-1200-shelf-object.l	(coe-1200-shelf) for make model~%")
(load "models/coe-1200-shelf-object.l")

(warning-message 2 "loading coe-800-shelf-object.l	(coe-800-shelf) for make model~%")
(load "models/coe-800-shelf-object.l")

(warning-message 2 "loading sakae-900-desk-object.l	(sakae-900-desk) for make model~%")
(load "models/sakae-900-desk-object.l")

(warning-message 2 "loading coe-demospace-object.l	(coe-demospace) for make model~%")
(load "models/coe-demospace-object.l")

(warning-message 2 "loading coe-sofa-object.l	(coe-sofa) for make model~%")
(load "models/coe-sofa-object.l")

(warning-message 2 "loading coe-sofa2-object.l	(coe-sofa2) for make model~%")
(load "models/coe-sofa2-object.l")

(warning-message 2 "loading akiba-room-door-object.l	(akiba-room-door) for make model~%")
(load "models/akiba-room-door-object.l")

(warning-message 2 "loading akiba-door-window-object.l	(akiba-door-window) for make model~%")
(load "models/akiba-door-window-object.l")

(warning-message 2 "loading akiba-door-object.l	(akiba-door) for make model~%")
(load "models/akiba-door-object.l")

(warning-message 2 "loading door-object.l	(door) for make model~%")
(load "models/door-object.l")

(warning-message 2 "loading bariera-1400-middle-object.l	(bariera-1400-middle) for make model~%")
(load "models/bariera-1400-middle-object.l")

(warning-message 2 "loading bariera-1200-right-object.l	(bariera-1200-right) for make model~%")
(load "models/bariera-1200-right-object.l")

(warning-message 2 "loading bariera-1200-middle3-object.l	(bariera-1200-middle3) for make model~%")
(load "models/bariera-1200-middle3-object.l")

(warning-message 2 "loading bariera-1200-middle2-object.l	(bariera-1200-middle2) for make model~%")
(load "models/bariera-1200-middle2-object.l")

(warning-message 2 "loading bariera-1200-middle-object.l	(bariera-1200-middle) for make model~%")
(load "models/bariera-1200-middle-object.l")

(warning-message 2 "loading bariera-1200-corner-object.l	(bariera-1200-corner) for make model~%")
(load "models/bariera-1200-corner-object.l")

(warning-message 2 "loading bariera-900-middle-object.l	(bariera-900-middle) for make model~%")
(load "models/bariera-900-middle-object.l")

(warning-message 2 "loading bariera-900-left-object.l	(bariera-900-left) for make model~%")
(load "models/bariera-900-left-object.l")

(warning-message 2 "loading naiki-6door-closet-object.l	(naiki-6door-closet) for make model~%")
(load "models/naiki-6door-closet-object.l")

(warning-message 2 "loading itoki-900-slide-shelf-object.l	(itoki-900-slide-shelf) for make model~%")
(load "models/itoki-900-slide-shelf-object.l")

(warning-message 2 "loading itoki-900-shelf-object.l	(itoki-900-shelf) for make model~%")
(load "models/itoki-900-shelf-object.l")

(warning-message 2 "loading itoki-900-closet-object.l	(itoki-900-closet) for make model~%")
(load "models/itoki-900-closet-object.l")

(warning-message 2 "loading akiba-desk-object.l	(akiba-desk) for make model~%")
(load "models/akiba-desk-object.l")

(warning-message 2 "loading akiba-tv-desk-object.l	(akiba-tv-desk) for make model~%")
(load "models/akiba-tv-desk-object.l")

(warning-message 2 "loading sharp-iron-object.l	(sharp-iron) for make model~%")
(load "models/sharp-iron-object.l")

(warning-message 2 "loading toshiba-iron-object.l	(toshiba-iron) for make model~%")
(load "models/toshiba-iron-object.l")

(warning-message 2 "loading flower-tea-cup-dish-object.l	(flower-tea-cup-dish) for make model~%")
(load "models/flower-tea-cup-dish-object.l")

(warning-message 2 "loading white-bread-salad-dish-object.l	(white-bread-salad-dish) for make model~%")
(load "models/white-bread-salad-dish-object.l")

(warning-message 2 "loading blue-clear-cup-object.l	(blue-clear-cup) for make model~%")
(load "models/blue-clear-cup-object.l")

(warning-message 2 "loading iron-table-object.l	(iron-table) for make model~%")
(load "models/iron-table-object.l")

(warning-message 2 "loading frying-pan-object.l	(frying-pan) for make model~%")
(load "models/frying-pan-object.l")

(warning-message 2 "loading tognana-pasta-dish-object.l	(tognana-pasta-dish) for make model~%")
(load "models/tognana-pasta-dish-object.l")

(warning-message 2 "loading wanda-object.l	(wanda) for make model~%")
(load "models/wanda-object.l")

(warning-message 2 "loading iemon-object.l	(iemon) for make model~%")
(load "models/iemon-object.l")

(warning-message 2 "loading georgia-emerald-mountain-object.l	(georgia-emerald-mountain) for make model~%")
(load "models/georgia-emerald-mountain-object.l")

(warning-message 2 "loading sushi-cup-object.l	(sushi-cup) for make model~%")
(load "models/sushi-cup-object.l")

(warning-message 2 "loading docomo-p504is-object.l	(docomo-p504is) for make model~%")
(load "models/docomo-p504is-object.l")

(warning-message 2 "loading maruman-55-analog-clock-object.l	(maruman-55-analog-clock) for make model~%")
(load "models/maruman-55-analog-clock-object.l")

(warning-message 2 "loading casio-330-analog-clock-object.l	(casio-330-analog-clock) for make model~%")
(load "models/casio-330-analog-clock-object.l")

(warning-message 2 "loading ideainternational-300-analog-clock-object.l	(ideainternational-300-analog-clock) for make model~%")
(load "models/ideainternational-300-analog-clock-object.l")

(warning-message 2 "loading sharp-52-aquostv-object.l	(sharp-52-aquostv) for make model~%")
(load "models/sharp-52-aquostv-object.l")

(warning-message 2 "loading hitachi-42-plasmatv-object.l	(hitachi-42-plasmatv) for make model~%")
(load "models/hitachi-42-plasmatv-object.l")

(warning-message 2 "loading fujitsu-42-plasmatv-object.l	(fujitsu-42-plasmatv) for make model~%")
(load "models/fujitsu-42-plasmatv-object.l")

(warning-message 2 "loading toyama-1800-sidetable-object.l	(toyama-1800-sidetable) for make model~%")
(load "models/toyama-1800-sidetable-object.l")

(warning-message 2 "loading toyama-900-sidetable-object.l	(toyama-900-sidetable) for make model~%")
(load "models/toyama-900-sidetable-object.l")

(warning-message 2 "loading kokuyo-600-sidetable-object.l	(kokuyo-600-sidetable) for make model~%")
(load "models/kokuyo-600-sidetable-object.l")

(warning-message 2 "loading karimoku-1200-desk-object.l	(karimoku-1200-desk) for make model~%")
(load "models/karimoku-1200-desk-object.l")

(warning-message 2 "loading nissen-1100-desk-object.l	(nissen-1100-desk) for make model~%")
(load "models/nissen-1100-desk-object.l")

(warning-message 2 "loading askul-2700-desk-object.l	(askul-2700-desk) for make model~%")
(load "models/askul-2700-desk-object.l")

(warning-message 2 "loading askul-corner-desk-object.l	(askul-corner-desk) for make model~%")
(load "models/askul-corner-desk-object.l")

(warning-message 2 "loading askul-1200-desk-object.l	(askul-1200-desk) for make model~%")
(load "models/askul-1200-desk-object.l")

(warning-message 2 "loading askul-1400-desk-object.l	(askul-1400-desk) for make model~%")
(load "models/askul-1400-desk-object.l")

(warning-message 2 "loading askul-1200x700-desk-object.l	(askul-1200x700-desk) for make model~%")
(load "models/askul-1200x700-desk-object.l")

(warning-message 2 "loading desk-object.l	(desk) for make model~%")
(load "models/desk-object.l")

(warning-message 2 "loading toyama-900-desk-object.l	(toyama-900-desk) for make model~%")
(load "models/toyama-900-desk-object.l")

(warning-message 2 "loading uchida-desk-object.l	(uchida-desk) for make model~%")
(load "models/uchida-desk-object.l")

(warning-message 2 "loading uchida-monitor-leg-object.l	(uchida-monitor-leg) for make model~%")
(load "models/uchida-monitor-leg-object.l")

(warning-message 2 "loading uchida-2400-desk-object.l	(uchida-2400-desk) for make model~%")
(load "models/uchida-2400-desk-object.l")

(warning-message 2 "loading uchida-1800-desk-object.l	(uchida-1800-desk) for make model~%")
(load "models/uchida-1800-desk-object.l")

(warning-message 2 "loading unknown-1200-desk-object.l	(unknown-1200-desk) for make model~%")
(load "models/unknown-1200-desk-object.l")

(warning-message 2 "loading lion-1800-desk-object.l	(lion-1800-desk) for make model~%")
(load "models/lion-1800-desk-object.l")

(warning-message 2 "loading plus-1800-pdesk-object.l	(plus-1800-pdesk) for make model~%")
(load "models/plus-1800-pdesk-object.l")

(warning-message 2 "loading uchida-1000-desk-object.l	(uchida-1000-desk) for make model~%")
(load "models/uchida-1000-desk-object.l")

(warning-message 2 "loading uchida-1500-desk-object.l	(uchida-1500-desk) for make model~%")
(load "models/uchida-1500-desk-object.l")

(warning-message 2 "loading plus-6200-desk-object.l	(plus-6200-desk) for make model~%")
(load "models/plus-6200-desk-object.l")

(warning-message 2 "loading plus-7000-desk-object.l	(plus-7000-desk) for make model~%")
(load "models/plus-7000-desk-object.l")

(warning-message 2 "loading plus-2800-desk-object.l	(plus-2800-desk) for make model~%")
(load "models/plus-2800-desk-object.l")

(warning-message 2 "loading plus-1800w-desk-object.l	(plus-1800w-desk) for make model~%")
(load "models/plus-1800w-desk-object.l")

(warning-message 2 "loading plus-1800-desk-object.l	(plus-1800-desk) for make model~%")
(load "models/plus-1800-desk-object.l")

(warning-message 2 "loading plus-1700-desk-object.l	(plus-1700-desk) for make model~%")
(load "models/plus-1700-desk-object.l")

(warning-message 2 "loading plus-900-square-desk-object.l	(plus-900-square-desk) for make model~%")
(load "models/plus-900-square-desk-object.l")

(warning-message 2 "loading square-600-closet-object.l	(square-600-closet) for make model~%")
(load "models/square-600-closet-object.l")

(warning-message 2 "loading laundry-machine-object.l	(laundry-machine) for make model~%")
(load "models/laundry-machine-object.l")

(warning-message 2 "loading washing-machine-object.l	(washing-machine) for make model~%")
(load "models/washing-machine-object.l")

(warning-message 2 "loading kokuyo-450-locker-object.l	(kokuyo-450-locker) for make model~%")
(load "models/kokuyo-450-locker-object.l")

(warning-message 2 "loading 3t-600-clean-locker-object.l	(3t-600-clean-locker) for make model~%")
(load "models/3t-600-clean-locker-object.l")

(warning-message 2 "loading hitachi-18l-oven-object.l	(hitachi-18l-oven) for make model~%")
(load "models/hitachi-18l-oven-object.l")

(warning-message 2 "loading toshiba-microwave-oven-object.l	(toshiba-microwave-oven) for make model~%")
(load "models/toshiba-microwave-oven-object.l")

(warning-message 2 "loading refrigerator-object.l	(refrigerator) for make model~%")
(load "models/refrigerator-object.l")

(warning-message 2 "loading hitachi-fiesta-refrigerator-object.l	(hitachi-fiesta-refrigerator) for make model~%")
(load "models/hitachi-fiesta-refrigerator-object.l")

(warning-message 2 "loading toshiba-clacio-refrigerator-object.l	(toshiba-clacio-refrigerator) for make model~%")
(load "models/toshiba-clacio-refrigerator-object.l")

(warning-message 2 "loading askul-ntr-cupboard-object.l	(askul-ntr-cupboard) for make model~%")
(load "models/askul-ntr-cupboard-object.l")

(warning-message 2 "loading askul-gdh-cupboard-object.l	(askul-gdh-cupboard) for make model~%")
(load "models/askul-gdh-cupboard-object.l")

(warning-message 2 "loading arrow-1000-object.l	(arrow-1000) for make model~%")
(load "models/arrow-1000-object.l")

(warning-message 2 "loading arrow-object.l	(arrow) for make model~%")
(load "models/arrow-object.l")

(warning-message 2 "loading bidirectional-arrow-object.l	(bidirectional-arrow) for make model~%")
(load "models/bidirectional-arrow-object.l")
;;
;;
(setq *irt-all-objects* '( (bidirectional-arrow) (arrow) (arrow-1000) (askul-gdh-cupboard) (askul-ntr-cupboard) (toshiba-clacio-refrigerator) (hitachi-fiesta-refrigerator) (refrigerator) (toshiba-microwave-oven) (hitachi-18l-oven) (3t-600-clean-locker) (kokuyo-450-locker) (washing-machine) (laundry-machine) (square-600-closet) (plus-900-square-desk) (plus-1700-desk) (plus-1800-desk) (plus-1800w-desk) (plus-2800-desk) (plus-7000-desk) (plus-6200-desk) (uchida-1500-desk) (uchida-1000-desk) (plus-1800-pdesk) (lion-1800-desk) (unknown-1200-desk) (uchida-1800-desk) (uchida-2400-desk) (uchida-monitor-leg) (uchida-desk) (toyama-900-desk) (desk) (askul-1200x700-desk) (askul-1400-desk) (askul-1200-desk) (askul-corner-desk) (askul-2700-desk) (nissen-1100-desk) (karimoku-1200-desk) (kokuyo-600-sidetable) (toyama-900-sidetable) (toyama-1800-sidetable) (fujitsu-42-plasmatv) (hitachi-42-plasmatv) (sharp-52-aquostv) (ideainternational-300-analog-clock) (casio-330-analog-clock) (maruman-55-analog-clock) (docomo-p504is) (sushi-cup) (georgia-emerald-mountain) (iemon) (wanda) (tognana-pasta-dish) (frying-pan) (iron-table) (blue-clear-cup) (white-bread-salad-dish) (flower-tea-cup-dish) (toshiba-iron) (sharp-iron) (akiba-tv-desk) (akiba-desk) (itoki-900-closet) (itoki-900-shelf) (itoki-900-slide-shelf) (naiki-6door-closet) (bariera-900-left) (bariera-900-middle) (bariera-1200-corner) (bariera-1200-middle) (bariera-1200-middle2) (bariera-1200-middle3) (bariera-1200-right) (bariera-1400-middle) (door) (akiba-door) (akiba-door-window) (akiba-room-door) (coe-sofa2) (coe-sofa) (coe-demospace) (sakae-900-desk) (coe-800-shelf) (coe-1200-shelf) (coe-450-shelf) (coe-corner-shelf) (uchida-shelf-1100) (uchida-shelf-1300) (red-chair0) (trashbox) (coe-stairs) (coe-light) (askul-1600-desk) (fridge) (cupboard) (audio_shelf) (asplund-cabinet-4dan) (asplund-cabinet-4danbase) (shelf) (karimoku-kannonbiraki-cabinet) (tray) (bears-panel) (brown-board) (multibox-shelf) (single-arrow) (cardboard-box) (chair) (kettle) (knife) (sponge) (dining-table) (cup) (mug-cup) (ball) (book) (petbottle) (pockey) (plasmatv) (hrp2-rack) (hrp2-shelf) (coffee-cup-cup) (coffee-cup-saucer) (picaladder) (azuma-broom) (azuma-short-broom) (bamboo-broom) (broom) (elevator_call_panel) (elevator_inside_panel) (elevator_call_panel_eng2) (elevator_inside_panel_eng2) (chessboard-108-9x7) (chessboard-30-7x5) (chessboard-30-6x5) (chessboard-10-3x3) (foldable-desk) (gifuplastic-900-cart) (gifuplastic-720-cart) (darkgoldenrod-stairs) (misumi-handrail) (dewalt-drill) (unknown-side-table) (73b2-cupboard-without-door) (maxvalu-supermarket-basket) (asahikasei-small-basket) (hasegawa-ladder) (alinco-ccst-100-ladder) (alinco-tbf-5-ladder)))