;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib;;
;;
(provide :drcbox-scene "models/drcbox-scene.l")

(load "models/drcbox-hose-reel-object.l")
(load "models/drcbox-fire-hose-fitting-object.l")
(load "models/drcbox-drill-table-object.l")
(load "models/drcbox-drill-wall-object.l")
(load "models/drcbox-valve-bar-object.l")
(load "models/drcbox-valve-large-object.l")
(load "models/drcbox-valve-small-object.l")
(load "models/drcbox-drcbox-base-object.l")

(defclass drcbox-scene
  :super scene-model
  :slots ())
(defmethod drcbox-scene
  (:init
   (&rest args &key (name "drcbox") &allow-other-keys)
   (let ()
     (send-super*
      :init :name name :objects
      (list
 (send (drcbox-hose-reel) :transform (make-coords :pos (float-vector 312.0 462.0 1006.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (drcbox-fire-hose-fitting) :transform (make-coords :pos (float-vector -225.0 462.0 993.0) :rot #2f((0.0 1.0 0.0) (0.0 0.0 1.0) (1.0 0.0 0.0))))
 (send (drcbox-drill-table) :transform (make-coords :pos (float-vector 547.0 -410.5 866.0) :rot #2f((2.220446e-16 -1.0 0.0) (1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (drcbox-drill-wall) :transform (make-coords :pos (float-vector 0.0 -444.0 693.0) :rot #2f((2.220446e-16 -1.0 0.0) (1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (drcbox-valve-bar) :transform (make-coords :pos (float-vector -182.75 -390.0 1063.0) :rot #2f((0.0 0.0 -1.0) (1.0 0.0 0.0) (0.0 -1.0 0.0))))
 (send (drcbox-valve-large) :transform (make-coords :pos (float-vector -196.75 -60.0 1268.0) :rot #2f((2.220446e-16 0.0 -1.0) (0.0 1.0 0.0) (1.0 0.0 2.220446e-16))))
 (send (drcbox-valve-small) :transform (make-coords :pos (float-vector -141.75 385.0 1065.0) :rot #2f((2.220446e-16 0.0 -1.0) (0.0 1.0 0.0) (1.0 0.0 2.220446e-16))))
 (send (drcbox-drcbox-base) :transform (make-coords :pos (float-vector 0.0 0.0 0.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
       )
      args)
     ))
  )

(defun drcbox (&rest args) (setq *drcbox* (instance* drcbox-scene :init args)))
