;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defclass sharp-iron-object
  :super cascaded-link
  :slots (sensors
         ))
(defmethod sharp-iron-object
  (:init
   (&rest args &key (name "sharp-iron") (pos (float-vector 0 0 0)) (rot (unit-matrix 3)) &allow-other-keys)
   (let (c bc
         blink0
                 )
     (send-super* :init :name name args)

     ;; definition of link

     ;; definition of :sharp-iron-bodyset2
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 57.0 130.0 0.0) (float-vector 62.0 130.0 100.0) (float-vector -38.0 130.0 100.0) (float-vector -33.0 130.0 0.0) (float-vector -28.0 130.0 -50.0) (float-vector -18.0 130.0 -80.0) (float-vector -8.0 130.0 -90.0) (float-vector 2.0 130.0 -95.0) (float-vector 12.0 130.0 -100.0) (float-vector 22.0 130.0 -95.0) (float-vector 32.0 130.0 -90.0) (float-vector 42.0 130.0 -80.0) (float-vector 52.0 130.0 -50.0)))
         (instance face :init :vertices (list (float-vector 62.0 135.0 100.0) (float-vector 57.0 135.0 0.0) (float-vector 52.0 135.0 -50.0) (float-vector 42.0 135.0 -80.0) (float-vector 32.0 135.0 -90.0) (float-vector 22.0 135.0 -95.0) (float-vector 12.0 135.0 -100.0) (float-vector 2.0 135.0 -95.0) (float-vector -8.0 135.0 -90.0) (float-vector -18.0 135.0 -80.0) (float-vector -28.0 135.0 -50.0) (float-vector -33.0 135.0 0.0) (float-vector -38.0 135.0 100.0)))
         (instance face :init :vertices (list (float-vector 57.0 130.0 0.0) (float-vector 57.0 135.0 0.0) (float-vector 62.0 135.0 100.0) (float-vector 62.0 130.0 100.0)))
         (instance face :init :vertices (list (float-vector 52.0 130.0 -50.0) (float-vector 52.0 135.0 -50.0) (float-vector 57.0 135.0 0.0) (float-vector 57.0 130.0 0.0)))
         (instance face :init :vertices (list (float-vector 42.0 130.0 -80.0) (float-vector 42.0 135.0 -80.0) (float-vector 52.0 135.0 -50.0) (float-vector 52.0 130.0 -50.0)))
         (instance face :init :vertices (list (float-vector 32.0 130.0 -90.0) (float-vector 32.0 135.0 -90.0) (float-vector 42.0 135.0 -80.0) (float-vector 42.0 130.0 -80.0)))
         (instance face :init :vertices (list (float-vector 22.0 130.0 -95.0) (float-vector 22.0 135.0 -95.0) (float-vector 32.0 135.0 -90.0) (float-vector 32.0 130.0 -90.0)))
         (instance face :init :vertices (list (float-vector 12.0 130.0 -100.0) (float-vector 12.0 135.0 -100.0) (float-vector 22.0 135.0 -95.0) (float-vector 22.0 130.0 -95.0)))
         (instance face :init :vertices (list (float-vector 2.0 130.0 -95.0) (float-vector 2.0 135.0 -95.0) (float-vector 12.0 135.0 -100.0) (float-vector 12.0 130.0 -100.0)))
         (instance face :init :vertices (list (float-vector -8.0 130.0 -90.0) (float-vector -8.0 135.0 -90.0) (float-vector 2.0 135.0 -95.0) (float-vector 2.0 130.0 -95.0)))
         (instance face :init :vertices (list (float-vector -18.0 130.0 -80.0) (float-vector -18.0 135.0 -80.0) (float-vector -8.0 135.0 -90.0) (float-vector -8.0 130.0 -90.0)))
         (instance face :init :vertices (list (float-vector -28.0 130.0 -50.0) (float-vector -28.0 135.0 -50.0) (float-vector -18.0 135.0 -80.0) (float-vector -18.0 130.0 -80.0)))
         (instance face :init :vertices (list (float-vector -33.0 130.0 0.0) (float-vector -33.0 135.0 0.0) (float-vector -28.0 135.0 -50.0) (float-vector -28.0 130.0 -50.0)))
         (instance face :init :vertices (list (float-vector -38.0 130.0 100.0) (float-vector -38.0 135.0 100.0) (float-vector -33.0 135.0 0.0) (float-vector -33.0 130.0 0.0)))
         (instance face :init :vertices (list (float-vector 62.0 130.0 100.0) (float-vector 62.0 135.0 100.0) (float-vector -38.0 135.0 100.0) (float-vector -38.0 130.0 100.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 62.0 120.0 100.0) (float-vector 62.0 120.0 130.0) (float-vector -38.0 120.0 130.0) (float-vector -38.0 120.0 100.0) (float-vector -33.0 120.0 0.0) (float-vector -28.0 120.0 -50.0) (float-vector -18.0 120.0 -80.0) (float-vector -8.0 120.0 -90.0) (float-vector 2.0 120.0 -95.0) (float-vector 12.0 120.0 -100.0) (float-vector 22.0 120.0 -95.0) (float-vector 32.0 120.0 -90.0) (float-vector 42.0 120.0 -80.0) (float-vector 52.0 120.0 -50.0) (float-vector 57.0 120.0 0.0)))
         (instance face :init :vertices (list (float-vector 62.0 130.0 130.0) (float-vector 62.0 130.0 100.0) (float-vector 57.0 130.0 0.0) (float-vector 52.0 130.0 -50.0) (float-vector 42.0 130.0 -80.0) (float-vector 32.0 130.0 -90.0) (float-vector 22.0 130.0 -95.0) (float-vector 12.0 130.0 -100.0) (float-vector 2.0 130.0 -95.0) (float-vector -8.0 130.0 -90.0) (float-vector -18.0 130.0 -80.0) (float-vector -28.0 130.0 -50.0) (float-vector -33.0 130.0 0.0) (float-vector -38.0 130.0 100.0) (float-vector -38.0 130.0 130.0)))
         (instance face :init :vertices (list (float-vector 62.0 120.0 100.0) (float-vector 62.0 130.0 100.0) (float-vector 62.0 130.0 130.0) (float-vector 62.0 120.0 130.0)))
         (instance face :init :vertices (list (float-vector 57.0 120.0 0.0) (float-vector 57.0 130.0 0.0) (float-vector 62.0 130.0 100.0) (float-vector 62.0 120.0 100.0)))
         (instance face :init :vertices (list (float-vector 52.0 120.0 -50.0) (float-vector 52.0 130.0 -50.0) (float-vector 57.0 130.0 0.0) (float-vector 57.0 120.0 0.0)))
         (instance face :init :vertices (list (float-vector 42.0 120.0 -80.0) (float-vector 42.0 130.0 -80.0) (float-vector 52.0 130.0 -50.0) (float-vector 52.0 120.0 -50.0)))
         (instance face :init :vertices (list (float-vector 32.0 120.0 -90.0) (float-vector 32.0 130.0 -90.0) (float-vector 42.0 130.0 -80.0) (float-vector 42.0 120.0 -80.0)))
         (instance face :init :vertices (list (float-vector 22.0 120.0 -95.0) (float-vector 22.0 130.0 -95.0) (float-vector 32.0 130.0 -90.0) (float-vector 32.0 120.0 -90.0)))
         (instance face :init :vertices (list (float-vector 12.0 120.0 -100.0) (float-vector 12.0 130.0 -100.0) (float-vector 22.0 130.0 -95.0) (float-vector 22.0 120.0 -95.0)))
         (instance face :init :vertices (list (float-vector 2.0 120.0 -95.0) (float-vector 2.0 130.0 -95.0) (float-vector 12.0 130.0 -100.0) (float-vector 12.0 120.0 -100.0)))
         (instance face :init :vertices (list (float-vector -8.0 120.0 -90.0) (float-vector -8.0 130.0 -90.0) (float-vector 2.0 130.0 -95.0) (float-vector 2.0 120.0 -95.0)))
         (instance face :init :vertices (list (float-vector -18.0 120.0 -80.0) (float-vector -18.0 130.0 -80.0) (float-vector -8.0 130.0 -90.0) (float-vector -8.0 120.0 -90.0)))
         (instance face :init :vertices (list (float-vector -28.0 120.0 -50.0) (float-vector -28.0 130.0 -50.0) (float-vector -18.0 130.0 -80.0) (float-vector -18.0 120.0 -80.0)))
         (instance face :init :vertices (list (float-vector -33.0 120.0 0.0) (float-vector -33.0 130.0 0.0) (float-vector -28.0 130.0 -50.0) (float-vector -28.0 120.0 -50.0)))
         (instance face :init :vertices (list (float-vector -38.0 120.0 100.0) (float-vector -38.0 130.0 100.0) (float-vector -33.0 130.0 0.0) (float-vector -33.0 120.0 0.0)))
         (instance face :init :vertices (list (float-vector -38.0 120.0 130.0) (float-vector -38.0 130.0 130.0) (float-vector -38.0 130.0 100.0) (float-vector -38.0 120.0 100.0)))
         (instance face :init :vertices (list (float-vector 62.0 120.0 130.0) (float-vector 62.0 130.0 130.0) (float-vector -38.0 130.0 130.0) (float-vector -38.0 120.0 130.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 57.0 70.0 -1.421085e-14) (float-vector 62.0 70.0 150.0) (float-vector -38.0 70.0 150.0) (float-vector -38.0 70.0 100.0) (float-vector -33.0 70.0 -1.421085e-14) (float-vector -28.0 70.0 -50.0) (float-vector -18.0 70.0 -80.0) (float-vector -8.0 70.0 -90.0) (float-vector 2.0 70.0 -95.0) (float-vector 12.0 70.0 -100.0) (float-vector 22.0 70.0 -95.0) (float-vector 32.0 70.0 -90.0) (float-vector 42.0 70.0 -80.0) (float-vector 52.0 70.0 -50.0)))
         (instance face :init :vertices (list (float-vector 62.0 120.0 150.0) (float-vector 57.0 120.0 0.0) (float-vector 52.0 120.0 -50.0) (float-vector 42.0 120.0 -80.0) (float-vector 32.0 120.0 -90.0) (float-vector 22.0 120.0 -95.0) (float-vector 12.0 120.0 -100.0) (float-vector 2.0 120.0 -95.0) (float-vector -8.0 120.0 -90.0) (float-vector -18.0 120.0 -80.0) (float-vector -28.0 120.0 -50.0) (float-vector -33.0 120.0 0.0) (float-vector -38.0 120.0 100.0) (float-vector -38.0 120.0 150.0)))
         (instance face :init :vertices (list (float-vector 57.0 70.0 -1.421085e-14) (float-vector 57.0 120.0 0.0) (float-vector 62.0 120.0 150.0) (float-vector 62.0 70.0 150.0)))
         (instance face :init :vertices (list (float-vector 52.0 70.0 -50.0) (float-vector 52.0 120.0 -50.0) (float-vector 57.0 120.0 0.0) (float-vector 57.0 70.0 -1.421085e-14)))
         (instance face :init :vertices (list (float-vector 42.0 70.0 -80.0) (float-vector 42.0 120.0 -80.0) (float-vector 52.0 120.0 -50.0) (float-vector 52.0 70.0 -50.0)))
         (instance face :init :vertices (list (float-vector 32.0 70.0 -90.0) (float-vector 32.0 120.0 -90.0) (float-vector 42.0 120.0 -80.0) (float-vector 42.0 70.0 -80.0)))
         (instance face :init :vertices (list (float-vector 22.0 70.0 -95.0) (float-vector 22.0 120.0 -95.0) (float-vector 32.0 120.0 -90.0) (float-vector 32.0 70.0 -90.0)))
         (instance face :init :vertices (list (float-vector 12.0 70.0 -100.0) (float-vector 12.0 120.0 -100.0) (float-vector 22.0 120.0 -95.0) (float-vector 22.0 70.0 -95.0)))
         (instance face :init :vertices (list (float-vector 2.0 70.0 -95.0) (float-vector 2.0 120.0 -95.0) (float-vector 12.0 120.0 -100.0) (float-vector 12.0 70.0 -100.0)))
         (instance face :init :vertices (list (float-vector -8.0 70.0 -90.0) (float-vector -8.0 120.0 -90.0) (float-vector 2.0 120.0 -95.0) (float-vector 2.0 70.0 -95.0)))
         (instance face :init :vertices (list (float-vector -18.0 70.0 -80.0) (float-vector -18.0 120.0 -80.0) (float-vector -8.0 120.0 -90.0) (float-vector -8.0 70.0 -90.0)))
         (instance face :init :vertices (list (float-vector -28.0 70.0 -50.0) (float-vector -28.0 120.0 -50.0) (float-vector -18.0 120.0 -80.0) (float-vector -18.0 70.0 -80.0)))
         (instance face :init :vertices (list (float-vector -33.0 70.0 -1.421085e-14) (float-vector -33.0 120.0 0.0) (float-vector -28.0 120.0 -50.0) (float-vector -28.0 70.0 -50.0)))
         (instance face :init :vertices (list (float-vector -38.0 70.0 100.0) (float-vector -38.0 120.0 100.0) (float-vector -33.0 120.0 0.0) (float-vector -33.0 70.0 -1.421085e-14)))
         (instance face :init :vertices (list (float-vector -38.0 70.0 150.0) (float-vector -38.0 120.0 150.0) (float-vector -38.0 120.0 100.0) (float-vector -38.0 70.0 100.0)))
         (instance face :init :vertices (list (float-vector 62.0 70.0 150.0) (float-vector 62.0 120.0 150.0) (float-vector -38.0 120.0 150.0) (float-vector -38.0 70.0 150.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 62.0 70.0 150.0) (float-vector -38.0 70.0 150.0) (float-vector -33.0 50.0 150.0) (float-vector -28.0 35.0 150.0) (float-vector -18.0 15.0 150.0) (float-vector -1.0 -2.842171e-14 150.0) (float-vector 24.0 -2.842171e-14 150.0) (float-vector 42.0 15.0 150.0) (float-vector 52.0 35.0 150.0) (float-vector 57.0 50.0 150.0)))
         (instance face :init :vertices (list (float-vector -38.0 70.0 120.0) (float-vector 62.0 70.0 120.0) (float-vector 57.0 50.0 120.0) (float-vector 52.0 35.0 120.0) (float-vector 42.0 15.0 120.0) (float-vector 24.0 -2.842171e-14 120.0) (float-vector -1.0 0.0 120.0) (float-vector -18.0 15.0 120.0) (float-vector -28.0 35.0 120.0) (float-vector -33.0 50.0 120.0)))
         (instance face :init :vertices (list (float-vector 62.0 70.0 150.0) (float-vector 62.0 70.0 120.0) (float-vector -38.0 70.0 120.0) (float-vector -38.0 70.0 150.0)))
         (instance face :init :vertices (list (float-vector 57.0 50.0 150.0) (float-vector 57.0 50.0 120.0) (float-vector 62.0 70.0 120.0) (float-vector 62.0 70.0 150.0)))
         (instance face :init :vertices (list (float-vector 52.0 35.0 150.0) (float-vector 52.0 35.0 120.0) (float-vector 57.0 50.0 120.0) (float-vector 57.0 50.0 150.0)))
         (instance face :init :vertices (list (float-vector 42.0 15.0 150.0) (float-vector 42.0 15.0 120.0) (float-vector 52.0 35.0 120.0) (float-vector 52.0 35.0 150.0)))
         (instance face :init :vertices (list (float-vector 24.0 -2.842171e-14 150.0) (float-vector 24.0 -2.842171e-14 120.0) (float-vector 42.0 15.0 120.0) (float-vector 42.0 15.0 150.0)))
         (instance face :init :vertices (list (float-vector -1.0 -2.842171e-14 150.0) (float-vector -1.0 0.0 120.0) (float-vector 24.0 -2.842171e-14 120.0) (float-vector 24.0 -2.842171e-14 150.0)))
         (instance face :init :vertices (list (float-vector -18.0 15.0 150.0) (float-vector -18.0 15.0 120.0) (float-vector -1.0 0.0 120.0) (float-vector -1.0 -2.842171e-14 150.0)))
         (instance face :init :vertices (list (float-vector -28.0 35.0 150.0) (float-vector -28.0 35.0 120.0) (float-vector -18.0 15.0 120.0) (float-vector -18.0 15.0 150.0)))
         (instance face :init :vertices (list (float-vector -33.0 50.0 150.0) (float-vector -33.0 50.0 120.0) (float-vector -28.0 35.0 120.0) (float-vector -28.0 35.0 150.0)))
         (instance face :init :vertices (list (float-vector -38.0 70.0 150.0) (float-vector -38.0 70.0 120.0) (float-vector -33.0 50.0 120.0) (float-vector -33.0 50.0 150.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 62.0 120.0 130.0) (float-vector 62.0 120.0 150.0) (float-vector -38.0 120.0 150.0) (float-vector -38.0 120.0 130.0)))
         (instance face :init :vertices (list (float-vector 62.0 130.0 150.0) (float-vector 62.0 130.0 130.0) (float-vector -38.0 130.0 130.0) (float-vector -38.0 130.0 150.0)))
         (instance face :init :vertices (list (float-vector 62.0 120.0 130.0) (float-vector 62.0 130.0 130.0) (float-vector 62.0 130.0 150.0) (float-vector 62.0 120.0 150.0)))
         (instance face :init :vertices (list (float-vector -38.0 120.0 130.0) (float-vector -38.0 130.0 130.0) (float-vector 62.0 130.0 130.0) (float-vector 62.0 120.0 130.0)))
         (instance face :init :vertices (list (float-vector -38.0 120.0 150.0) (float-vector -38.0 130.0 150.0) (float-vector -38.0 130.0 130.0) (float-vector -38.0 120.0 130.0)))
         (instance face :init :vertices (list (float-vector 62.0 120.0 150.0) (float-vector 62.0 130.0 150.0) (float-vector -38.0 130.0 150.0) (float-vector -38.0 120.0 150.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 52.0 70.0 -50.0) (float-vector 57.0 70.0 0.0) (float-vector 41.25 2.842171e-14 17.5) (float-vector 38.0 -2.842171e-14 -15.0)))
         (instance face :init :vertices (list (float-vector 42.0 70.0 -80.0) (float-vector 52.0 70.0 -50.0) (float-vector 38.0 -2.842171e-14 -15.0) (float-vector 31.5 0.0 -34.5)))
         (instance face :init :vertices (list (float-vector 32.0 70.0 -90.0) (float-vector 42.0 70.0 -80.0) (float-vector 31.5 0.0 -34.5) (float-vector 25.0 0.0 -41.0)))
         (instance face :init :vertices (list (float-vector 22.0 70.0 -95.0) (float-vector 32.0 70.0 -90.0) (float-vector 25.0 0.0 -41.0) (float-vector 18.5 0.0 -44.25)))
         (instance face :init :vertices (list (float-vector 12.0 70.0 -100.0) (float-vector 22.0 70.0 -95.0) (float-vector 18.5 0.0 -44.25) (float-vector 12.0 0.0 -47.5)))
         (instance face :init :vertices (list (float-vector 2.0 70.0 -95.0) (float-vector 12.0 70.0 -100.0) (float-vector 12.0 0.0 -47.5) (float-vector 5.5 0.0 -44.25)))
         (instance face :init :vertices (list (float-vector -8.0 70.0 -90.0) (float-vector 2.0 70.0 -95.0) (float-vector 5.5 0.0 -44.25) (float-vector -1.0 0.0 -41.0)))
         (instance face :init :vertices (list (float-vector -18.0 70.0 -80.0) (float-vector -8.0 70.0 -90.0) (float-vector -1.0 0.0 -41.0) (float-vector -7.5 0.0 -34.5)))
         (instance face :init :vertices (list (float-vector -28.0 70.0 -50.0) (float-vector -18.0 70.0 -80.0) (float-vector -7.5 0.0 -34.5) (float-vector -14.0 0.0 -15.0)))
         (instance face :init :vertices (list (float-vector -33.0 70.0 0.0) (float-vector -28.0 70.0 -50.0) (float-vector -14.0 0.0 -15.0) (float-vector -17.25 -2.842171e-14 17.5)))
         (instance face :init :vertices (list (float-vector 57.0 70.0 0.0) (float-vector -33.0 70.0 0.0) (float-vector -17.25 -2.842171e-14 17.5) (float-vector 41.25 2.842171e-14 17.5)))
         (instance face :init :vertices (list (float-vector -33.0 70.0 0.0) (float-vector 57.0 70.0 0.0) (float-vector 52.0 70.0 -50.0) (float-vector 42.0 70.0 -80.0) (float-vector 32.0 70.0 -90.0) (float-vector 22.0 70.0 -95.0) (float-vector 12.0 70.0 -100.0) (float-vector 2.0 70.0 -95.0) (float-vector -8.0 70.0 -90.0) (float-vector -18.0 70.0 -80.0) (float-vector -28.0 70.0 -50.0)))
         (instance face :init :vertices (list (float-vector -17.25 -2.842171e-14 17.5) (float-vector -14.0 0.0 -15.0) (float-vector -7.5 0.0 -34.5) (float-vector -1.0 0.0 -41.0) (float-vector 5.5 0.0 -44.25) (float-vector 12.0 0.0 -47.5) (float-vector 18.5 0.0 -44.25) (float-vector 25.0 0.0 -41.0) (float-vector 31.5 0.0 -34.5) (float-vector 38.0 -2.842171e-14 -15.0) (float-vector 41.25 2.842171e-14 17.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0718 -8.0 -16.0) (float-vector 4.0 -8.0 -20.0) (float-vector 5.0718 -8.0 -24.0) (float-vector 8.0 -8.0 -26.9282) (float-vector 12.0 -8.0 -28.0) (float-vector 16.0 -8.0 -26.9282) (float-vector 18.9282 -8.0 -24.0) (float-vector 20.0 -8.0 -20.0) (float-vector 18.9282 -8.0 -16.0) (float-vector 16.0 -8.0 -13.0718) (float-vector 12.0 -8.0 -12.0) (float-vector 8.0 -8.0 -13.0718)))
         (instance face :init :vertices (list (float-vector 4.0 0.0 -20.0) (float-vector 5.0718 0.0 -16.0) (float-vector 8.0 0.0 -13.0718) (float-vector 12.0 0.0 -12.0) (float-vector 16.0 0.0 -13.0718) (float-vector 18.9282 0.0 -16.0) (float-vector 20.0 0.0 -20.0) (float-vector 18.9282 0.0 -24.0) (float-vector 16.0 0.0 -26.9282) (float-vector 12.0 0.0 -28.0) (float-vector 8.0 0.0 -26.9282) (float-vector 5.0718 0.0 -24.0)))
         (instance face :init :vertices (list (float-vector 5.0718 -8.0 -16.0) (float-vector 5.0718 0.0 -16.0) (float-vector 4.0 0.0 -20.0) (float-vector 4.0 -8.0 -20.0)))
         (instance face :init :vertices (list (float-vector 8.0 -8.0 -13.0718) (float-vector 8.0 0.0 -13.0718) (float-vector 5.0718 0.0 -16.0) (float-vector 5.0718 -8.0 -16.0)))
         (instance face :init :vertices (list (float-vector 12.0 -8.0 -12.0) (float-vector 12.0 0.0 -12.0) (float-vector 8.0 0.0 -13.0718) (float-vector 8.0 -8.0 -13.0718)))
         (instance face :init :vertices (list (float-vector 16.0 -8.0 -13.0718) (float-vector 16.0 0.0 -13.0718) (float-vector 12.0 0.0 -12.0) (float-vector 12.0 -8.0 -12.0)))
         (instance face :init :vertices (list (float-vector 18.9282 -8.0 -16.0) (float-vector 18.9282 0.0 -16.0) (float-vector 16.0 0.0 -13.0718) (float-vector 16.0 -8.0 -13.0718)))
         (instance face :init :vertices (list (float-vector 20.0 -8.0 -20.0) (float-vector 20.0 0.0 -20.0) (float-vector 18.9282 0.0 -16.0) (float-vector 18.9282 -8.0 -16.0)))
         (instance face :init :vertices (list (float-vector 18.9282 -8.0 -24.0) (float-vector 18.9282 0.0 -24.0) (float-vector 20.0 0.0 -20.0) (float-vector 20.0 -8.0 -20.0)))
         (instance face :init :vertices (list (float-vector 16.0 -8.0 -26.9282) (float-vector 16.0 0.0 -26.9282) (float-vector 18.9282 0.0 -24.0) (float-vector 18.9282 -8.0 -24.0)))
         (instance face :init :vertices (list (float-vector 12.0 -8.0 -28.0) (float-vector 12.0 0.0 -28.0) (float-vector 16.0 0.0 -26.9282) (float-vector 16.0 -8.0 -26.9282)))
         (instance face :init :vertices (list (float-vector 8.0 -8.0 -26.9282) (float-vector 8.0 0.0 -26.9282) (float-vector 12.0 0.0 -28.0) (float-vector 12.0 -8.0 -28.0)))
         (instance face :init :vertices (list (float-vector 5.0718 -8.0 -24.0) (float-vector 5.0718 0.0 -24.0) (float-vector 8.0 0.0 -26.9282) (float-vector 8.0 -8.0 -26.9282)))
         (instance face :init :vertices (list (float-vector 4.0 -8.0 -20.0) (float-vector 4.0 0.0 -20.0) (float-vector 5.0718 0.0 -24.0) (float-vector 5.0718 -8.0 -24.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 0.0 0.0 120.0) (float-vector 24.0 -2.842171e-14 120.0) (float-vector 23.0 10.0 120.0) (float-vector 22.0 20.0 120.0) (float-vector 18.0 27.0 120.0) (float-vector 12.0 30.0 120.0) (float-vector 6.0 27.0 120.0) (float-vector 2.0 20.0 120.0) (float-vector 1.0 10.0 120.0)))
         (instance face :init :vertices (list (float-vector 24.0 -2.842171e-14 0.0) (float-vector 0.0 0.0 0.0) (float-vector 1.0 10.0 0.0) (float-vector 2.0 20.0 -7.105427e-15) (float-vector 6.0 27.0 0.0) (float-vector 12.0 30.0 0.0) (float-vector 18.0 27.0 0.0) (float-vector 22.0 20.0 -7.105427e-15) (float-vector 23.0 10.0 0.0)))
         (instance face :init :vertices (list (float-vector 0.0 0.0 120.0) (float-vector 0.0 0.0 0.0) (float-vector 24.0 -2.842171e-14 0.0) (float-vector 24.0 -2.842171e-14 120.0)))
         (instance face :init :vertices (list (float-vector 1.0 10.0 120.0) (float-vector 1.0 10.0 0.0) (float-vector 0.0 0.0 0.0) (float-vector 0.0 0.0 120.0)))
         (instance face :init :vertices (list (float-vector 2.0 20.0 120.0) (float-vector 2.0 20.0 -7.105427e-15) (float-vector 1.0 10.0 0.0) (float-vector 1.0 10.0 120.0)))
         (instance face :init :vertices (list (float-vector 6.0 27.0 120.0) (float-vector 6.0 27.0 0.0) (float-vector 2.0 20.0 -7.105427e-15) (float-vector 2.0 20.0 120.0)))
         (instance face :init :vertices (list (float-vector 12.0 30.0 120.0) (float-vector 12.0 30.0 0.0) (float-vector 6.0 27.0 0.0) (float-vector 6.0 27.0 120.0)))
         (instance face :init :vertices (list (float-vector 18.0 27.0 120.0) (float-vector 18.0 27.0 0.0) (float-vector 12.0 30.0 0.0) (float-vector 12.0 30.0 120.0)))
         (instance face :init :vertices (list (float-vector 22.0 20.0 120.0) (float-vector 22.0 20.0 -7.105427e-15) (float-vector 18.0 27.0 0.0) (float-vector 18.0 27.0 120.0)))
         (instance face :init :vertices (list (float-vector 23.0 10.0 120.0) (float-vector 23.0 10.0 0.0) (float-vector 22.0 20.0 -7.105427e-15) (float-vector 22.0 20.0 120.0)))
         (instance face :init :vertices (list (float-vector 24.0 -2.842171e-14 120.0) (float-vector 24.0 -2.842171e-14 0.0) (float-vector 23.0 10.0 0.0) (float-vector 23.0 10.0 120.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :mistyrose4)
     (send (elt bc 1) :set-color :black)
     (send (elt bc 2) :set-color :pink)
     (send (elt bc 3) :set-color :pink)
     (send (elt bc 4) :set-color :pink)
     (send (elt bc 5) :set-color :pink)
     (send (elt bc 6) :set-color :pink)
     (send (elt bc 7) :set-color :pink)
     (setq blink0 (instance bodyset-link :init (make-cascoords) :bodies bc :name :sharp-iron-bodyset2 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of assoc
     (send self :assoc blink0)

     ;; definition of end-coords

     ;; definition of joint


     ;; init-ending 
     (setq links (list blink0))
     (setq joint-list (list))
     (send self :init-ending)
     (send self :move-to (make-coords :pos pos :rot rot))
     (send-all links :worldcoords)

     self))

  (:handle (&rest args) (forward-message-to-all (list ) args))
  (:attention (&rest args) (forward-message-to-all (list ) args))
  (:button (&rest args) (forward-message-to-all (list ) args))
  )

(defun sharp-iron (&rest args) (instance* sharp-iron-object :init args))
;; (format *error-output* "(instance sharp-iron-object :init) for generating model~%")
