extern sgeco_(), sgefa_(), sgesl_(), sgedi_();
extern sgbco_(), sgbfa_(), sgbsl_(), sgbdi_();
extern spoco_(), spofa_(), sposl_(), spodi_();
extern sppco_(), sppfa_(), sppsl_(), sppdi_();
extern spbco_(), spbfa_(), spbsl_(), spbdi_();
extern ssico_(), ssifa_(), ssisl_(), ssidi_();
extern sspco_(), sspfa_(), sspsl_(), sspdi_();
extern chico_(), chifa_(), chisl_(), chidi_();
extern chpco_(), chpfa_(), chpsl_(), chpdi_();
extern strco_(), strsl_(), strdi_(), sgtsl_();
extern sptsl_();
extern schdc_();
extern sqrdc_();
extern sqrsl_();
extern schud_();
extern schdd_();
extern schex_();
extern ssvdc_();
/*
extern sgemv_();
extern sgemm_(); */


linpack()
{ sgbco_();
  sgbfa_();
  sgbsl_();
  sgbdi_();
  spoco_();
  spofa_();
  sposl_();
  spodi_();
  sppco_();
  sppfa_();
  sppsl_();
  sppdi_();
  spbco_();
  spbfa_();
  spbsl_();
  spbdi_();
  ssico_();
  ssifa_();
  ssisl_();
  ssidi_();
  sspco_();
  sspfa_();
  sspsl_();
  sspdi_();
  chico_();
  chifa_();
  chisl_();
  chidi_();
  chpco_();
  chpfa_();
  chpsl_();
  chpdi_();
  strco_();
  strsl_();
  strdi_();
  sgtsl_();
  sptsl_();
  schdc_();
  sqrdc_();
  sqrsl_();
  schud_();
  schdd_();
  schex_();
  ssvdc_();
/*  sgemv_();
  sgemm_(); */
  }
