(in-package "X")

(require :Xdecl "Xdecl.l")
(require :xcanvas "Xcanvas.l")
(require :xtext "Xtext.l")
(require :xscroll "Xscroll.l")
(require :xpanel "Xpanel.l")
(require :xmenu "Xmenu.l")
(require :xitem "Xitem.l")
(require :xcolors "Xcol.l")
;; (require :xpixmap "Xpixmap.l")

