;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defclass tray-object
  :super cascaded-link
  :slots (sensors
         handle0 handle1 handle2 handle3 handle4 handle5
         ))
(defmethod tray-object
  (:init
   (&rest args &key (name "tray") (pos (float-vector 0 0 0)) (rot (unit-matrix 3)) &allow-other-keys)
   (let (c bc
         blink0
                 )
     (send-super* :init :name name args)

     ;; definition of link

     ;; definition of :tray-bodyset2
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 150.0 200.0 15.0) (float-vector -150.0 200.0 15.0) (float-vector -150.0 -200.0 15.0) (float-vector 150.0 -200.0 15.0)))
         (instance face :init :vertices (list (float-vector -150.0 200.0 5.0) (float-vector 150.0 200.0 5.0) (float-vector 150.0 -200.0 5.0) (float-vector -150.0 -200.0 5.0)))
         (instance face :init :vertices (list (float-vector 150.0 200.0 15.0) (float-vector 150.0 200.0 5.0) (float-vector -150.0 200.0 5.0) (float-vector -150.0 200.0 15.0)))
         (instance face :init :vertices (list (float-vector 150.0 -200.0 15.0) (float-vector 150.0 -200.0 5.0) (float-vector 150.0 200.0 5.0) (float-vector 150.0 200.0 15.0)))
         (instance face :init :vertices (list (float-vector -150.0 -200.0 15.0) (float-vector -150.0 -200.0 5.0) (float-vector 150.0 -200.0 5.0) (float-vector 150.0 -200.0 15.0)))
         (instance face :init :vertices (list (float-vector -150.0 200.0 15.0) (float-vector -150.0 200.0 5.0) (float-vector -150.0 -200.0 5.0) (float-vector -150.0 -200.0 15.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 150.0 205.0 37.5) (float-vector -150.0 205.0 37.5) (float-vector -150.0 195.0 37.5) (float-vector 150.0 195.0 37.5)))
         (instance face :init :vertices (list (float-vector -150.0 205.0 7.5) (float-vector 150.0 205.0 7.5) (float-vector 150.0 195.0 7.5) (float-vector -150.0 195.0 7.5)))
         (instance face :init :vertices (list (float-vector 150.0 205.0 37.5) (float-vector 150.0 205.0 7.5) (float-vector -150.0 205.0 7.5) (float-vector -150.0 205.0 37.5)))
         (instance face :init :vertices (list (float-vector 150.0 195.0 37.5) (float-vector 150.0 195.0 7.5) (float-vector 150.0 205.0 7.5) (float-vector 150.0 205.0 37.5)))
         (instance face :init :vertices (list (float-vector -150.0 195.0 37.5) (float-vector -150.0 195.0 7.5) (float-vector 150.0 195.0 7.5) (float-vector 150.0 195.0 37.5)))
         (instance face :init :vertices (list (float-vector -150.0 205.0 37.5) (float-vector -150.0 205.0 7.5) (float-vector -150.0 195.0 7.5) (float-vector -150.0 195.0 37.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 150.0 -195.0 37.5) (float-vector -150.0 -195.0 37.5) (float-vector -150.0 -205.0 37.5) (float-vector 150.0 -205.0 37.5)))
         (instance face :init :vertices (list (float-vector -150.0 -195.0 7.5) (float-vector 150.0 -195.0 7.5) (float-vector 150.0 -205.0 7.5) (float-vector -150.0 -205.0 7.5)))
         (instance face :init :vertices (list (float-vector 150.0 -195.0 37.5) (float-vector 150.0 -195.0 7.5) (float-vector -150.0 -195.0 7.5) (float-vector -150.0 -195.0 37.5)))
         (instance face :init :vertices (list (float-vector 150.0 -205.0 37.5) (float-vector 150.0 -205.0 7.5) (float-vector 150.0 -195.0 7.5) (float-vector 150.0 -195.0 37.5)))
         (instance face :init :vertices (list (float-vector -150.0 -205.0 37.5) (float-vector -150.0 -205.0 7.5) (float-vector 150.0 -205.0 7.5) (float-vector 150.0 -205.0 37.5)))
         (instance face :init :vertices (list (float-vector -150.0 -195.0 37.5) (float-vector -150.0 -195.0 7.5) (float-vector -150.0 -205.0 7.5) (float-vector -150.0 -205.0 37.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 155.0 200.0 37.5) (float-vector 145.0 200.0 37.5) (float-vector 145.0 -200.0 37.5) (float-vector 155.0 -200.0 37.5)))
         (instance face :init :vertices (list (float-vector 145.0 200.0 7.5) (float-vector 155.0 200.0 7.5) (float-vector 155.0 -200.0 7.5) (float-vector 145.0 -200.0 7.5)))
         (instance face :init :vertices (list (float-vector 155.0 200.0 37.5) (float-vector 155.0 200.0 7.5) (float-vector 145.0 200.0 7.5) (float-vector 145.0 200.0 37.5)))
         (instance face :init :vertices (list (float-vector 155.0 -200.0 37.5) (float-vector 155.0 -200.0 7.5) (float-vector 155.0 200.0 7.5) (float-vector 155.0 200.0 37.5)))
         (instance face :init :vertices (list (float-vector 145.0 -200.0 37.5) (float-vector 145.0 -200.0 7.5) (float-vector 155.0 -200.0 7.5) (float-vector 155.0 -200.0 37.5)))
         (instance face :init :vertices (list (float-vector 145.0 200.0 37.5) (float-vector 145.0 200.0 7.5) (float-vector 145.0 -200.0 7.5) (float-vector 145.0 -200.0 37.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -145.0 200.0 37.5) (float-vector -155.0 200.0 37.5) (float-vector -155.0 -200.0 37.5) (float-vector -145.0 -200.0 37.5)))
         (instance face :init :vertices (list (float-vector -155.0 200.0 7.5) (float-vector -145.0 200.0 7.5) (float-vector -145.0 -200.0 7.5) (float-vector -155.0 -200.0 7.5)))
         (instance face :init :vertices (list (float-vector -145.0 200.0 37.5) (float-vector -145.0 200.0 7.5) (float-vector -155.0 200.0 7.5) (float-vector -155.0 200.0 37.5)))
         (instance face :init :vertices (list (float-vector -145.0 -200.0 37.5) (float-vector -145.0 -200.0 7.5) (float-vector -145.0 200.0 7.5) (float-vector -145.0 200.0 37.5)))
         (instance face :init :vertices (list (float-vector -155.0 -200.0 37.5) (float-vector -155.0 -200.0 7.5) (float-vector -145.0 -200.0 7.5) (float-vector -145.0 -200.0 37.5)))
         (instance face :init :vertices (list (float-vector -155.0 200.0 37.5) (float-vector -155.0 200.0 7.5) (float-vector -155.0 -200.0 7.5) (float-vector -155.0 -200.0 37.5)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :brown)
     (send (elt bc 1) :set-color :brown)
     (send (elt bc 2) :set-color :brown)
     (send (elt bc 3) :set-color :brown)
     (send (elt bc 4) :set-color :brown)
     (setq blink0 (instance bodyset-link :init (make-cascoords) :bodies bc :name :tray-bodyset2 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of assoc
     (send self :assoc blink0)

     ;; definition of end-coords

     ;; definition of joint


     ;; definition of :handle
     (setq handle0 (make-cascoords :pos (float-vector 150.0 0.0 30.0) :rot #2f((0.0 -1.0 0.0) (0.0 0.0 1.0) (-1.0 0.0 0.0)) :name ":rarm-handle"))
     (send blink0 :assoc handle0)
     (setq handle1 (make-cascoords :pos (float-vector -150.0 0.0 30.0) :rot #2f((0.0 -1.0 0.0) (0.0 0.0 1.0) (-1.0 0.0 0.0)) :name ":larm-handle"))
     (send blink0 :assoc handle1)
     (setq handle2 (make-cascoords :pos (float-vector 150.0 -160.0 30.0) :rot #2f((-0.707107 -6.106227e-16 -0.707107) (0.707107 -2.775558e-16 -0.707107) (2.775558e-16 -1.0 6.106227e-16)) :name "handle2"))
     (send blink0 :assoc handle2)
     (setq handle3 (make-cascoords :pos (float-vector 150.0 160.0 30.0) :rot #2f((-0.707107 6.106227e-16 -0.707107) (-0.707107 -2.775558e-16 0.707107) (2.775558e-16 1.0 6.106227e-16)) :name "handle3"))
     (send blink0 :assoc handle3)
     (setq handle4 (make-cascoords :pos (float-vector -150.0 -160.0 30.0) :rot #2f((0.707107 -6.661338e-16 -0.707107) (0.707107 5.828671e-16 0.707107) (-5.551115e-17 -1.0 8.604228e-16)) :name "handle4"))
     (send blink0 :assoc handle4)
     (setq handle5 (make-cascoords :pos (float-vector -150.0 160.0 30.0) :rot #2f((0.707107 6.661338e-16 -0.707107) (-0.707107 5.828671e-16 -0.707107) (-5.551115e-17 1.0 8.604228e-16)) :name "handle5"))
     (send blink0 :assoc handle5)

     ;; init-ending 
     (setq links (list blink0))
     (setq joint-list (list))
     (send self :init-ending)
     (send self :move-to (make-coords :pos pos :rot rot))
     (send-all links :worldcoords)

     self))

  (:handle (&rest args) (forward-message-to-all (list  handle0 handle1 handle2 handle3 handle4 handle5) args))
  (:handle-rarm-handle (&rest args) (forward-message-to handle0 args))
  (:handle-larm-handle (&rest args) (forward-message-to handle1 args))
  (:handle-handle2 (&rest args) (forward-message-to handle2 args))
  (:handle-handle3 (&rest args) (forward-message-to handle3 args))
  (:handle-handle4 (&rest args) (forward-message-to handle4 args))
  (:handle-handle5 (&rest args) (forward-message-to handle5 args))
  (:attention (&rest args) (forward-message-to-all (list ) args))
  (:button (&rest args) (forward-message-to-all (list ) args))
  )

(defun tray (&rest args) (instance* tray-object :init args))
;; (format *error-output* "(instance tray-object :init) for generating model~%")
