;; a driver program to compile constrained motion computation program

(comp:compile-file-if-src-newer "model2constRobust.l")
(comp:compile-file-if-src-newer "drawconst.l")
(comp:compile-file-if-src-newer "inequalities.l")


