;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defclass chessboard-108-9x7-object
  :super cascaded-link
  :slots (sensors
         attention0
         ))
(defmethod chessboard-108-9x7-object
  (:init
   (&rest args &key (name "chessboard-108-9x7") (pos (float-vector 0 0 0)) (rot (unit-matrix 3)) &allow-other-keys)
   (let (c bc
         blink0
                 )
     (send-super* :init :name name args)

     ;; definition of link

     ;; definition of 'chessboard-108-9x7
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 600.0 425.0 4.5) (float-vector -600.0 425.0 4.5) (float-vector -600.0 -425.0 4.5) (float-vector 600.0 -425.0 4.5)))
         (instance face :init :vertices (list (float-vector -600.0 425.0 -4.5) (float-vector 600.0 425.0 -4.5) (float-vector 600.0 -425.0 -4.5) (float-vector -600.0 -425.0 -4.5)))
         (instance face :init :vertices (list (float-vector 600.0 425.0 4.5) (float-vector 600.0 425.0 -4.5) (float-vector -600.0 425.0 -4.5) (float-vector -600.0 425.0 4.5)))
         (instance face :init :vertices (list (float-vector 600.0 -425.0 4.5) (float-vector 600.0 -425.0 -4.5) (float-vector 600.0 425.0 -4.5) (float-vector 600.0 425.0 4.5)))
         (instance face :init :vertices (list (float-vector -600.0 -425.0 4.5) (float-vector -600.0 -425.0 -4.5) (float-vector 600.0 -425.0 -4.5) (float-vector 600.0 -425.0 4.5)))
         (instance face :init :vertices (list (float-vector -600.0 425.0 4.5) (float-vector -600.0 425.0 -4.5) (float-vector -600.0 -425.0 -4.5) (float-vector -600.0 -425.0 4.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 378.0 5.0) (float-vector 378.0 378.0 5.0) (float-vector 378.0 270.0 5.0) (float-vector 486.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 378.0 -5.0) (float-vector 486.0 378.0 -5.0) (float-vector 486.0 270.0 -5.0) (float-vector 378.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 378.0 5.0) (float-vector 486.0 378.0 -5.0) (float-vector 378.0 378.0 -5.0) (float-vector 378.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 270.0 5.0) (float-vector 486.0 270.0 -5.0) (float-vector 486.0 378.0 -5.0) (float-vector 486.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 270.0 5.0) (float-vector 378.0 270.0 -5.0) (float-vector 486.0 270.0 -5.0) (float-vector 486.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 378.0 5.0) (float-vector 378.0 378.0 -5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 270.0 5.0) (float-vector 378.0 270.0 5.0) (float-vector 378.0 162.0 5.0) (float-vector 486.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 270.0 -5.0) (float-vector 486.0 270.0 -5.0) (float-vector 486.0 162.0 -5.0) (float-vector 378.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 270.0 5.0) (float-vector 486.0 270.0 -5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 162.0 5.0) (float-vector 486.0 162.0 -5.0) (float-vector 486.0 270.0 -5.0) (float-vector 486.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 162.0 5.0) (float-vector 378.0 162.0 -5.0) (float-vector 486.0 162.0 -5.0) (float-vector 486.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 270.0 5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 162.0 5.0) (float-vector 378.0 162.0 5.0) (float-vector 378.0 54.0 5.0) (float-vector 486.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 162.0 -5.0) (float-vector 486.0 162.0 -5.0) (float-vector 486.0 54.0 -5.0) (float-vector 378.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 162.0 5.0) (float-vector 486.0 162.0 -5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 54.0 5.0) (float-vector 486.0 54.0 -5.0) (float-vector 486.0 162.0 -5.0) (float-vector 486.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 54.0 5.0) (float-vector 378.0 54.0 -5.0) (float-vector 486.0 54.0 -5.0) (float-vector 486.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 162.0 5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 54.0 5.0) (float-vector 378.0 54.0 5.0) (float-vector 378.0 -54.0 5.0) (float-vector 486.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 54.0 -5.0) (float-vector 486.0 54.0 -5.0) (float-vector 486.0 -54.0 -5.0) (float-vector 378.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 54.0 5.0) (float-vector 486.0 54.0 -5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -54.0 5.0) (float-vector 486.0 -54.0 -5.0) (float-vector 486.0 54.0 -5.0) (float-vector 486.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -54.0 5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 486.0 -54.0 -5.0) (float-vector 486.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 54.0 5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 -54.0 5.0) (float-vector 378.0 -54.0 5.0) (float-vector 378.0 -162.0 5.0) (float-vector 486.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -54.0 -5.0) (float-vector 486.0 -54.0 -5.0) (float-vector 486.0 -162.0 -5.0) (float-vector 378.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -54.0 5.0) (float-vector 486.0 -54.0 -5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -162.0 5.0) (float-vector 486.0 -162.0 -5.0) (float-vector 486.0 -54.0 -5.0) (float-vector 486.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -162.0 5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 486.0 -162.0 -5.0) (float-vector 486.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -54.0 5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 -162.0 5.0) (float-vector 378.0 -162.0 5.0) (float-vector 378.0 -270.0 5.0) (float-vector 486.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -162.0 -5.0) (float-vector 486.0 -162.0 -5.0) (float-vector 486.0 -270.0 -5.0) (float-vector 378.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -162.0 5.0) (float-vector 486.0 -162.0 -5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -270.0 5.0) (float-vector 486.0 -270.0 -5.0) (float-vector 486.0 -162.0 -5.0) (float-vector 486.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -270.0 5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 486.0 -270.0 -5.0) (float-vector 486.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -162.0 5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 486.0 -270.0 5.0) (float-vector 378.0 -270.0 5.0) (float-vector 378.0 -378.0 5.0) (float-vector 486.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -270.0 -5.0) (float-vector 486.0 -270.0 -5.0) (float-vector 486.0 -378.0 -5.0) (float-vector 378.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -270.0 5.0) (float-vector 486.0 -270.0 -5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 486.0 -378.0 5.0) (float-vector 486.0 -378.0 -5.0) (float-vector 486.0 -270.0 -5.0) (float-vector 486.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -378.0 5.0) (float-vector 378.0 -378.0 -5.0) (float-vector 486.0 -378.0 -5.0) (float-vector 486.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -270.0 5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -378.0 -5.0) (float-vector 378.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 378.0 5.0) (float-vector 270.0 378.0 5.0) (float-vector 270.0 270.0 5.0) (float-vector 378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 378.0 -5.0) (float-vector 378.0 378.0 -5.0) (float-vector 378.0 270.0 -5.0) (float-vector 270.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 378.0 5.0) (float-vector 378.0 378.0 -5.0) (float-vector 270.0 378.0 -5.0) (float-vector 270.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 270.0 5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 378.0 -5.0) (float-vector 378.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 270.0 5.0) (float-vector 270.0 270.0 -5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 378.0 5.0) (float-vector 270.0 378.0 -5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 270.0 5.0) (float-vector 270.0 270.0 5.0) (float-vector 270.0 162.0 5.0) (float-vector 378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 270.0 -5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 162.0 -5.0) (float-vector 270.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 270.0 5.0) (float-vector 378.0 270.0 -5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 162.0 5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 270.0 -5.0) (float-vector 378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 162.0 5.0) (float-vector 270.0 162.0 -5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 270.0 5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 162.0 5.0) (float-vector 270.0 162.0 5.0) (float-vector 270.0 54.0 5.0) (float-vector 378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 162.0 -5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 54.0 -5.0) (float-vector 270.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 162.0 5.0) (float-vector 378.0 162.0 -5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 54.0 5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 162.0 -5.0) (float-vector 378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 54.0 5.0) (float-vector 270.0 54.0 -5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 162.0 5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 54.0 5.0) (float-vector 270.0 54.0 5.0) (float-vector 270.0 -54.0 5.0) (float-vector 378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 54.0 -5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 270.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 54.0 5.0) (float-vector 378.0 54.0 -5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -54.0 5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 54.0 -5.0) (float-vector 378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -54.0 5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 54.0 5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 -54.0 5.0) (float-vector 270.0 -54.0 5.0) (float-vector 270.0 -162.0 5.0) (float-vector 378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -54.0 -5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 270.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -54.0 5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -162.0 5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -54.0 -5.0) (float-vector 378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -162.0 5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -54.0 5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 -162.0 5.0) (float-vector 270.0 -162.0 5.0) (float-vector 270.0 -270.0 5.0) (float-vector 378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -162.0 -5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 270.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -162.0 5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -270.0 5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -162.0 -5.0) (float-vector 378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -270.0 5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -162.0 5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 378.0 -270.0 5.0) (float-vector 270.0 -270.0 5.0) (float-vector 270.0 -378.0 5.0) (float-vector 378.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -270.0 -5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -378.0 -5.0) (float-vector 270.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -270.0 5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 378.0 -378.0 5.0) (float-vector 378.0 -378.0 -5.0) (float-vector 378.0 -270.0 -5.0) (float-vector 378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -378.0 5.0) (float-vector 270.0 -378.0 -5.0) (float-vector 378.0 -378.0 -5.0) (float-vector 378.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -270.0 5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -378.0 -5.0) (float-vector 270.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 378.0 5.0) (float-vector 162.0 378.0 5.0) (float-vector 162.0 270.0 5.0) (float-vector 270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 378.0 -5.0) (float-vector 270.0 378.0 -5.0) (float-vector 270.0 270.0 -5.0) (float-vector 162.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 378.0 5.0) (float-vector 270.0 378.0 -5.0) (float-vector 162.0 378.0 -5.0) (float-vector 162.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 270.0 5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 378.0 -5.0) (float-vector 270.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 270.0 5.0) (float-vector 162.0 270.0 -5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 378.0 5.0) (float-vector 162.0 378.0 -5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 270.0 5.0) (float-vector 162.0 270.0 5.0) (float-vector 162.0 162.0 5.0) (float-vector 270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 270.0 -5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 162.0 -5.0) (float-vector 162.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 270.0 5.0) (float-vector 270.0 270.0 -5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 162.0 5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 270.0 -5.0) (float-vector 270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 162.0 5.0) (float-vector 162.0 162.0 -5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 270.0 5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 162.0 5.0) (float-vector 162.0 162.0 5.0) (float-vector 162.0 54.0 5.0) (float-vector 270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 162.0 -5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 54.0 -5.0) (float-vector 162.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 162.0 5.0) (float-vector 270.0 162.0 -5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 54.0 5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 162.0 -5.0) (float-vector 270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 54.0 5.0) (float-vector 162.0 54.0 -5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 162.0 5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 54.0 5.0) (float-vector 162.0 54.0 5.0) (float-vector 162.0 -54.0 5.0) (float-vector 270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 54.0 -5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 162.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 54.0 5.0) (float-vector 270.0 54.0 -5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -54.0 5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 54.0 -5.0) (float-vector 270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -54.0 5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 54.0 5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 -54.0 5.0) (float-vector 162.0 -54.0 5.0) (float-vector 162.0 -162.0 5.0) (float-vector 270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -54.0 -5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 162.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -54.0 5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -162.0 5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -54.0 -5.0) (float-vector 270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -162.0 5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -54.0 5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 -162.0 5.0) (float-vector 162.0 -162.0 5.0) (float-vector 162.0 -270.0 5.0) (float-vector 270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -162.0 -5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 162.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -162.0 5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -270.0 5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -162.0 -5.0) (float-vector 270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -270.0 5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -162.0 5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 270.0 -270.0 5.0) (float-vector 162.0 -270.0 5.0) (float-vector 162.0 -378.0 5.0) (float-vector 270.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -270.0 -5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -378.0 -5.0) (float-vector 162.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -270.0 5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 270.0 -378.0 5.0) (float-vector 270.0 -378.0 -5.0) (float-vector 270.0 -270.0 -5.0) (float-vector 270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -378.0 5.0) (float-vector 162.0 -378.0 -5.0) (float-vector 270.0 -378.0 -5.0) (float-vector 270.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -270.0 5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -378.0 -5.0) (float-vector 162.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 378.0 5.0) (float-vector 54.0 378.0 5.0) (float-vector 54.0 270.0 5.0) (float-vector 162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 378.0 -5.0) (float-vector 162.0 378.0 -5.0) (float-vector 162.0 270.0 -5.0) (float-vector 54.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 378.0 5.0) (float-vector 162.0 378.0 -5.0) (float-vector 54.0 378.0 -5.0) (float-vector 54.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 270.0 5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 378.0 -5.0) (float-vector 162.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 270.0 5.0) (float-vector 54.0 270.0 -5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 378.0 5.0) (float-vector 54.0 378.0 -5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 270.0 5.0) (float-vector 54.0 270.0 5.0) (float-vector 54.0 162.0 5.0) (float-vector 162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 270.0 -5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 162.0 -5.0) (float-vector 54.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 270.0 5.0) (float-vector 162.0 270.0 -5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 162.0 5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 270.0 -5.0) (float-vector 162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 162.0 5.0) (float-vector 54.0 162.0 -5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 270.0 5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 162.0 5.0) (float-vector 54.0 162.0 5.0) (float-vector 54.0 54.0 5.0) (float-vector 162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 162.0 -5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 54.0 -5.0) (float-vector 54.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 162.0 5.0) (float-vector 162.0 162.0 -5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 54.0 5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 162.0 -5.0) (float-vector 162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 54.0 5.0) (float-vector 54.0 54.0 -5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 162.0 5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 54.0 5.0) (float-vector 54.0 54.0 5.0) (float-vector 54.0 -54.0 5.0) (float-vector 162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 54.0 -5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 54.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 54.0 5.0) (float-vector 162.0 54.0 -5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -54.0 5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 54.0 -5.0) (float-vector 162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -54.0 5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 54.0 5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 -54.0 5.0) (float-vector 54.0 -54.0 5.0) (float-vector 54.0 -162.0 5.0) (float-vector 162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -54.0 -5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 54.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -54.0 5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -162.0 5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -54.0 -5.0) (float-vector 162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -162.0 5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -54.0 5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 -162.0 5.0) (float-vector 54.0 -162.0 5.0) (float-vector 54.0 -270.0 5.0) (float-vector 162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -162.0 -5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 54.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -162.0 5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -270.0 5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -162.0 -5.0) (float-vector 162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -270.0 5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -162.0 5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 162.0 -270.0 5.0) (float-vector 54.0 -270.0 5.0) (float-vector 54.0 -378.0 5.0) (float-vector 162.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -270.0 -5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -378.0 -5.0) (float-vector 54.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -270.0 5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 162.0 -378.0 5.0) (float-vector 162.0 -378.0 -5.0) (float-vector 162.0 -270.0 -5.0) (float-vector 162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -378.0 5.0) (float-vector 54.0 -378.0 -5.0) (float-vector 162.0 -378.0 -5.0) (float-vector 162.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -270.0 5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -378.0 -5.0) (float-vector 54.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 378.0 5.0) (float-vector -54.0 378.0 5.0) (float-vector -54.0 270.0 5.0) (float-vector 54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 378.0 -5.0) (float-vector 54.0 378.0 -5.0) (float-vector 54.0 270.0 -5.0) (float-vector -54.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 378.0 5.0) (float-vector 54.0 378.0 -5.0) (float-vector -54.0 378.0 -5.0) (float-vector -54.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 270.0 5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 378.0 -5.0) (float-vector 54.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 270.0 5.0) (float-vector -54.0 270.0 -5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 378.0 5.0) (float-vector -54.0 378.0 -5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 270.0 5.0) (float-vector -54.0 270.0 5.0) (float-vector -54.0 162.0 5.0) (float-vector 54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 270.0 -5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 162.0 -5.0) (float-vector -54.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 270.0 5.0) (float-vector 54.0 270.0 -5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 162.0 5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 270.0 -5.0) (float-vector 54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 162.0 5.0) (float-vector -54.0 162.0 -5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 270.0 5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 162.0 5.0) (float-vector -54.0 162.0 5.0) (float-vector -54.0 54.0 5.0) (float-vector 54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 162.0 -5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 54.0 -5.0) (float-vector -54.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 162.0 5.0) (float-vector 54.0 162.0 -5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 54.0 5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 162.0 -5.0) (float-vector 54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 54.0 5.0) (float-vector -54.0 54.0 -5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 162.0 5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 54.0 5.0) (float-vector -54.0 54.0 5.0) (float-vector -54.0 -54.0 5.0) (float-vector 54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 54.0 -5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 -54.0 -5.0) (float-vector -54.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 54.0 5.0) (float-vector 54.0 54.0 -5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -54.0 5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 54.0 -5.0) (float-vector 54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -54.0 5.0) (float-vector -54.0 -54.0 -5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 54.0 5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 -54.0 5.0) (float-vector -54.0 -54.0 5.0) (float-vector -54.0 -162.0 5.0) (float-vector 54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -54.0 -5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 -162.0 -5.0) (float-vector -54.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -54.0 5.0) (float-vector 54.0 -54.0 -5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -162.0 5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -54.0 -5.0) (float-vector 54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -162.0 5.0) (float-vector -54.0 -162.0 -5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -54.0 5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 -162.0 5.0) (float-vector -54.0 -162.0 5.0) (float-vector -54.0 -270.0 5.0) (float-vector 54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -162.0 -5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -270.0 -5.0) (float-vector -54.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -162.0 5.0) (float-vector 54.0 -162.0 -5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -270.0 5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -162.0 -5.0) (float-vector 54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -270.0 5.0) (float-vector -54.0 -270.0 -5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -162.0 5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 54.0 -270.0 5.0) (float-vector -54.0 -270.0 5.0) (float-vector -54.0 -378.0 5.0) (float-vector 54.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -270.0 -5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -378.0 -5.0) (float-vector -54.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -270.0 5.0) (float-vector 54.0 -270.0 -5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector 54.0 -378.0 5.0) (float-vector 54.0 -378.0 -5.0) (float-vector 54.0 -270.0 -5.0) (float-vector 54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -378.0 5.0) (float-vector -54.0 -378.0 -5.0) (float-vector 54.0 -378.0 -5.0) (float-vector 54.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -270.0 5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -378.0 -5.0) (float-vector -54.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 378.0 5.0) (float-vector -162.0 378.0 5.0) (float-vector -162.0 270.0 5.0) (float-vector -54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 378.0 -5.0) (float-vector -54.0 378.0 -5.0) (float-vector -54.0 270.0 -5.0) (float-vector -162.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 378.0 5.0) (float-vector -54.0 378.0 -5.0) (float-vector -162.0 378.0 -5.0) (float-vector -162.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 270.0 5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 378.0 -5.0) (float-vector -54.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 270.0 5.0) (float-vector -162.0 270.0 -5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 378.0 5.0) (float-vector -162.0 378.0 -5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 270.0 5.0) (float-vector -162.0 270.0 5.0) (float-vector -162.0 162.0 5.0) (float-vector -54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 270.0 -5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 162.0 -5.0) (float-vector -162.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 270.0 5.0) (float-vector -54.0 270.0 -5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 162.0 5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 270.0 -5.0) (float-vector -54.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 162.0 5.0) (float-vector -162.0 162.0 -5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 270.0 5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 162.0 5.0) (float-vector -162.0 162.0 5.0) (float-vector -162.0 54.0 5.0) (float-vector -54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 162.0 -5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 54.0 -5.0) (float-vector -162.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 162.0 5.0) (float-vector -54.0 162.0 -5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 54.0 5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 162.0 -5.0) (float-vector -54.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 54.0 5.0) (float-vector -162.0 54.0 -5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 162.0 5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 54.0 5.0) (float-vector -162.0 54.0 5.0) (float-vector -162.0 -54.0 5.0) (float-vector -54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 54.0 -5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -162.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 54.0 5.0) (float-vector -54.0 54.0 -5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -54.0 5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 54.0 -5.0) (float-vector -54.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -54.0 5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 54.0 5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 -54.0 5.0) (float-vector -162.0 -54.0 5.0) (float-vector -162.0 -162.0 5.0) (float-vector -54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -54.0 -5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -162.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -54.0 5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -162.0 5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -54.0 -5.0) (float-vector -54.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -162.0 5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -54.0 5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 -162.0 5.0) (float-vector -162.0 -162.0 5.0) (float-vector -162.0 -270.0 5.0) (float-vector -54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -162.0 -5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -162.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -162.0 5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -270.0 5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -162.0 -5.0) (float-vector -54.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -270.0 5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -162.0 5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -54.0 -270.0 5.0) (float-vector -162.0 -270.0 5.0) (float-vector -162.0 -378.0 5.0) (float-vector -54.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -270.0 -5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -378.0 -5.0) (float-vector -162.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -270.0 5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -54.0 -378.0 5.0) (float-vector -54.0 -378.0 -5.0) (float-vector -54.0 -270.0 -5.0) (float-vector -54.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -378.0 5.0) (float-vector -162.0 -378.0 -5.0) (float-vector -54.0 -378.0 -5.0) (float-vector -54.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -270.0 5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -378.0 -5.0) (float-vector -162.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 378.0 5.0) (float-vector -270.0 378.0 5.0) (float-vector -270.0 270.0 5.0) (float-vector -162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 378.0 -5.0) (float-vector -162.0 378.0 -5.0) (float-vector -162.0 270.0 -5.0) (float-vector -270.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 378.0 5.0) (float-vector -162.0 378.0 -5.0) (float-vector -270.0 378.0 -5.0) (float-vector -270.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 270.0 5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 378.0 -5.0) (float-vector -162.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 270.0 5.0) (float-vector -270.0 270.0 -5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 378.0 5.0) (float-vector -270.0 378.0 -5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 270.0 5.0) (float-vector -270.0 270.0 5.0) (float-vector -270.0 162.0 5.0) (float-vector -162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 270.0 -5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 162.0 -5.0) (float-vector -270.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 270.0 5.0) (float-vector -162.0 270.0 -5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 162.0 5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 270.0 -5.0) (float-vector -162.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 162.0 5.0) (float-vector -270.0 162.0 -5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 270.0 5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 162.0 5.0) (float-vector -270.0 162.0 5.0) (float-vector -270.0 54.0 5.0) (float-vector -162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 162.0 -5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 54.0 -5.0) (float-vector -270.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 162.0 5.0) (float-vector -162.0 162.0 -5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 54.0 5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 162.0 -5.0) (float-vector -162.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 54.0 5.0) (float-vector -270.0 54.0 -5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 162.0 5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 54.0 5.0) (float-vector -270.0 54.0 5.0) (float-vector -270.0 -54.0 5.0) (float-vector -162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 54.0 -5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -270.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 54.0 5.0) (float-vector -162.0 54.0 -5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -54.0 5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 54.0 -5.0) (float-vector -162.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -54.0 5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 54.0 5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 -54.0 5.0) (float-vector -270.0 -54.0 5.0) (float-vector -270.0 -162.0 5.0) (float-vector -162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -54.0 -5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -270.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -54.0 5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -162.0 5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -54.0 -5.0) (float-vector -162.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -162.0 5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -54.0 5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 -162.0 5.0) (float-vector -270.0 -162.0 5.0) (float-vector -270.0 -270.0 5.0) (float-vector -162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -162.0 -5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -270.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -162.0 5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -270.0 5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -162.0 -5.0) (float-vector -162.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -270.0 5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -162.0 5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -162.0 -270.0 5.0) (float-vector -270.0 -270.0 5.0) (float-vector -270.0 -378.0 5.0) (float-vector -162.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -270.0 -5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -378.0 -5.0) (float-vector -270.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -270.0 5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -162.0 -378.0 5.0) (float-vector -162.0 -378.0 -5.0) (float-vector -162.0 -270.0 -5.0) (float-vector -162.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -378.0 5.0) (float-vector -270.0 -378.0 -5.0) (float-vector -162.0 -378.0 -5.0) (float-vector -162.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -270.0 5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -378.0 -5.0) (float-vector -270.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 378.0 5.0) (float-vector -378.0 378.0 5.0) (float-vector -378.0 270.0 5.0) (float-vector -270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 378.0 -5.0) (float-vector -270.0 378.0 -5.0) (float-vector -270.0 270.0 -5.0) (float-vector -378.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 378.0 5.0) (float-vector -270.0 378.0 -5.0) (float-vector -378.0 378.0 -5.0) (float-vector -378.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 270.0 5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 378.0 -5.0) (float-vector -270.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 270.0 5.0) (float-vector -378.0 270.0 -5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 378.0 5.0) (float-vector -378.0 378.0 -5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 270.0 5.0) (float-vector -378.0 270.0 5.0) (float-vector -378.0 162.0 5.0) (float-vector -270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 270.0 -5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 162.0 -5.0) (float-vector -378.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 270.0 5.0) (float-vector -270.0 270.0 -5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 162.0 5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 270.0 -5.0) (float-vector -270.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 162.0 5.0) (float-vector -378.0 162.0 -5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 270.0 5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 162.0 5.0) (float-vector -378.0 162.0 5.0) (float-vector -378.0 54.0 5.0) (float-vector -270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 162.0 -5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 54.0 -5.0) (float-vector -378.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 162.0 5.0) (float-vector -270.0 162.0 -5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 54.0 5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 162.0 -5.0) (float-vector -270.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 54.0 5.0) (float-vector -378.0 54.0 -5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 162.0 5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 54.0 5.0) (float-vector -378.0 54.0 5.0) (float-vector -378.0 -54.0 5.0) (float-vector -270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 54.0 -5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -378.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 54.0 5.0) (float-vector -270.0 54.0 -5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -54.0 5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 54.0 -5.0) (float-vector -270.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -54.0 5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 54.0 5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 -54.0 5.0) (float-vector -378.0 -54.0 5.0) (float-vector -378.0 -162.0 5.0) (float-vector -270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -54.0 -5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -378.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -54.0 5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -162.0 5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -54.0 -5.0) (float-vector -270.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -162.0 5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -54.0 5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 -162.0 5.0) (float-vector -378.0 -162.0 5.0) (float-vector -378.0 -270.0 5.0) (float-vector -270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -162.0 -5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -378.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -162.0 5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -270.0 5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -162.0 -5.0) (float-vector -270.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -270.0 5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -162.0 5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -270.0 -270.0 5.0) (float-vector -378.0 -270.0 5.0) (float-vector -378.0 -378.0 5.0) (float-vector -270.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -270.0 -5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -378.0 -5.0) (float-vector -378.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -270.0 5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -270.0 -378.0 5.0) (float-vector -270.0 -378.0 -5.0) (float-vector -270.0 -270.0 -5.0) (float-vector -270.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -378.0 5.0) (float-vector -378.0 -378.0 -5.0) (float-vector -270.0 -378.0 -5.0) (float-vector -270.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -270.0 5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -378.0 -5.0) (float-vector -378.0 -378.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 378.0 5.0) (float-vector -486.0 378.0 5.0) (float-vector -486.0 270.0 5.0) (float-vector -378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 378.0 -5.0) (float-vector -378.0 378.0 -5.0) (float-vector -378.0 270.0 -5.0) (float-vector -486.0 270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 378.0 5.0) (float-vector -378.0 378.0 -5.0) (float-vector -486.0 378.0 -5.0) (float-vector -486.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 270.0 5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 378.0 -5.0) (float-vector -378.0 378.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 270.0 5.0) (float-vector -486.0 270.0 -5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 378.0 5.0) (float-vector -486.0 378.0 -5.0) (float-vector -486.0 270.0 -5.0) (float-vector -486.0 270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 270.0 5.0) (float-vector -486.0 270.0 5.0) (float-vector -486.0 162.0 5.0) (float-vector -378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 270.0 -5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 162.0 -5.0) (float-vector -486.0 162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 270.0 5.0) (float-vector -378.0 270.0 -5.0) (float-vector -486.0 270.0 -5.0) (float-vector -486.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 162.0 5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 270.0 -5.0) (float-vector -378.0 270.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 162.0 5.0) (float-vector -486.0 162.0 -5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 270.0 5.0) (float-vector -486.0 270.0 -5.0) (float-vector -486.0 162.0 -5.0) (float-vector -486.0 162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 162.0 5.0) (float-vector -486.0 162.0 5.0) (float-vector -486.0 54.0 5.0) (float-vector -378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 162.0 -5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 54.0 -5.0) (float-vector -486.0 54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 162.0 5.0) (float-vector -378.0 162.0 -5.0) (float-vector -486.0 162.0 -5.0) (float-vector -486.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 54.0 5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 162.0 -5.0) (float-vector -378.0 162.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 54.0 5.0) (float-vector -486.0 54.0 -5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 162.0 5.0) (float-vector -486.0 162.0 -5.0) (float-vector -486.0 54.0 -5.0) (float-vector -486.0 54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 54.0 5.0) (float-vector -486.0 54.0 5.0) (float-vector -486.0 -54.0 5.0) (float-vector -378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 54.0 -5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -486.0 -54.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 54.0 5.0) (float-vector -378.0 54.0 -5.0) (float-vector -486.0 54.0 -5.0) (float-vector -486.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -54.0 5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 54.0 -5.0) (float-vector -378.0 54.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -54.0 5.0) (float-vector -486.0 -54.0 -5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 54.0 5.0) (float-vector -486.0 54.0 -5.0) (float-vector -486.0 -54.0 -5.0) (float-vector -486.0 -54.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 -54.0 5.0) (float-vector -486.0 -54.0 5.0) (float-vector -486.0 -162.0 5.0) (float-vector -378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -54.0 -5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -486.0 -162.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -54.0 5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -486.0 -54.0 -5.0) (float-vector -486.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -162.0 5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -54.0 -5.0) (float-vector -378.0 -54.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -162.0 5.0) (float-vector -486.0 -162.0 -5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -54.0 5.0) (float-vector -486.0 -54.0 -5.0) (float-vector -486.0 -162.0 -5.0) (float-vector -486.0 -162.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 -162.0 5.0) (float-vector -486.0 -162.0 5.0) (float-vector -486.0 -270.0 5.0) (float-vector -378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -162.0 -5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -486.0 -270.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -162.0 5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -486.0 -162.0 -5.0) (float-vector -486.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -270.0 5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -162.0 -5.0) (float-vector -378.0 -162.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -270.0 5.0) (float-vector -486.0 -270.0 -5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -162.0 5.0) (float-vector -486.0 -162.0 -5.0) (float-vector -486.0 -270.0 -5.0) (float-vector -486.0 -270.0 5.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -378.0 -270.0 5.0) (float-vector -486.0 -270.0 5.0) (float-vector -486.0 -378.0 5.0) (float-vector -378.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -270.0 -5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -378.0 -5.0) (float-vector -486.0 -378.0 -5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -270.0 5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -486.0 -270.0 -5.0) (float-vector -486.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -378.0 -378.0 5.0) (float-vector -378.0 -378.0 -5.0) (float-vector -378.0 -270.0 -5.0) (float-vector -378.0 -270.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -378.0 5.0) (float-vector -486.0 -378.0 -5.0) (float-vector -378.0 -378.0 -5.0) (float-vector -378.0 -378.0 5.0)))
         (instance face :init :vertices (list (float-vector -486.0 -270.0 5.0) (float-vector -486.0 -270.0 -5.0) (float-vector -486.0 -378.0 -5.0) (float-vector -486.0 -378.0 5.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :white)
     (send (elt bc 1) :set-color :black)
     (send (elt bc 2) :set-color :white)
     (send (elt bc 3) :set-color :black)
     (send (elt bc 4) :set-color :white)
     (send (elt bc 5) :set-color :black)
     (send (elt bc 6) :set-color :white)
     (send (elt bc 7) :set-color :black)
     (send (elt bc 8) :set-color :white)
     (send (elt bc 9) :set-color :black)
     (send (elt bc 10) :set-color :white)
     (send (elt bc 11) :set-color :black)
     (send (elt bc 12) :set-color :white)
     (send (elt bc 13) :set-color :black)
     (send (elt bc 14) :set-color :white)
     (send (elt bc 15) :set-color :black)
     (send (elt bc 16) :set-color :white)
     (send (elt bc 17) :set-color :black)
     (send (elt bc 18) :set-color :white)
     (send (elt bc 19) :set-color :black)
     (send (elt bc 20) :set-color :white)
     (send (elt bc 21) :set-color :black)
     (send (elt bc 22) :set-color :white)
     (send (elt bc 23) :set-color :black)
     (send (elt bc 24) :set-color :white)
     (send (elt bc 25) :set-color :black)
     (send (elt bc 26) :set-color :white)
     (send (elt bc 27) :set-color :black)
     (send (elt bc 28) :set-color :white)
     (send (elt bc 29) :set-color :black)
     (send (elt bc 30) :set-color :white)
     (send (elt bc 31) :set-color :black)
     (send (elt bc 32) :set-color :white)
     (send (elt bc 33) :set-color :black)
     (send (elt bc 34) :set-color :white)
     (send (elt bc 35) :set-color :black)
     (send (elt bc 36) :set-color :white)
     (send (elt bc 37) :set-color :black)
     (send (elt bc 38) :set-color :white)
     (send (elt bc 39) :set-color :black)
     (send (elt bc 40) :set-color :white)
     (send (elt bc 41) :set-color :black)
     (send (elt bc 42) :set-color :white)
     (send (elt bc 43) :set-color :black)
     (send (elt bc 44) :set-color :white)
     (send (elt bc 45) :set-color :black)
     (send (elt bc 46) :set-color :white)
     (send (elt bc 47) :set-color :black)
     (send (elt bc 48) :set-color :white)
     (send (elt bc 49) :set-color :black)
     (send (elt bc 50) :set-color :white)
     (send (elt bc 51) :set-color :black)
     (send (elt bc 52) :set-color :white)
     (send (elt bc 53) :set-color :black)
     (send (elt bc 54) :set-color :white)
     (send (elt bc 55) :set-color :black)
     (send (elt bc 56) :set-color :white)
     (send (elt bc 57) :set-color :black)
     (send (elt bc 58) :set-color :white)
     (send (elt bc 59) :set-color :black)
     (send (elt bc 60) :set-color :white)
     (send (elt bc 61) :set-color :black)
     (send (elt bc 62) :set-color :white)
     (send (elt bc 63) :set-color :black)
     (setq blink0 (instance bodyset-link :init (make-cascoords) :bodies bc :name 'chessboard-108-9x7 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of assoc
     (send self :assoc blink0)

     ;; definition of end-coords

     ;; definition of joint


     ;; definition of :attention
     (setq attention0 (make-cascoords :pos (float-vector -378.0 -270.0 0.0) :rot #2f((2.220446e-16 1.0 0.0) (1.0 -2.220446e-16 0.0) (0.0 0.0 -1.0)) :name "origin"))
     (send blink0 :assoc attention0)

     ;; init-ending 
     (setq links (list blink0))
     (setq joint-list (list))
     (send self :init-ending)
     (send self :move-to (make-coords :pos pos :rot rot))
     (send-all links :worldcoords)

     self))

  (:handle (&rest args) (forward-message-to-all (list ) args))
  (:attention (&rest args) (forward-message-to-all (list  attention0) args))
  (:attention-origin (&rest args) (forward-message-to attention0 args))
  (:button (&rest args) (forward-message-to-all (list ) args))
  )

(defun chessboard-108-9x7 (&rest args) (instance* chessboard-108-9x7-object :init args))
;; (format *error-output* "(instance chessboard-108-9x7-object :init) for generating model~%")
