/*
 *   This file was automatically generated by version 1.6 of cextract.
 *   Manual editing not recommended.
 *
 *   Created: Thu Apr 25 14:11:52 1996
 */

#include "eus.h"

extern void glClearColorfv (/* float v[4] */);
extern void glOrtho6f (/* float left, float right, float bottom, float top, float near, float far */);
extern void glOrthofv (/* float v[6] */);
extern void glRotatefv (/* float v[4] */);
extern void glTranslatefv (/* float v[3] */);
extern void auxSolidTeapotfv (/* float v[1] */);
extern void glClipPlanef (/* int pl, float eqn[4] */);
extern void auxSolidSpherefv (/* float v[1] */);
extern void auxWireSpherefv (/* float v[1] */);
extern void auxSolidCylinderfv (/* float v[2] */);
extern void auxSolidConefv (/* float v[2] */);
extern void auxSolidBoxfv (/* float v[3] */);
extern void gluPerspectivefv (/* float v[4] */);
extern void gluLookAtfv (/* float v[9] */);
extern void gluTessCallbackl (/* GLUtriangulatorObj *tobj, GLenum which, void (*fn)() */);
extern void auxSolidTorusfv (/* float v[2] */);
extern void auxSolidCubefv (/* float v[1] */);
extern void auxSolidOctahedronfv (/* float v[1] */);
extern void glClearAccumfv (/* float v[4] */);
extern void glAccumfv (/* int op, float v[1] */);
extern void gluDiskfv (/* GLUquadricObj *qobj, float v[4] */);
extern void gluPartialDiskfv (/* GLUquadricObj *qobj, float v[6] */);
extern void auxWireIcosahedronfv (/* float v[1] */);
extern void glScalefv (/* GLfloat v[3] */);
extern void gluNurbsPropertyfv (/* GLUnurbsObj *nobj, GLenum property, GLfloat *value */);
extern void glMapGrid2fv (/* GLfloat v[6] */);
extern void glBitmapfv (/* GLsizei w, GLsizei h, GLfloat v[4], GLubyte *b */);
extern void glMap1fv (/* GLenum target, GLfloat args[4], GLfloat points[] */);
extern void glMap2fv (/* GLenum target, GLfloat args[8], GLfloat points[] */);
extern void auxSetOneColorfv (/* int color, GLfloat v[3] */);
extern void glFrustumfv (/* GLfloat v[6] */);
extern void gluOrtho2Dfv (/* GLfloat v[4] */);
extern void glPointSizefv (/* GLfloat v[1] */);
extern void glClearIndexfv (/* GLfloat v[1] */);
extern eusinteger_t alloctessinfo (/* GLfloat vert[3], GLint tp, GLfloat tcoord[2], GLint np, GLfloat ncoord[2] */);
extern void tess_vertex_cb (/* struct tessinfo *i */);
extern void glDepthRangefv (/* GLfloat clamp[2] */);
extern void glPolygonOffsetEXTfv (/* GLfloat v[2] */);

