(compile-file "readmacro.l" :cc nil)
(compile-file "object.l"    :cc nil)
(compile-file "packsym.l"   :cc nil)
(compile-file "common.l"    :cc nil)
(compile-file "stream.l"    :cc nil)
(compile-file "string.l"    :cc nil)
(compile-file "loader.l"    :cc nil)
(compile-file "pprint.l"    :cc nil)
(compile-file "process.l"    :cc nil)
(compile-file "hashtab.l"    :cc nil)
(compile-file "array.l"    :cc nil)
(compile-file "extnum.l"    :cc nil)
(compile-file "mathtran.l"    :cc nil)
(compile-file "eusdebug.l"    :cc nil)
(in-package "GEO")
(compile-file "coordinates.l"    :cc nil)

