;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib version 
;;
;;
(defclass room73b2-cupboard-right-object
  :super cascaded-link
  :slots (sensors
         handle0
         joint0 joint1 joint2 joint3 joint4 ))
(defmethod room73b2-cupboard-right-object
  (:init
   (&rest args &key (name "room73b2-cupboard-right") (pos (float-vector 0 0 0)) (rot (unit-matrix 3)) &allow-other-keys)
   (let (c bc
         blink0 blink1 blink2 blink3 blink4 blink5
                 )
     (send-super* :init :name name args)

     ;; definition of link

     ;; definition of :root
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 176.5 293.0 1816.0) (float-vector -176.5 293.0 1816.0) (float-vector -176.5 -293.0 1816.0) (float-vector 176.5 -293.0 1816.0)))
         (instance face :init :vertices (list (float-vector -176.5 293.0 1796.0) (float-vector 176.5 293.0 1796.0) (float-vector 176.5 -293.0 1796.0) (float-vector -176.5 -293.0 1796.0)))
         (instance face :init :vertices (list (float-vector 176.5 293.0 1816.0) (float-vector 176.5 293.0 1796.0) (float-vector -176.5 293.0 1796.0) (float-vector -176.5 293.0 1816.0)))
         (instance face :init :vertices (list (float-vector 176.5 -293.0 1816.0) (float-vector 176.5 -293.0 1796.0) (float-vector 176.5 293.0 1796.0) (float-vector 176.5 293.0 1816.0)))
         (instance face :init :vertices (list (float-vector -176.5 -293.0 1816.0) (float-vector -176.5 -293.0 1796.0) (float-vector 176.5 -293.0 1796.0) (float-vector 176.5 -293.0 1816.0)))
         (instance face :init :vertices (list (float-vector -176.5 293.0 1816.0) (float-vector -176.5 293.0 1796.0) (float-vector -176.5 -293.0 1796.0) (float-vector -176.5 -293.0 1816.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 293.0 84.5) (float-vector -176.5 293.0 84.5) (float-vector -176.5 -293.0 84.5) (float-vector 166.5 -293.0 84.5)))
         (instance face :init :vertices (list (float-vector -176.5 293.0 -0.5) (float-vector 166.5 293.0 -0.5) (float-vector 166.5 -293.0 -0.5) (float-vector -176.5 -293.0 -0.5)))
         (instance face :init :vertices (list (float-vector 166.5 293.0 84.5) (float-vector 166.5 293.0 -0.5) (float-vector -176.5 293.0 -0.5) (float-vector -176.5 293.0 84.5)))
         (instance face :init :vertices (list (float-vector 166.5 -293.0 84.5) (float-vector 166.5 -293.0 -0.5) (float-vector 166.5 293.0 -0.5) (float-vector 166.5 293.0 84.5)))
         (instance face :init :vertices (list (float-vector -176.5 -293.0 84.5) (float-vector -176.5 -293.0 -0.5) (float-vector 166.5 -293.0 -0.5) (float-vector 166.5 -293.0 84.5)))
         (instance face :init :vertices (list (float-vector -176.5 293.0 84.5) (float-vector -176.5 293.0 -0.5) (float-vector -176.5 -293.0 -0.5) (float-vector -176.5 -293.0 84.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -166.0 283.0 1795.5) (float-vector -176.0 283.0 1795.5) (float-vector -176.0 -283.0 1795.5) (float-vector -166.0 -283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.0 283.0 84.5) (float-vector -166.0 283.0 84.5) (float-vector -166.0 -283.0 84.5) (float-vector -176.0 -283.0 84.5)))
         (instance face :init :vertices (list (float-vector -166.0 283.0 1795.5) (float-vector -166.0 283.0 84.5) (float-vector -176.0 283.0 84.5) (float-vector -176.0 283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -166.0 -283.0 1795.5) (float-vector -166.0 -283.0 84.5) (float-vector -166.0 283.0 84.5) (float-vector -166.0 283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.0 -283.0 1795.5) (float-vector -176.0 -283.0 84.5) (float-vector -166.0 -283.0 84.5) (float-vector -166.0 -283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.0 283.0 1795.5) (float-vector -176.0 283.0 84.5) (float-vector -176.0 -283.0 84.5) (float-vector -176.0 -283.0 1795.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 293.0 1795.5) (float-vector -176.5 293.0 1795.5) (float-vector -176.5 283.0 1795.5) (float-vector 166.5 283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.5 293.0 84.5) (float-vector 166.5 293.0 84.5) (float-vector 166.5 283.0 84.5) (float-vector -176.5 283.0 84.5)))
         (instance face :init :vertices (list (float-vector 166.5 293.0 1795.5) (float-vector 166.5 293.0 84.5) (float-vector -176.5 293.0 84.5) (float-vector -176.5 293.0 1795.5)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 1795.5) (float-vector 166.5 283.0 84.5) (float-vector 166.5 293.0 84.5) (float-vector 166.5 293.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.5 283.0 1795.5) (float-vector -176.5 283.0 84.5) (float-vector 166.5 283.0 84.5) (float-vector 166.5 283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.5 293.0 1795.5) (float-vector -176.5 293.0 84.5) (float-vector -176.5 283.0 84.5) (float-vector -176.5 283.0 1795.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 -283.0 1795.5) (float-vector -176.5 -283.0 1795.5) (float-vector -176.5 -293.0 1795.5) (float-vector 166.5 -293.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.5 -283.0 84.5) (float-vector 166.5 -283.0 84.5) (float-vector 166.5 -293.0 84.5) (float-vector -176.5 -293.0 84.5)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 1795.5) (float-vector 166.5 -283.0 84.5) (float-vector -176.5 -283.0 84.5) (float-vector -176.5 -283.0 1795.5)))
         (instance face :init :vertices (list (float-vector 166.5 -293.0 1795.5) (float-vector 166.5 -293.0 84.5) (float-vector 166.5 -283.0 84.5) (float-vector 166.5 -283.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.5 -293.0 1795.5) (float-vector -176.5 -293.0 84.5) (float-vector 166.5 -293.0 84.5) (float-vector 166.5 -293.0 1795.5)))
         (instance face :init :vertices (list (float-vector -176.5 -283.0 1795.5) (float-vector -176.5 -283.0 84.5) (float-vector -176.5 -293.0 84.5) (float-vector -176.5 -293.0 1795.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 283.0 375.0) (float-vector -166.5 283.0 375.0) (float-vector -166.5 -283.0 375.0) (float-vector 166.5 -283.0 375.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 365.0) (float-vector 166.5 283.0 365.0) (float-vector 166.5 -283.0 365.0) (float-vector -166.5 -283.0 365.0)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 375.0) (float-vector 166.5 283.0 365.0) (float-vector -166.5 283.0 365.0) (float-vector -166.5 283.0 375.0)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 375.0) (float-vector 166.5 -283.0 365.0) (float-vector 166.5 283.0 365.0) (float-vector 166.5 283.0 375.0)))
         (instance face :init :vertices (list (float-vector -166.5 -283.0 375.0) (float-vector -166.5 -283.0 365.0) (float-vector 166.5 -283.0 365.0) (float-vector 166.5 -283.0 375.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 375.0) (float-vector -166.5 283.0 365.0) (float-vector -166.5 -283.0 365.0) (float-vector -166.5 -283.0 375.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 283.0 770.0) (float-vector -166.5 283.0 770.0) (float-vector -166.5 -283.0 770.0) (float-vector 166.5 -283.0 770.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 760.0) (float-vector 166.5 283.0 760.0) (float-vector 166.5 -283.0 760.0) (float-vector -166.5 -283.0 760.0)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 770.0) (float-vector 166.5 283.0 760.0) (float-vector -166.5 283.0 760.0) (float-vector -166.5 283.0 770.0)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 770.0) (float-vector 166.5 -283.0 760.0) (float-vector 166.5 283.0 760.0) (float-vector 166.5 283.0 770.0)))
         (instance face :init :vertices (list (float-vector -166.5 -283.0 770.0) (float-vector -166.5 -283.0 760.0) (float-vector 166.5 -283.0 760.0) (float-vector 166.5 -283.0 770.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 770.0) (float-vector -166.5 283.0 760.0) (float-vector -166.5 -283.0 760.0) (float-vector -166.5 -283.0 770.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 283.0 890.0) (float-vector -166.5 283.0 890.0) (float-vector -166.5 -283.0 890.0) (float-vector 166.5 -283.0 890.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 880.0) (float-vector 166.5 283.0 880.0) (float-vector 166.5 -283.0 880.0) (float-vector -166.5 -283.0 880.0)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 890.0) (float-vector 166.5 283.0 880.0) (float-vector -166.5 283.0 880.0) (float-vector -166.5 283.0 890.0)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 890.0) (float-vector 166.5 -283.0 880.0) (float-vector 166.5 283.0 880.0) (float-vector 166.5 283.0 890.0)))
         (instance face :init :vertices (list (float-vector -166.5 -283.0 890.0) (float-vector -166.5 -283.0 880.0) (float-vector 166.5 -283.0 880.0) (float-vector 166.5 -283.0 890.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 890.0) (float-vector -166.5 283.0 880.0) (float-vector -166.5 -283.0 880.0) (float-vector -166.5 -283.0 890.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 283.0 1110.0) (float-vector -166.5 283.0 1110.0) (float-vector -166.5 -283.0 1110.0) (float-vector 166.5 -283.0 1110.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 1100.0) (float-vector 166.5 283.0 1100.0) (float-vector 166.5 -283.0 1100.0) (float-vector -166.5 -283.0 1100.0)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 1110.0) (float-vector 166.5 283.0 1100.0) (float-vector -166.5 283.0 1100.0) (float-vector -166.5 283.0 1110.0)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 1110.0) (float-vector 166.5 -283.0 1100.0) (float-vector 166.5 283.0 1100.0) (float-vector 166.5 283.0 1110.0)))
         (instance face :init :vertices (list (float-vector -166.5 -283.0 1110.0) (float-vector -166.5 -283.0 1100.0) (float-vector 166.5 -283.0 1100.0) (float-vector 166.5 -283.0 1110.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 1110.0) (float-vector -166.5 283.0 1100.0) (float-vector -166.5 -283.0 1100.0) (float-vector -166.5 -283.0 1110.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 283.0 1340.0) (float-vector -166.5 283.0 1340.0) (float-vector -166.5 -283.0 1340.0) (float-vector 166.5 -283.0 1340.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 1330.0) (float-vector 166.5 283.0 1330.0) (float-vector 166.5 -283.0 1330.0) (float-vector -166.5 -283.0 1330.0)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 1340.0) (float-vector 166.5 283.0 1330.0) (float-vector -166.5 283.0 1330.0) (float-vector -166.5 283.0 1340.0)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 1340.0) (float-vector 166.5 -283.0 1330.0) (float-vector 166.5 283.0 1330.0) (float-vector 166.5 283.0 1340.0)))
         (instance face :init :vertices (list (float-vector -166.5 -283.0 1340.0) (float-vector -166.5 -283.0 1330.0) (float-vector 166.5 -283.0 1330.0) (float-vector 166.5 -283.0 1340.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 1340.0) (float-vector -166.5 283.0 1330.0) (float-vector -166.5 -283.0 1330.0) (float-vector -166.5 -283.0 1340.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 166.5 283.0 1570.0) (float-vector -166.5 283.0 1570.0) (float-vector -166.5 -283.0 1570.0) (float-vector 166.5 -283.0 1570.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 1560.0) (float-vector 166.5 283.0 1560.0) (float-vector 166.5 -283.0 1560.0) (float-vector -166.5 -283.0 1560.0)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 1570.0) (float-vector 166.5 283.0 1560.0) (float-vector -166.5 283.0 1560.0) (float-vector -166.5 283.0 1570.0)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 1570.0) (float-vector 166.5 -283.0 1560.0) (float-vector 166.5 283.0 1560.0) (float-vector 166.5 283.0 1570.0)))
         (instance face :init :vertices (list (float-vector -166.5 -283.0 1570.0) (float-vector -166.5 -283.0 1560.0) (float-vector 166.5 -283.0 1560.0) (float-vector 166.5 -283.0 1570.0)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 1570.0) (float-vector -166.5 283.0 1560.0) (float-vector -166.5 -283.0 1560.0) (float-vector -166.5 -283.0 1570.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :beige)
     (send (elt bc 1) :set-color :beige)
     (send (elt bc 2) :set-color :beige)
     (send (elt bc 3) :set-color :beige)
     (send (elt bc 4) :set-color :beige)
     (send (elt bc 5) :set-color :beige)
     (send (elt bc 6) :set-color :beige)
     (send (elt bc 7) :set-color :beige)
     (send (elt bc 8) :set-color :beige)
     (send (elt bc 9) :set-color :beige)
     (send (elt bc 10) :set-color :beige)
     (setq blink0 (instance bodyset-link :init (make-cascoords) :bodies bc :name :root :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of :room73b2-cupboard-right-bodyset2
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 5.5 684.5) (float-vector -5.0 5.5 684.5) (float-vector -5.0 -287.5 684.5) (float-vector 5.0 -287.5 684.5)))
         (instance face :init :vertices (list (float-vector -5.0 5.5 -0.5) (float-vector 5.0 5.5 -0.5) (float-vector 5.0 -287.5 -0.5) (float-vector -5.0 -287.5 -0.5)))
         (instance face :init :vertices (list (float-vector 5.0 5.5 684.5) (float-vector 5.0 5.5 -0.5) (float-vector -5.0 5.5 -0.5) (float-vector -5.0 5.5 684.5)))
         (instance face :init :vertices (list (float-vector 5.0 -287.5 684.5) (float-vector 5.0 -287.5 -0.5) (float-vector 5.0 5.5 -0.5) (float-vector 5.0 5.5 684.5)))
         (instance face :init :vertices (list (float-vector -5.0 -287.5 684.5) (float-vector -5.0 -287.5 -0.5) (float-vector 5.0 -287.5 -0.5) (float-vector 5.0 -287.5 684.5)))
         (instance face :init :vertices (list (float-vector -5.0 5.5 684.5) (float-vector -5.0 5.5 -0.5) (float-vector -5.0 -287.5 -0.5) (float-vector -5.0 -287.5 684.5)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :lightgoldenrodyellow)
     (setq blink5 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room73b2-cupboard-right-bodyset2 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of :room73b2-cupboard-right-bodyset3
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 287.5 684.5) (float-vector -5.0 287.5 684.5) (float-vector -5.0 -5.5 684.5) (float-vector 5.0 -5.5 684.5)))
         (instance face :init :vertices (list (float-vector -5.0 287.5 -0.5) (float-vector 5.0 287.5 -0.5) (float-vector 5.0 -5.5 -0.5) (float-vector -5.0 -5.5 -0.5)))
         (instance face :init :vertices (list (float-vector 5.0 287.5 684.5) (float-vector 5.0 287.5 -0.5) (float-vector -5.0 287.5 -0.5) (float-vector -5.0 287.5 684.5)))
         (instance face :init :vertices (list (float-vector 5.0 -5.5 684.5) (float-vector 5.0 -5.5 -0.5) (float-vector 5.0 287.5 -0.5) (float-vector 5.0 287.5 684.5)))
         (instance face :init :vertices (list (float-vector -5.0 -5.5 684.5) (float-vector -5.0 -5.5 -0.5) (float-vector 5.0 -5.5 -0.5) (float-vector 5.0 -5.5 684.5)))
         (instance face :init :vertices (list (float-vector -5.0 287.5 684.5) (float-vector -5.0 287.5 -0.5) (float-vector -5.0 -5.5 -0.5) (float-vector -5.0 -5.5 684.5)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :lightgoldenrodyellow)
     (setq blink4 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room73b2-cupboard-right-bodyset3 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of :room73b2-cupboard-right-bodyset4
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector -166.5 283.0 -52.5) (float-vector -166.5 -283.0 -52.5) (float-vector -166.5 -283.0 52.5) (float-vector -166.5 283.0 52.5)))
         (instance face :init :vertices (list (float-vector 166.5 -283.0 -52.5) (float-vector 166.5 -283.0 52.5) (float-vector -166.5 -283.0 52.5) (float-vector -166.5 -283.0 -52.5)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 -52.5) (float-vector 166.5 283.0 52.5) (float-vector 166.5 273.0 52.5) (float-vector 166.5 273.0 -42.5) (float-vector 166.5 -273.0 -42.5) (float-vector 166.5 -273.0 52.5) (float-vector 166.5 -283.0 52.5) (float-vector 166.5 -283.0 -52.5)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 52.5) (float-vector -166.5 -283.0 52.5) (float-vector 166.5 -283.0 52.5) (float-vector 166.5 -273.0 52.5) (float-vector -156.5 -273.0 52.5) (float-vector -156.5 273.0 52.5) (float-vector 166.5 273.0 52.5) (float-vector 166.5 283.0 52.5)))
         (instance face :init :vertices (list (float-vector 166.5 283.0 -52.5) (float-vector 166.5 -283.0 -52.5) (float-vector -166.5 -283.0 -52.5) (float-vector -166.5 283.0 -52.5)))
         (instance face :init :vertices (list (float-vector -166.5 283.0 -52.5) (float-vector -166.5 283.0 52.5) (float-vector 166.5 283.0 52.5) (float-vector 166.5 283.0 -52.5)))
         (instance face :init :vertices (list (float-vector -156.5 273.0 52.5) (float-vector -156.5 -273.0 52.5) (float-vector -156.5 -273.0 -42.5) (float-vector -156.5 273.0 -42.5)))
         (instance face :init :vertices (list (float-vector -156.5 -273.0 -42.5) (float-vector -156.5 -273.0 52.5) (float-vector 166.5 -273.0 52.5) (float-vector 166.5 -273.0 -42.5)))
         (instance face :init :vertices (list (float-vector 166.5 273.0 -42.5) (float-vector 166.5 273.0 52.5) (float-vector -156.5 273.0 52.5) (float-vector -156.5 273.0 -42.5)))
         (instance face :init :vertices (list (float-vector -156.5 273.0 -42.5) (float-vector -156.5 -273.0 -42.5) (float-vector 166.5 -273.0 -42.5) (float-vector 166.5 273.0 -42.5)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 176.0 293.0 62.5) (float-vector 166.0 293.0 62.5) (float-vector 166.0 -293.0 62.5) (float-vector 176.0 -293.0 62.5)))
         (instance face :init :vertices (list (float-vector 166.0 293.0 -52.5) (float-vector 176.0 293.0 -52.5) (float-vector 176.0 -293.0 -52.5) (float-vector 166.0 -293.0 -52.5)))
         (instance face :init :vertices (list (float-vector 176.0 293.0 62.5) (float-vector 176.0 293.0 -52.5) (float-vector 166.0 293.0 -52.5) (float-vector 166.0 293.0 62.5)))
         (instance face :init :vertices (list (float-vector 176.0 -293.0 62.5) (float-vector 176.0 -293.0 -52.5) (float-vector 176.0 293.0 -52.5) (float-vector 176.0 293.0 62.5)))
         (instance face :init :vertices (list (float-vector 166.0 -293.0 62.5) (float-vector 166.0 -293.0 -52.5) (float-vector 176.0 -293.0 -52.5) (float-vector 176.0 -293.0 62.5)))
         (instance face :init :vertices (list (float-vector 166.0 293.0 62.5) (float-vector 166.0 293.0 -52.5) (float-vector 166.0 -293.0 -52.5) (float-vector 166.0 -293.0 62.5)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :white)
     (send (elt bc 1) :set-color :lightgoldenrodyellow)
     (setq blink3 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room73b2-cupboard-right-bodyset4 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of :room73b2-cupboard-right-bodyset5
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 5.0 910.0) (float-vector -5.0 5.0 910.0) (float-vector -5.0 -55.0 910.0) (float-vector 5.0 -55.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 5.0 0.0) (float-vector 5.0 5.0 0.0) (float-vector 5.0 -55.0 0.0) (float-vector -5.0 -55.0 0.0)))
         (instance face :init :vertices (list (float-vector 5.0 5.0 910.0) (float-vector 5.0 5.0 0.0) (float-vector -5.0 5.0 0.0) (float-vector -5.0 5.0 910.0)))
         (instance face :init :vertices (list (float-vector 5.0 -55.0 910.0) (float-vector 5.0 -55.0 0.0) (float-vector 5.0 5.0 0.0) (float-vector 5.0 5.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -55.0 910.0) (float-vector -5.0 -55.0 0.0) (float-vector 5.0 -55.0 0.0) (float-vector 5.0 -55.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 5.0 910.0) (float-vector -5.0 5.0 0.0) (float-vector -5.0 -55.0 0.0) (float-vector -5.0 -55.0 910.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 -54.5 910.0) (float-vector -5.0 -54.5 910.0) (float-vector -5.0 -227.5 910.0) (float-vector 5.0 -227.5 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -54.5 0.0) (float-vector 5.0 -54.5 0.0) (float-vector 5.0 -227.5 0.0) (float-vector -5.0 -227.5 0.0)))
         (instance face :init :vertices (list (float-vector 5.0 -54.5 910.0) (float-vector 5.0 -54.5 0.0) (float-vector -5.0 -54.5 0.0) (float-vector -5.0 -54.5 910.0)))
         (instance face :init :vertices (list (float-vector 5.0 -227.5 910.0) (float-vector 5.0 -227.5 0.0) (float-vector 5.0 -54.5 0.0) (float-vector 5.0 -54.5 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -227.5 910.0) (float-vector -5.0 -227.5 0.0) (float-vector 5.0 -227.5 0.0) (float-vector 5.0 -227.5 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -54.5 910.0) (float-vector -5.0 -54.5 0.0) (float-vector -5.0 -227.5 0.0) (float-vector -5.0 -227.5 910.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 -227.0 910.0) (float-vector -5.0 -227.0 910.0) (float-vector -5.0 -287.0 910.0) (float-vector 5.0 -287.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -227.0 0.0) (float-vector 5.0 -227.0 0.0) (float-vector 5.0 -287.0 0.0) (float-vector -5.0 -287.0 0.0)))
         (instance face :init :vertices (list (float-vector 5.0 -227.0 910.0) (float-vector 5.0 -227.0 0.0) (float-vector -5.0 -227.0 0.0) (float-vector -5.0 -227.0 910.0)))
         (instance face :init :vertices (list (float-vector 5.0 -287.0 910.0) (float-vector 5.0 -287.0 0.0) (float-vector 5.0 -227.0 0.0) (float-vector 5.0 -227.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -287.0 910.0) (float-vector -5.0 -287.0 0.0) (float-vector 5.0 -287.0 0.0) (float-vector 5.0 -287.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -227.0 910.0) (float-vector -5.0 -227.0 0.0) (float-vector -5.0 -287.0 0.0) (float-vector -5.0 -287.0 910.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :lightgoldenrodyellow)
     (send (elt bc 1) :set-color :white)
     (gl::transparent (elt bc 1) 0.5)
     (send (elt bc 2) :set-color :lightgoldenrodyellow)
     (setq blink2 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room73b2-cupboard-right-bodyset5 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of :room73b2-cupboard-right-bodyset6
     (setq bc (list
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 287.0 910.0) (float-vector -5.0 287.0 910.0) (float-vector -5.0 227.0 910.0) (float-vector 5.0 227.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 287.0 0.0) (float-vector 5.0 287.0 0.0) (float-vector 5.0 227.0 0.0) (float-vector -5.0 227.0 0.0)))
         (instance face :init :vertices (list (float-vector 5.0 287.0 910.0) (float-vector 5.0 287.0 0.0) (float-vector -5.0 287.0 0.0) (float-vector -5.0 287.0 910.0)))
         (instance face :init :vertices (list (float-vector 5.0 227.0 910.0) (float-vector 5.0 227.0 0.0) (float-vector 5.0 287.0 0.0) (float-vector 5.0 287.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 227.0 910.0) (float-vector -5.0 227.0 0.0) (float-vector 5.0 227.0 0.0) (float-vector 5.0 227.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 287.0 910.0) (float-vector -5.0 287.0 0.0) (float-vector -5.0 227.0 0.0) (float-vector -5.0 227.0 910.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 227.5 910.0) (float-vector -5.0 227.5 910.0) (float-vector -5.0 54.5 910.0) (float-vector 5.0 54.5 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 227.5 0.0) (float-vector 5.0 227.5 0.0) (float-vector 5.0 54.5 0.0) (float-vector -5.0 54.5 0.0)))
         (instance face :init :vertices (list (float-vector 5.0 227.5 910.0) (float-vector 5.0 227.5 0.0) (float-vector -5.0 227.5 0.0) (float-vector -5.0 227.5 910.0)))
         (instance face :init :vertices (list (float-vector 5.0 54.5 910.0) (float-vector 5.0 54.5 0.0) (float-vector 5.0 227.5 0.0) (float-vector 5.0 227.5 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 54.5 910.0) (float-vector -5.0 54.5 0.0) (float-vector 5.0 54.5 0.0) (float-vector 5.0 54.5 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 227.5 910.0) (float-vector -5.0 227.5 0.0) (float-vector -5.0 54.5 0.0) (float-vector -5.0 54.5 910.0)))
       ))
       (instance faceset :init :faces (list
         (instance face :init :vertices (list (float-vector 5.0 55.0 910.0) (float-vector -5.0 55.0 910.0) (float-vector -5.0 -5.0 910.0) (float-vector 5.0 -5.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 55.0 0.0) (float-vector 5.0 55.0 0.0) (float-vector 5.0 -5.0 0.0) (float-vector -5.0 -5.0 0.0)))
         (instance face :init :vertices (list (float-vector 5.0 55.0 910.0) (float-vector 5.0 55.0 0.0) (float-vector -5.0 55.0 0.0) (float-vector -5.0 55.0 910.0)))
         (instance face :init :vertices (list (float-vector 5.0 -5.0 910.0) (float-vector 5.0 -5.0 0.0) (float-vector 5.0 55.0 0.0) (float-vector 5.0 55.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 -5.0 910.0) (float-vector -5.0 -5.0 0.0) (float-vector 5.0 -5.0 0.0) (float-vector 5.0 -5.0 910.0)))
         (instance face :init :vertices (list (float-vector -5.0 55.0 910.0) (float-vector -5.0 55.0 0.0) (float-vector -5.0 -5.0 0.0) (float-vector -5.0 -5.0 910.0)))
       ))
       ))
     (dolist (b (cdr bc)) (send (car bc) :assoc b))
     (send (elt bc 0) :set-color :lightgoldenrodyellow)
     (send (elt bc 1) :set-color :white)
     (gl::transparent (elt bc 1) 0.5)
     (send (elt bc 2) :set-color :lightgoldenrodyellow)
     (setq blink1 (instance bodyset-link :init (make-cascoords) :bodies bc :name :room73b2-cupboard-right-bodyset6 :weight 1 :centroid (float-vector 0.0 0.0 0.0) :inertia-tensor #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))

     ;; definition of assoc
     (send blink1 :newcoords (make-coords :pos (float-vector 171.0 -288.0 885.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
     (send blink0 :assoc blink1)
     (send blink2 :newcoords (make-coords :pos (float-vector 171.0 288.0 885.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
     (send blink0 :assoc blink2)
     (send blink3 :newcoords (make-coords :pos (float-vector 0.0 0.0 822.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
     (send blink0 :assoc blink3)
     (send blink4 :newcoords (make-coords :pos (float-vector 171.0 -288.0 80.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
     (send blink0 :assoc blink4)
     (send blink5 :newcoords (make-coords :pos (float-vector 171.0 288.0 80.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
     (send blink0 :assoc blink5)
     (send self :assoc blink0)

     ;; definition of end-coords

     ;; definition of joint

     ;; definition of :door-upper-right
     (setq joint0 (instance rotational-joint :init
			     :parent-link blink0 :child-link blink1 :name :door-upper-right :axis :-z
			     :min    0.0 :max   90.0 :max-joint-velocity 5 :max-joint-torque 100))

     ;; definition of :door-upper-left
     (setq joint1 (instance rotational-joint :init
			     :parent-link blink0 :child-link blink2 :name :door-upper-left :axis :z 
			     :min    0.0 :max   90.0 :max-joint-velocity 5 :max-joint-torque 100))

     ;; definition of :drawer
     (setq joint2 (instance linear-joint :init
			     :parent-link blink0 :child-link blink3 :name :drawer    :axis (float-vector 1.0 0.0 0.0)
			     :min    0.0 :max  353.0 :max-joint-velocity 0.785398 :max-joint-torque 100))

     ;; definition of :door-lower-right
     (setq joint3 (instance rotational-joint :init
			     :parent-link blink0 :child-link blink4 :name :door-lower-right :axis :-z
			     :min    0.0 :max   90.0 :max-joint-velocity 5 :max-joint-torque 100))

     ;; definition of :door-lower-left
     (setq joint4 (instance rotational-joint :init
			     :parent-link blink0 :child-link blink5 :name :door-lower-left :axis :z 
			     :min    0.0 :max   90.0 :max-joint-velocity 5 :max-joint-torque 100))


     ;; definition of :handle
     (setq handle0 (make-cascoords :pos (float-vector 0.0 0.0 0.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0)) :name "handle0"))
     (send blink0 :assoc handle0)

     ;; init-ending 
     (setq links (list blink0 blink1 blink2 blink3 blink4 blink5))
     (setq joint-list (list joint0 joint1 joint2 joint3 joint4))
     (send self :init-ending)
     (send self :move-to (make-coords :pos pos :rot rot))
     (send-all links :worldcoords)

     self))

  (:door-upper-right (&rest args) (forward-message-to joint0 args))
  (:door-upper-left (&rest args) (forward-message-to joint1 args))
  (:drawer (&rest args) (forward-message-to joint2 args))
  (:door-lower-right (&rest args) (forward-message-to joint3 args))
  (:door-lower-left (&rest args) (forward-message-to joint4 args))
  (:handle (&rest args) (forward-message-to-all (list  handle0) args))
  (:handle-handle0 (&rest args) (forward-message-to handle0 args))
  (:attention (&rest args) (forward-message-to-all (list ) args))
  (:button (&rest args) (forward-message-to-all (list ) args))
  )

(defun room73b2-cupboard-right (&rest args) (instance* room73b2-cupboard-right-object :init args))
;; (format *error-output* "(instance room73b2-cupboard-right-object :init) for generating model~%")
