(defforeign adr_to_string xview "_adr_to_string" () :integer)
(defforeign canvas_event xview "_canvas_event" () :integer)
(defforeign canvas_paint_window xview "_canvas_paint_window" () :integer)
(defforeign canvas_pixwin xview "_canvas_pixwin" () :integer)
(defforeign canvas_window_event xview "_canvas_window_event" () :integer)
(defforeign create_scrollbar xview "_create_scrollbar" () :integer)
(defforeign event_id xview "_event_id" () :integer)
(defforeign event_is_button xview "_event_is_button" () :integer)
(defforeign event_is_down xview "_event_is_down" () :integer)
(defforeign event_is_up xview "_event_is_up" () :integer)
(defforeign event_time xview "_event_time" () :integer)
(defforeign event_x xview "_event_x" () :integer)
(defforeign event_y xview "_event_y" () :integer)
(defforeign icon_create xview "_icon_create" () :integer)
(defforeign icon_load_mpr xview "_icon_load_mpr" () :integer)
(defforeign mem_create xview "_mem_create" () :integer)
(defforeign mem_point xview "_mem_point" () :integer)
(defforeign menu_create xview "_menu_create" () :integer)
(defforeign menu_create_item xview "_menu_create_item" () :integer)
(defforeign menu_find xview "_menu_find" () :integer)
(defforeign menu_get xview "_menu_get" () :integer)
(defforeign menu_item_set xview "_menu_item_set" () :integer)
(defforeign menu_set xview "_menu_set" () :integer)
(defforeign menu_show xview "_menu_show" () :integer)
(defforeign notify_do_dispatch xview "_notify_do_dispatch" () :integer)
(defforeign notify_no_dispatch xview "_notify_no_dispatch" () :integer)
(defforeign notify_set_itimer_func xview "_notify_set_itimer_func" () :integer)
(defforeign panel_button_image xview "_panel_button_image" () :integer)
(defforeign panel_create_button xview "_panel_create_button" () :integer)
(defforeign panel_create_choice xview "_panel_create_choice" () :integer)
(defforeign panel_create_cycle xview "_panel_create_cycle" () :integer)
(defforeign panel_create_message xview "_panel_create_message" () :integer)
(defforeign panel_create_slider xview "_panel_create_slider" () :integer)
(defforeign panel_create_text xview "_panel_create_text" () :integer)
(defforeign panel_create_toggle xview "_panel_create_toggle" () :integer)
(defforeign panel_default_handle_event xview "_panel_default_handle_event" () :integer)
(defforeign panel_event xview "_panel_event" () :integer)
(defforeign panel_get xview "_panel_get" () :integer)
(defforeign panel_get xview "_panel_get" () :integer)
(defforeign panel_set xview "_panel_set" () :integer)
(defforeign panel_update_scrolling_size xview "_panel_update_scrolling_size" () :integer)
(defforeign panel_window_event xview "_panel_window_event" () :integer)
(defforeign pf_open xview "_pf_open" () :integer)
(defforeign pr_rop xview "_pr_rop" () :integer)
(defforeign pr_text xview "_pr_text" () :integer)
(defforeign pr_vector xview "_pr_vector" () :integer)
(defforeign pw_batch_off xview "_pw_batch_off" () :integer)
(defforeign pw_copy xview "_pw_copy" () :integer)
(defforeign pw_getcmsname xview "_pw_getcmsname" () :integer)
(defforeign pw_getcolormap xview "_pw_getcolormap" () :integer)
(defforeign pw_polygon_2 xview "_pw_polygon_2" () :integer)
(defforeign pw_polyline xview "_pw_polyline" () :integer)
(defforeign pw_putcolormap xview "_pw_putcolormap" () :integer)
(defforeign pw_rop xview "_pw_rop" () :integer)
(defforeign pw_setcmsname xview "_pw_setcmsname" () :integer)
(defforeign pw_text xview "_pw_text" () :integer)
(defforeign pw_ttext xview "_pw_ttext" () :integer)
(defforeign pw_vector xview "_pw_vector" () :integer)
(defforeign pw_write xview "_pw_write" () :integer)
(defforeign pw_writebackground xview "_pw_writebackground" () :integer)
(defforeign textstring xview "_textstring" () :integer)
(defforeign textsw_delete xview "_textsw_delete" () :integer)
(defforeign textsw_erase xview "_textsw_erase" () :integer)
(defforeign textsw_insert xview "_textsw_insert" () :integer)
(defforeign textsw_replace_bytes xview "_textsw_replace_bytes" () :integer)
(defforeign textsw_save xview "_textsw_save" () :integer)
(defforeign textsw_screen_line_count xview "_textsw_screen_line_count" () :integer)
(defforeign textsw_scroll_lines xview "_textsw_scroll_lines" () :integer)
(defforeign textsw_store_file xview "_textsw_store_file" () :integer)
(defforeign ttysw_input xview "_ttysw_input" () :integer)
(defforeign ttysw_output xview "_ttysw_output" () :integer)
(defforeign window_create_canvas xview "_window_create_canvas" () :integer)
(defforeign window_create_frame xview "_window_create_frame" () :integer)
(defforeign window_create_panel xview "_window_create_panel" () :integer)
(defforeign window_create_textsw xview "_window_create_textsw" () :integer)
(defforeign window_create_tty xview "_window_create_tty" () :integer)
(defforeign window_destroy xview "_window_destroy" () :integer)
(defforeign window_fit_height xview "_window_fit_height" () :integer)
(defforeign window_fit_width xview "_window_fit_width" () :integer)
(defforeign window_get xview "_window_get" () :integer)
(defforeign window_set xview "_window_set" () :integer)
(defforeign xv_col xview "_xv_col" () :integer)
(defforeign xv_row xview "_xv_row" () :integer)
