;;
;; DO NOT EDIT THIS FILE
;; this file is automatically generated from euslisp+euslib;;
;;
(provide :eng2-7f-elevatorhall-scene "models/eng2-7f-elevatorhall-scene.l")

(load "models/eng2-7f-elevatorhall-2000x1800-elevator-call-panel-object.l")
(load "models/eng2-7f-elevatorhall-eng2-7f-elevatorhall-ground-object.l")
(load "models/eng2-7f-elevatorhall-wall61-object.l")
(load "models/eng2-7f-elevatorhall-wall59-object.l")
(load "models/eng2-7f-elevatorhall-wall57-object.l")
(load "models/eng2-7f-elevatorhall-wall55-object.l")
(load "models/eng2-7f-elevatorhall-wall53-object.l")
(load "models/eng2-7f-elevatorhall-wall51-object.l")
(load "models/eng2-7f-elevatorhall-wall49-object.l")
(load "models/eng2-7f-elevatorhall-wall47-object.l")
(load "models/eng2-7f-elevatorhall-wall45-object.l")
(load "models/eng2-7f-elevatorhall-wall43-object.l")
(load "models/eng2-7f-elevatorhall-wall41-object.l")
(load "models/eng2-7f-elevatorhall-wall39-object.l")
(load "models/eng2-7f-elevatorhall-wall37-object.l")
(load "models/eng2-7f-elevatorhall-wall35-object.l")
(load "models/eng2-7f-elevatorhall-wall33-object.l")
(load "models/eng2-7f-elevatorhall-wall31-object.l")
(load "models/eng2-7f-elevatorhall-wall29-object.l")
(load "models/eng2-7f-elevatorhall-wall27-object.l")
(load "models/eng2-7f-elevatorhall-wall25-object.l")
(load "models/eng2-7f-elevatorhall-wall23-object.l")
(load "models/eng2-7f-elevatorhall-wall21-object.l")
(load "models/eng2-7f-elevatorhall-wall19-object.l")
(load "models/eng2-7f-elevatorhall-wall17-object.l")
(load "models/eng2-7f-elevatorhall-wall15-object.l")
(load "models/eng2-7f-elevatorhall-wall13-object.l")
(load "models/eng2-7f-elevatorhall-wall11-object.l")
(load "models/eng2-7f-elevatorhall-wall9-object.l")
(load "models/eng2-7f-elevatorhall-wall7-object.l")
(load "models/eng2-7f-elevatorhall-wall5-object.l")

(defclass eng2-7f-elevatorhall-scene
  :super scene-model
  :slots ())
(defmethod eng2-7f-elevatorhall-scene
  (:init
   (&rest args &key (name "eng2-7f-elevatorhall") &allow-other-keys)
   (let ()
     (send-super*
      :init :name name :objects
      (list
 (send (eng2-7f-elevatorhall-2000x1800-elevator-call-panel) :transform (make-coords :pos (float-vector 3345.0 -2705.0 1159.5) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-eng2-7f-elevatorhall-ground) :transform (make-coords :pos (float-vector 3117.0 -80.0 0.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall61) :transform (make-coords :pos (float-vector 580.0 545.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall59) :transform (make-coords :pos (float-vector 1125.0 0.0 1375.0) :rot #2f((2.220446e-16 -1.0 0.0) (1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall57) :transform (make-coords :pos (float-vector 580.0 -545.0 1375.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall55) :transform (make-coords :pos (float-vector 500.0 -1130.0 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall53) :transform (make-coords :pos (float-vector 100.0 -4800.0 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall51) :transform (make-coords :pos (float-vector 1272.5 -4760.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall49) :transform (make-coords :pos (float-vector 2445.0 -5350.0 1375.0) :rot #2f((2.220446e-16 -1.0 0.0) (1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall47) :transform (make-coords :pos (float-vector 2912.5 -6005.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall45) :transform (make-coords :pos (float-vector 3380.0 -5260.0 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall43) :transform (make-coords :pos (float-vector 3650.75 -4529.52 1375.0) :rot #2f((-0.986394 0.164399 0.0) (-0.164399 -0.986394 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall41) :transform (make-coords :pos (float-vector 4190.0 -4480.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall39) :transform (make-coords :pos (float-vector 4190.0 -2910.0 1375.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall37) :transform (make-coords :pos (float-vector 3650.75 -2860.48 1375.0) :rot #2f((0.986394 0.164399 0.0) (-0.164399 0.986394 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall35) :transform (make-coords :pos (float-vector 3380.0 -2415.0 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall33) :transform (make-coords :pos (float-vector 3650.75 -1969.52 1375.0) :rot #2f((-0.986394 0.164399 0.0) (-0.164399 -0.986394 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall31) :transform (make-coords :pos (float-vector 4070.0 -1920.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall29) :transform (make-coords :pos (float-vector 4070.0 -1050.0 1375.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall27) :transform (make-coords :pos (float-vector 3650.75 -1000.48 1375.0) :rot #2f((0.986394 0.164399 0.0) (-0.164399 0.986394 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall25) :transform (make-coords :pos (float-vector 3380.0 -175.0 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall23) :transform (make-coords :pos (float-vector 3650.75 650.476 1375.0) :rot #2f((-0.986394 0.164399 0.0) (-0.164399 -0.986394 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall21) :transform (make-coords :pos (float-vector 4070.0 700.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall19) :transform (make-coords :pos (float-vector 4070.0 1570.0 1375.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall17) :transform (make-coords :pos (float-vector 3650.75 1619.52 1375.0) :rot #2f((0.986394 0.164399 0.0) (-0.164399 0.986394 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall15) :transform (make-coords :pos (float-vector 3380.0 2162.5 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall13) :transform (make-coords :pos (float-vector 3880.0 2655.0 1375.0) :rot #2f((-1.0 1.224647e-16 0.0) (-1.224647e-16 -1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall11) :transform (make-coords :pos (float-vector 4450.0 4262.5 1375.0) :rot #2f((2.220446e-16 1.0 0.0) (-1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall9) :transform (make-coords :pos (float-vector 2707.5 5870.0 1375.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall7) :transform (make-coords :pos (float-vector 965.0 5355.0 1375.0) :rot #2f((2.220446e-16 -1.0 0.0) (1.0 2.220446e-16 0.0) (0.0 0.0 1.0))))
 (send (eng2-7f-elevatorhall-wall5) :transform (make-coords :pos (float-vector 500.0 4910.0 1375.0) :rot #2f((1.0 0.0 0.0) (0.0 1.0 0.0) (0.0 0.0 1.0))))
       )
      args)
     ))
  )

(defun eng2-7f-elevatorhall (&rest args) (setq *eng2-7f-elevatorhall* (instance* eng2-7f-elevatorhall-scene :init args)))
